@@ -25,6 +25,7 @@ struct Shishi_ap
 {
   Shishi *handle;
   Shishi_tkt *tkt;
+  Shishi_key *key;
   Shishi_asn1 authenticator;
   Shishi_asn1 apreq;
   Shishi_asn1 aprep;
@@ -46,6 +47,39 @@ struct Shishi_ap
  **/
 int
 shishi_ap (Shishi * handle, Shishi_ap ** ap)
+{
+  int res;
+
+  res = shishi_ap_nosubkey (handle, ap);
+  if (res != SHISHI_OK)
+    {
+      shishi_error_printf (handle, "Could not create Authenticator: %s\n",
+			   shishi_strerror_details (handle));
+      return res;
+    }
+
+  res = shishi_authenticator_add_random_subkey (handle, (*ap)->authenticator);
+  if (res != SHISHI_OK)
+    {
+      shishi_error_printf (handle, "Could not add random subkey in AP: %s\n",
+			   shishi_strerror (res));
+      return res;
+    }
+
+  return SHISHI_OK;
+}
+
+/**
+ * shishi_ap_nosubkey:
+ * @handle: shishi handle as allocated by shishi_init().
+ * @ap: pointer to new structure that holds information about AP exchange
+ *
+ * Create a new AP exchange without subkey in authenticator.
+ *
+ * Return value: Returns SHISHI_OK iff successful.
+ **/
+int
+shishi_ap_nosubkey (Shishi * handle, Shishi_ap ** ap)
 {
   Shishi_ap *lap;
 
@@ -430,32 +464,6 @@ shishi_ap_req_set (Shishi_ap * ap, Shishi_asn1 apreq)
 /**
  * shishi_ap_req_der:
  * @ap: structure that holds information about AP exchange
- * @out: output array with der encoding of AP-REQ.
- * @outlen: length of output array with der encoding of AP-REQ.
- *
- * Build AP-REQ using shishi_ap_req_build() and DER encode it.
- *
- * Return value: Returns SHISHI_OK iff successful.
- **/
-int
-shishi_ap_req_der (Shishi_ap * ap, char *out, int *outlen)
-{
-  int rc;
-
-  rc = shishi_ap_req_build (ap);
-  if (rc != SHISHI_OK)
-    return rc;
-
-  rc = shishi_a2d (ap->handle, ap->apreq, out, outlen);
-  if (rc != SHISHI_OK)
-    return rc;
-
-  return SHISHI_OK;
-}
-
-/**
- * shishi_ap_req_der_new:
- * @ap: structure that holds information about AP exchange
  * @out: pointer to output array with der encoding of AP-REQ.
  * @outlen: pointer to length of output array with der encoding of AP-REQ.
  *
@@ -466,7 +474,7 @@ shishi_ap_req_der (Shishi_ap * ap, char *out, int *outlen)
  * Return value: Returns SHISHI_OK iff successful.
  **/
 int
-shishi_ap_req_der_new (Shishi_ap * ap, char **out, int *outlen)
+shishi_ap_req_der (Shishi_ap * ap, char **out, size_t *outlen)
 {
   int rc;
 
@@ -566,9 +574,13 @@ shishi_ap_req_build (Shishi_ap * ap)
 }
 
 /**
- * shishi_ap_req_process:
+ * shishi_ap_req_process_keyusage:
  * @ap: structure that holds information about AP exchange
  * @key: cryptographic key used to decrypt ticket in AP-REQ.
+ * @keyusage: key usage to use during decryption, for normal
+ *   AP-REQ's this is normally SHISHI_KEYUSAGE_APREQ_AUTHENTICATOR,
+ *   for AP-REQ's part of TGS-REQ's, this is normally
+ *   SHISHI_KEYUSAGE_TGSREQ_APREQ_AUTHENTICATOR.
  *
  * Decrypt ticket in AP-REQ using supplied key and decrypt
  * Authenticator in AP-REQ using key in decrypted ticket, and on
@@ -577,7 +589,9 @@ shishi_ap_req_build (Shishi_ap * ap)
  * Return value: Returns SHISHI_OK iff successful.
  **/
 int
-shishi_ap_req_process (Shishi_ap * ap, Shishi_key * key)
+shishi_ap_req_process_keyusage (Shishi_ap * ap,
+				Shishi_key * key,
+				int32_t keyusage)
 {
   Shishi_asn1 ticket, authenticator;
   Shishi_tkt *tkt;
@@ -596,6 +610,9 @@ shishi_ap_req_process (Shishi_ap * ap, Shishi_key * key)
       return rc;
     }
 
+  if (VERBOSEASN1 (ap->handle))
+    shishi_ticket_print (ap->handle, stdout, ticket);
+
   tkt = shishi_tkt2 (ap->handle, ticket, NULL, NULL);
 
   rc = shishi_tkt_decrypt (tkt, key);
@@ -619,8 +636,8 @@ shishi_ap_req_process (Shishi_ap * ap, Shishi_key * key)
     shishi_encticketpart_print (ap->handle, stdout,
 				shishi_tkt_encticketpart (tkt));
 
-  rc = shishi_apreq_decrypt (ap->handle, ap->apreq, tktkey, SHISHI_KEYUSAGE_APREQ_AUTHENTICATOR,	/* XXX */
-			     &authenticator);
+  rc = shishi_apreq_decrypt (ap->handle, ap->apreq, tktkey,
+			     keyusage, &authenticator);
   if (rc != SHISHI_OK)
     {
       shishi_error_printf (ap->handle, "Error decrypting apreq: %s\n",
@@ -629,7 +646,7 @@ shishi_ap_req_process (Shishi_ap * ap, Shishi_key * key)
     }
 
   if (VERBOSEASN1 (ap->handle))
-    shishi_authenticator_print (ap->handle, stdout, ap->authenticator);
+    shishi_authenticator_print (ap->handle, stdout, authenticator);
 
   ap->tkt = tkt;
   ap->authenticator = authenticator;
@@ -637,6 +654,24 @@ shishi_ap_req_process (Shishi_ap * ap, Shishi_key * key)
   return SHISHI_OK;
 }
 
+/**
+ * shishi_ap_req_process:
+ * @ap: structure that holds information about AP exchange
+ * @key: cryptographic key used to decrypt ticket in AP-REQ.
+ *
+ * Decrypt ticket in AP-REQ using supplied key and decrypt
+ * Authenticator in AP-REQ using key in decrypted ticket, and on
+ * success set the Ticket and Authenticator fields in the AP exchange.
+ *
+ * Return value: Returns SHISHI_OK iff successful.
+ **/
+int
+shishi_ap_req_process (Shishi_ap * ap, Shishi_key * key)
+{
+  return shishi_ap_req_process_keyusage (ap, key,
+					 SHISHI_KEYUSAGE_APREQ_AUTHENTICATOR);
+}
+
 /**
  * shishi_ap_req_asn1:
  * @ap: structure that holds information about AP exchange
@@ -660,6 +695,31 @@ shishi_ap_req_asn1 (Shishi_ap * ap, Shishi_asn1 * apreq)
   return SHISHI_OK;
 }
 
+/**
+ * shishi_ap_key:
+ * @ap: structure that holds information about AP exchange
+ *
+ * Extract the application key from AP.  If subkeys are used, it is
+ * taken from the Authenticator, otherwise the session key is used.
+ *
+ * Return value: Return application key from AP.
+ **/
+Shishi_key *
+shishi_ap_key (Shishi_ap * ap)
+{
+  int rc;
+
+  /* XXX do real check if subkey is present, don't just assume error
+     means no subkey */
+
+  rc = shishi_authenticator_get_subkey (ap->handle, ap->authenticator,
+					&ap->key);
+  if (rc != SHISHI_OK)
+    ap->key = shishi_tkt_key (ap->tkt);
+
+  return ap->key;
+}
+
 /**
  * shishi_ap_rep:
  * @ap: structure that holds information about AP exchange
@@ -691,15 +751,17 @@ shishi_ap_rep_set (Shishi_ap * ap, Shishi_asn1 aprep)
 /**
  * shishi_ap_rep_der:
  * @ap: structure that holds information about AP exchange
- * @out: output array with der encoding of AP-REP.
- * @outlen: length of output array with der encoding of AP-REP.
+ * @out: output array with newly allocated DER encoding of AP-REP.
+ * @outlen: length of output array with DER encoding of AP-REP.
  *
- * Build AP-REP using shishi_ap_rep_build() and DER encode it.
+ * Build AP-REQ using shishi_ap_rep_build() and DER encode it.  @out
+ * is allocated by this function, and it is the responsibility of
+ * caller to deallocate it.
  *
  * Return value: Returns SHISHI_OK iff successful.
  **/
 int
-shishi_ap_rep_der (Shishi_ap * ap, char *out, size_t * outlen)
+shishi_ap_rep_der (Shishi_ap * ap, char **out, size_t * outlen)
 {
   int rc;
 
@@ -707,7 +769,7 @@ shishi_ap_rep_der (Shishi_ap * ap, char *out, size_t * outlen)
   if (rc != SHISHI_OK)
     return rc;
 
-  rc = shishi_a2d (ap->handle, ap->aprep, out, outlen);
+  rc = shishi_new_a2d (ap->handle, ap->aprep, out, outlen);
   if (rc != SHISHI_OK)
     return rc;
 
