@@ -125,7 +125,7 @@ shishi_as_req_build (Shishi_as * as)
 
   res = shishi_kdcreq_build (as->handle, as->asreq);
   if (res != SHISHI_OK)
-      return res;
+    return res;
 
   return SHISHI_OK;
 }
@@ -135,7 +135,7 @@ shishi_as_req_build (Shishi_as * as)
  * @as: structure that holds information about AS exchange
  * @asreq: asreq to store in AS.
  *
- * Set the AS-REQ in the AP exchange.
+ * Set the AS-REQ in the AS exchange.
  **/
 void
 shishi_as_req_set (Shishi_as * as, Shishi_asn1 asreq)
@@ -148,19 +148,20 @@ shishi_as_req_set (Shishi_as * as, Shishi_asn1 asreq)
 /**
  * shishi_as_req_der:
  * @as: structure that holds information about AS exchange
- * @out: output array with der encoding of AS-REQ.
- * @outlen: length of output array with der encoding of AS-REQ.
+ * @out: output array with newly allocated DER encoding of AS-REQ.
+ * @outlen: length of output array with DER encoding of AS-REQ.
  *
- * DER encode AS-REQ.
+ * DER encode AS-REQ.  @out is allocated by this function, and it is
+ * the responsibility of caller to deallocate it.
  *
  * Return value: Returns SHISHI_OK iff successful.
  **/
 int
-shishi_as_req_der (Shishi_as * as, char *out, int *outlen)
+shishi_as_req_der (Shishi_as * as, char **out, size_t *outlen)
 {
   int rc;
 
-  rc = shishi_a2d (as->handle, as->asreq, out, outlen);
+  rc = shishi_new_a2d (as->handle, as->asreq, out, outlen);
   if (rc != SHISHI_OK)
     return rc;
 
@@ -363,19 +364,20 @@ shishi_as_rep_build (Shishi_as * as, Shishi_key * key)
 /**
  * shishi_as_rep_der:
  * @as: structure that holds information about AS exchange
- * @out: output array with der encoding of AS-REP.
- * @outlen: length of output array with der encoding of AS-REP.
+ * @out: output array with newly allocated DER encoding of AS-REP.
+ * @outlen: length of output array with DER encoding of AS-REP.
  *
- * DER encode AS-REP.
+ * DER encode AS-REP. @out is allocated by this function, and it is
+ * the responsibility of caller to deallocate it.
  *
  * Return value: Returns SHISHI_OK iff successful.
  **/
 int
-shishi_as_rep_der (Shishi_as * as, char *out, int *outlen)
+shishi_as_rep_der (Shishi_as * as, char **out, size_t *outlen)
 {
   int rc;
 
-  rc = shishi_a2d (as->handle, as->asrep, out, outlen);
+  rc = shishi_new_a2d (as->handle, as->asrep, out, outlen);
   if (rc != SHISHI_OK)
     return rc;
 
@@ -387,7 +389,7 @@ shishi_as_rep_der (Shishi_as * as, char *out, int *outlen)
  * @as: structure that holds information about AS exchange
  * @asrep: asrep to store in AS.
  *
- * Set the AS-REP in the AP exchange.
+ * Set the AS-REP in the AS exchange.
  **/
 void
 shishi_as_rep_set (Shishi_as * as, Shishi_asn1 asrep)
@@ -439,19 +441,20 @@ shishi_as_krberror (Shishi_as * as)
 /**
  * shishi_as_krberror_der:
  * @as: structure that holds information about AS exchange
- * @out: output array with der encoding of KRB-ERROR.
- * @outlen: length of output array with der encoding of KRB-ERROR.
+ * @out: output array with newly allocated DER encoding of KRB-ERROR.
+ * @outlen: length of output array with DER encoding of KRB-ERROR.
  *
- * DER encode KRB-ERROR.
+ * DER encode KRB-ERROR. @out is allocated by this function, and it is
+ * the responsibility of caller to deallocate it.
  *
  * Return value: Returns SHISHI_OK iff successful.
  **/
 int
-shishi_as_krberror_der (Shishi_as * as, char *out, int *outlen)
+shishi_as_krberror_der (Shishi_as * as, char **out, size_t *outlen)
 {
   int rc;
 
-  rc = shishi_a2d (as->handle, as->krberror, out, outlen);
+  rc = shishi_new_a2d (as->handle, as->krberror, out, outlen);
   if (rc != SHISHI_OK)
     return rc;
 
@@ -463,7 +466,7 @@ shishi_as_krberror_der (Shishi_as * as, char *out, int *outlen)
  * @as: structure that holds information about AS exchange
  * @krberror: krberror to store in AS.
  *
- * Set the KRB-ERROR in the AP exchange.
+ * Set the KRB-ERROR in the AS exchange.
  **/
 void
 shishi_as_krberror_set (Shishi_as * as, Shishi_asn1 krberror)
@@ -491,7 +494,7 @@ shishi_as_tkt (Shishi_as * as)
  * @as: structure that holds information about AS exchange
  * @tkt: tkt to store in AS.
  *
- * Set the Tkt in the AP exchange.
+ * Set the Tkt in the AS exchange.
  **/
 void
 shishi_as_tkt_set (Shishi_as * as, Shishi_tkt * tkt)
