@@ -60,6 +60,18 @@ static char *const _shishi_opts[] = {
 
 int getsubopt (char **optionp, char *const *tokens, char **valuep);
 
+struct Shishi_realminfo *
+shishi_realminfo (Shishi * handle, const char *realm)
+{
+  size_t i;
+
+  for (i = 0; i < handle->nrealminfos; i++)
+    if (strcmp (realm, handle->realminfos[i].name) == 0)
+      return &handle->realminfos[i];
+
+  return NULL;
+}
+
 /**
  * shishi_cfg:
  * @handle: Shishi library handle create by shishi_init().
@@ -75,7 +87,7 @@ shishi_cfg (Shishi * handle, char *option)
   char *value;
   char *realm = NULL;
   int res;
-  int i;
+  size_t i;
 
   while (option != NULL && *option != '\0')
     {
@@ -101,15 +113,15 @@ shishi_cfg (Shishi * handle, char *option)
 
 	case TICKET_LIFE_OPTION:
 	  {
-	    time_t now = time(NULL);
+	    time_t now = time (NULL);
 	    time_t then = get_date (value, &now);
-	    int diff = (int) difftime (then, now);
+	    int diff = difftime (then, now);
 
 	    if (value && then != -1 && diff > 0)
 	      handle->ticketlife = diff;
-	    else if (diff <= 0 && diff + 60*60*24 > 0)
+	    else if (diff <= 0 && diff + 60 * 60 * 24 > 0)
 	      /* Hack to support "17:00" as always meaning the next 17:00. */
-	      handle->ticketlife = 60*60*24 + diff;
+	      handle->ticketlife = 60 * 60 * 24 + diff;
 	    else if (diff <= 0)
 	      shishi_warn (handle, "Negative ticket life date: `%s'", value);
 	    else if (then == -1)
@@ -121,9 +133,9 @@ shishi_cfg (Shishi * handle, char *option)
 
 	case RENEW_LIFE_OPTION:
 	  {
-	    time_t now = time(NULL);
+	    time_t now = time (NULL);
 	    time_t then = get_date (value, &now);
-	    int diff = (int) difftime (then, now);
+	    int diff = difftime (then, now);
 
 	    if (value && then != -1 && diff > 0)
 	      handle->renewlife = diff;
@@ -161,36 +173,46 @@ shishi_cfg (Shishi * handle, char *option)
 	  handle->realminfos[handle->nrealminfos].nkdcaddresses = 0;
 	  handle->nrealminfos++;
 	  break;
+
 	case DEFAULT_REALM_OPTION:
 	  handle->default_realm = strdup (value);
 	  break;
+
 	case DEFAULT_PRINCIPAL_OPTION:
 	  handle->default_principal = strdup (value);
 	  break;
+
 	case CLIENT_KDC_ETYPES_OPTION:
 	  res = shishi_cfg_clientkdcetype_set (handle, value);
 	  if (res != SHISHI_OK)
 	    return res;
 	  break;
+
 	case KDC_OPTION:
 	  handle->kdc = strdup (value);
 	  break;
+
 	case STRINGPROCESS_OPTION:
 	  handle->stringprocess = strdup (value);
 	  break;
+
 	case VERBOSE_OPTION:
 	  handle->verbose = value && atoi (value) ? atoi (value) :
 	    ~0 & ~VERBOSES;
 	  break;
+
 	case VERBOSE_CRYPTO_OPTION:
 	  handle->verbose |= SHISHI_VERBOSE_CRYPTO;
 	  break;
+
 	case VERBOSE_ASN1_OPTION:
 	  handle->verbose |= SHISHI_VERBOSE_ASN1;
 	  break;
+
 	case VERBOSE_NOICE_OPTION:
 	  handle->verbose |= SHISHI_VERBOSE_NOICE;
 	  break;
+
 	case -1:
 	  if (!value)
 	    break;
@@ -201,6 +223,23 @@ shishi_cfg (Shishi * handle, char *option)
 		struct sockaddr_in *sinaddr;
 		struct hostent *he;
 		struct servent *se;
+		char *p;
+		int protocol = UDP;
+
+		if ((p = strchr (value, '/')))
+		  {
+		    *p = '\0';
+		    p++;
+		    if (strcmp(p, "udp") == 0)
+		      protocol = UDP;
+		    else if (strcmp(p, "tcp") == 0)
+		      protocol = TCP;
+		    else if (strcmp(p, "tls") == 0)
+		      protocol = TLS;
+		    else
+		      shishi_warn (handle,
+				   "Ignoring unknown KDC parameter: %s", p);
+		  }
 
 		he = gethostbyname (value);	/* XXX move to netio.c */
 		if (he == NULL ||
@@ -212,12 +251,11 @@ shishi_cfg (Shishi * handle, char *option)
 		    break;
 		  }
 
-		ri->kdcaddresses = realloc (ri->kdcaddresses,
-					    (ri->nkdcaddresses + 1) *
-					    sizeof (*ri->kdcaddresses));
-		if (ri->kdcaddresses == NULL)
-		  return SHISHI_MALLOC_ERROR;
+		ri->kdcaddresses = xrealloc (ri->kdcaddresses,
+					     (ri->nkdcaddresses + 1) *
+					     sizeof (*ri->kdcaddresses));
 		ri->kdcaddresses[ri->nkdcaddresses].name = strdup (value);
+		ri->kdcaddresses[ri->nkdcaddresses].protocol = protocol;
 		sinaddr = (struct sockaddr_in *)
 		  &ri->kdcaddresses[ri->nkdcaddresses].sockaddress;
 		memset (sinaddr, 0, sizeof (struct sockaddr));
@@ -312,8 +350,8 @@ shishi_cfg_from_file (Shishi * handle, const char *cfg)
 int
 shishi_cfg_print (Shishi * handle, FILE * fh)
 {
-  int i, j;
-  time_t tmp, now = time(NULL);
+  size_t i, j;
+  time_t tmp, now = time (NULL);
 
   fprintf (fh, "Shishi initial library configuration:\n");
   fprintf (fh, "\tDefault realm: %s\n",
@@ -328,10 +366,10 @@ shishi_cfg_print (Shishi * handle, FILE * fh)
   fprintf (fh, "\tVerbose: %d\n", handle->verbose);
   tmp = now + handle->ticketlife;
   fprintf (fh, "\tTicket life: %d seconds. %s",
-	   handle->ticketlife, ctime(&tmp));
+	   handle->ticketlife, ctime (&tmp));
   tmp = now + handle->renewlife;
   fprintf (fh, "\tRenew life: %d seconds. %s",
-	   handle->renewlife, ctime(&tmp));
+	   handle->renewlife, ctime (&tmp));
   for (i = 0; i < handle->nrealminfos; i++)
     {
       fprintf (fh, "\tRealm %s's KDCs:", handle->realminfos[i].name);
