@@ -54,9 +54,11 @@ raw_des_verify (Shishi * handle, int algo, char *out, int *outlen)
 
   if (VERBOSECRYPTO (handle))
     {
-      puts("DES verify:");
-      hexprint(md, hlen); puts("");
-      hexprint (p, hlen); puts("");
+      puts ("DES verify:");
+      hexprint (md, hlen);
+      puts ("");
+      hexprint (p, hlen);
+      puts ("");
     }
 
   ok = memcmp (p, md, hlen) == 0;
@@ -106,9 +108,11 @@ raw_des_verify (Shishi * handle, int algo, char *out, int *outlen)
 
   if (VERBOSECRYPTO (handle))
     {
-      puts("DES verify:");
-      hexprint(incoming, hlen); puts("");
-      hexprint (computed, hlen); puts("");
+      puts ("DES verify:");
+      hexprint (incoming, hlen);
+      puts ("");
+      hexprint (computed, hlen);
+      puts ("");
     }
 
   if (memcmp (computed, incoming, hlen) != 0)
@@ -128,8 +132,7 @@ static int
 raw_des_checksum (Shishi * handle,
 		  int algo,
 		  const char *in, size_t inlen,
-		  char *out, size_t * outlen,
-		  int hashzeros)
+		  char *out, size_t * outlen, int hashzeros)
 {
 #ifdef USE_GCRYPT
   gpg_error_t err;
@@ -166,8 +169,8 @@ raw_des_checksum (Shishi * handle,
       return SHISHI_CRYPTO_INTERNAL_ERROR;
     }
 
-  memcpy(out, buffer, 8);
-  memcpy(out + 8, p, hlen);
+  memcpy (out, buffer, 8);
+  memcpy (out + 8, p, hlen);
 
   gcry_md_close (hd);
 
@@ -228,10 +231,10 @@ static int
 des_encrypt_checksum (Shishi * handle,
 		      Shishi_key * key,
 		      int keyusage,
-		      const char *iv,
-		      size_t ivlen,
-		      const char *in, size_t inlen, char **out, size_t * outlen,
-		      int algo)
+		      const char *iv, size_t ivlen,
+		      char **ivout, size_t * ivoutlen,
+		      const char *in, size_t inlen,
+		      char **out, size_t * outlen, int algo)
 {
   char cksum[8 + MAX_HASH_LEN];
   char *inpad;
@@ -261,7 +264,7 @@ des_encrypt_checksum (Shishi * handle,
 
   free (inpad);
 
-  res = simplified_encrypt (handle, key, 0, iv, ivlen,
+  res = simplified_encrypt (handle, key, 0, iv, ivlen, ivout, ivoutlen,
 			    pt, ptlen, out, outlen);
 
   free (pt);
@@ -279,12 +282,13 @@ static int
 des_crc_encrypt (Shishi * handle,
 		 Shishi_key * key,
 		 int keyusage,
-		 const char *iv,
-		 size_t ivlen,
+		 const char *iv, size_t ivlen,
+		 char **ivout, size_t * ivoutlen,
 		 const char *in, size_t inlen, char **out, size_t * outlen)
 {
-  return des_encrypt_checksum (handle, key, keyusage, iv, ivlen,
-			       in, inlen, out, outlen, SHISHI_DES_CBC_CRC);
+  return des_encrypt_checksum (handle, key, keyusage, iv, ivlen, ivout,
+			       ivoutlen, in, inlen, out, outlen,
+			       SHISHI_DES_CBC_CRC);
 }
 
 static int
@@ -293,10 +297,12 @@ des_md4_encrypt (Shishi * handle,
 		 int keyusage,
 		 const char *iv,
 		 size_t ivlen,
+		 char **ivout, size_t * ivoutlen,
 		 const char *in, size_t inlen, char **out, size_t * outlen)
 {
-  return des_encrypt_checksum (handle, key, keyusage, iv, ivlen,
-			       in, inlen, out, outlen, SHISHI_DES_CBC_MD4);
+  return des_encrypt_checksum (handle, key, keyusage, iv, ivlen, ivout,
+			       ivoutlen, in, inlen, out, outlen,
+			       SHISHI_DES_CBC_MD4);
 }
 
 static int
@@ -305,10 +311,12 @@ des_md5_encrypt (Shishi * handle,
 		 int keyusage,
 		 const char *iv,
 		 size_t ivlen,
+		 char **ivout, size_t * ivoutlen,
 		 const char *in, size_t inlen, char **out, size_t * outlen)
 {
-  return des_encrypt_checksum (handle, key, keyusage, iv, ivlen,
-			       in, inlen, out, outlen, SHISHI_DES_CBC_MD5);
+  return des_encrypt_checksum (handle, key, keyusage, iv, ivlen, ivout,
+			       ivoutlen, in, inlen, out, outlen,
+			       SHISHI_DES_CBC_MD5);
 }
 
 static int
@@ -317,9 +325,10 @@ des_none_encrypt (Shishi * handle,
 		  int keyusage,
 		  const char *iv,
 		  size_t ivlen,
+		  char **ivout, size_t * ivoutlen,
 		  const char *in, size_t inlen, char **out, size_t * outlen)
 {
-  return simplified_encrypt (handle, key, 0, iv, ivlen,
+  return simplified_encrypt (handle, key, 0, iv, ivlen, ivout, ivoutlen,
 			     in, inlen, out, outlen);
 }
 
@@ -328,13 +337,13 @@ des_decrypt_verify (Shishi * handle,
 		    Shishi_key * key,
 		    int keyusage,
 		    const char *iv, size_t ivlen,
+		    char **ivout, size_t * ivoutlen,
 		    const char *in, size_t inlen,
-		    char **out, size_t * outlen,
-		    int algo)
+		    char **out, size_t * outlen, int algo)
 {
   int res;
 
-  res = simplified_decrypt (handle, key, 0, iv, ivlen,
+  res = simplified_decrypt (handle, key, 0, iv, ivlen, ivout, ivoutlen,
 			    in, inlen, out, outlen);
   if (res != SHISHI_OK)
     {
@@ -358,10 +367,12 @@ des_crc_decrypt (Shishi * handle,
 		 int keyusage,
 		 const char *iv,
 		 size_t ivlen,
+		 char **ivout, size_t * ivoutlen,
 		 const char *in, size_t inlen, char **out, size_t * outlen)
 {
-  return des_decrypt_verify (handle, key, keyusage, iv, ivlen,
-			     in, inlen, out, outlen, SHISHI_DES_CBC_CRC);
+  return des_decrypt_verify (handle, key, keyusage, iv, ivlen, ivout,
+			     ivoutlen, in, inlen, out, outlen,
+			     SHISHI_DES_CBC_CRC);
 }
 
 static int
@@ -370,10 +381,12 @@ des_md4_decrypt (Shishi * handle,
 		 int keyusage,
 		 const char *iv,
 		 size_t ivlen,
+		 char **ivout, size_t * ivoutlen,
 		 const char *in, size_t inlen, char **out, size_t * outlen)
 {
-  return des_decrypt_verify (handle, key, keyusage, iv, ivlen,
-			     in, inlen, out, outlen, SHISHI_DES_CBC_MD4);
+  return des_decrypt_verify (handle, key, keyusage, iv, ivlen, ivout,
+			     ivoutlen, in, inlen, out, outlen,
+			     SHISHI_DES_CBC_MD4);
 }
 
 static int
@@ -382,10 +395,12 @@ des_md5_decrypt (Shishi * handle,
 		 int keyusage,
 		 const char *iv,
 		 size_t ivlen,
+		 char **ivout, size_t * ivoutlen,
 		 const char *in, size_t inlen, char **out, size_t * outlen)
 {
-  return des_decrypt_verify (handle, key, keyusage, iv, ivlen,
-			     in, inlen, out, outlen, SHISHI_DES_CBC_MD5);
+  return des_decrypt_verify (handle, key, keyusage, iv, ivlen, ivout,
+			     ivoutlen, in, inlen, out, outlen,
+			     SHISHI_DES_CBC_MD5);
 }
 
 static int
@@ -394,9 +409,10 @@ des_none_decrypt (Shishi * handle,
 		  int keyusage,
 		  const char *iv,
 		  size_t ivlen,
+		  char **ivout, size_t * ivoutlen,
 		  const char *in, size_t inlen, char **out, size_t * outlen)
 {
-  return simplified_decrypt (handle, key, 0, iv, ivlen,
+  return simplified_decrypt (handle, key, 0, iv, ivlen, ivout, ivoutlen,
 			     in, inlen, out, outlen);
 }
 
@@ -541,7 +557,7 @@ des_cbc_check (Shishi * handle, char key[8], char *data, int n_data)
 
   return SHISHI_OK;
 
- done:
+done:
   gcry_cipher_close (ch);
   return res;
 #else
@@ -765,8 +781,7 @@ des_checksum (Shishi * handle,
 	      Shishi_key * key,
 	      int keyusage,
 	      int cksumtype,
-	      char *in, size_t inlen, char **out, size_t * outlen,
-	      int algo)
+	      char *in, size_t inlen, char **out, size_t * outlen, int algo)
 {
   char cksum[8 + MAX_HASH_LEN];
   size_t cksumlen;
@@ -787,8 +802,8 @@ des_checksum (Shishi * handle,
   for (i = 0; i < 8; i++)
     keyp[i] ^= 0xF0;
 
-  res = simplified_dencrypt (handle, key, NULL, 0, cksum, cksumlen,
-			     out, outlen, 0);
+  res = simplified_dencrypt (handle, key, NULL, 0, NULL, NULL,
+			     cksum, cksumlen, out, outlen, 0);
 
   for (i = 0; i < 8; i++)
     keyp[i] ^= 0xF0;
@@ -810,8 +825,7 @@ des_md4_checksum (Shishi * handle,
 		  char *in, size_t inlen, char **out, size_t * outlen)
 {
   return des_checksum (handle, key, keyusage, cksumtype,
-		       in, inlen, out, outlen,
-		       SHISHI_DES_CBC_MD4);
+		       in, inlen, out, outlen, SHISHI_DES_CBC_MD4);
 }
 
 static int
@@ -822,8 +836,7 @@ des_md5_checksum (Shishi * handle,
 		  char *in, size_t inlen, char **out, size_t * outlen)
 {
   return des_checksum (handle, key, keyusage, cksumtype,
-		       in, inlen, out, outlen,
-		       SHISHI_DES_CBC_MD5);
+		       in, inlen, out, outlen, SHISHI_DES_CBC_MD5);
 }
 
 static int
@@ -831,8 +844,7 @@ gss_des_checksum (Shishi * handle,
 		  Shishi_key * key,
 		  int keyusage,
 		  int cksumtype,
-		  char *in, size_t inlen,
-		  char **out, size_t * outlen)
+		  char *in, size_t inlen, char **out, size_t * outlen)
 {
 #ifdef USE_GCRYPT
   char *keyp;
@@ -881,7 +893,7 @@ gss_des_checksum (Shishi * handle,
 
   md5_init (&md5);
   md5_update (&md5, inlen, in);
-  md5_digest (&md5, sizeof(digest), digest);
+  md5_digest (&md5, sizeof (digest), digest);
 
   rc = des_set_key (&des.ctx, shishi_key_value (key));
   if (!rc)
@@ -890,7 +902,7 @@ gss_des_checksum (Shishi * handle,
       return SHISHI_CRYPTO_INTERNAL_ERROR;
     }
 
-  memset(des.iv, 0, sizeof(des.iv));
+  memset (des.iv, 0, sizeof (des.iv));
 
   *outlen = DES_BLOCK_SIZE;
   *out = xmalloc (*outlen);
