@@ -371,6 +371,7 @@ static int
 simplified_dencrypt (Shishi * handle,
 		     Shishi_key * key,
 		     const char *iv, size_t ivlen,
+		     char **ivout, size_t * ivoutlen,
 		     const char *in, size_t inlen,
 		     char **out, size_t * outlen, int decryptp)
 {
@@ -442,12 +443,24 @@ simplified_dencrypt (Shishi * handle,
       return SHISHI_CRYPTO_INTERNAL_ERROR;
     }
 
-  gcry_cipher_close (ch);
+  if (ivout && ivoutlen)
+    {
+      *ivoutlen = gcry_cipher_get_algo_blklen (alg);
+      *ivout = xmalloc (*ivoutlen);
+      if (decryptp)
+	memcpy (*ivout, in + inlen - *ivoutlen, *ivoutlen);
+      else
+	/* XXX what is the output iv for CBC-CTS mode?
+	   but is this value useful at all for that mode anyway?
+	   Mostly it is DES apps that want the updated iv, so this is ok. */
+	memcpy (*ivout, *out + *outlen - *ivoutlen, *ivoutlen);
+    }
 
+  gcry_cipher_close (ch);
 #else
-  struct CBC_CTX(struct des_ctx, DES_BLOCK_SIZE) des;
-  struct CBC_CTX(struct des3_ctx, DES3_BLOCK_SIZE) des3;
-  struct CBC_CTS_CTX(struct aes_ctx, AES_BLOCK_SIZE) aes;
+  struct CBC_CTX (struct des_ctx, DES_BLOCK_SIZE) des;
+  struct CBC_CTX (struct des3_ctx, DES3_BLOCK_SIZE) des3;
+  struct CBC_CTS_CTX (struct aes_ctx, AES_BLOCK_SIZE) aes;
   int rc;
 
   *outlen = inlen;
@@ -464,14 +477,20 @@ simplified_dencrypt (Shishi * handle,
 	  shishi_error_printf (handle, "Nettle setkey failed");
 	  return SHISHI_CRYPTO_INTERNAL_ERROR;
 	}
-      if (iv)
-	CBC_SET_IV (&des, iv);
-      else
-	memset(des.iv, 0, sizeof(des.iv));
+      memset (des.iv, 0, sizeof (des.iv));
+      /* XXX Use CBC_SET_IV (&des, iv), but how with ivlen? */
+      memcpy (des.iv, iv, ivlen < sizeof (des.iv) ? ivlen : sizeof (des.iv));
       if (decryptp)
 	CBC_DECRYPT (&des, des_decrypt, inlen, *out, in);
       else
 	CBC_ENCRYPT (&des, des_encrypt, inlen, *out, in);
+      if (ivout && ivoutlen)
+	{
+	  *ivoutlen = sizeof (des.iv);
+	  *ivout = xmalloc (*ivoutlen);
+	  /* XXX see above */
+	  memcpy (*ivout, des.iv, *ivoutlen);
+	}
       break;
 
     case SHISHI_DES3_CBC_HMAC_SHA1_KD:
@@ -481,22 +500,28 @@ simplified_dencrypt (Shishi * handle,
 	  shishi_error_printf (handle, "Nettle setkey failed");
 	  return SHISHI_CRYPTO_INTERNAL_ERROR;
 	}
-      if (iv)
-	CBC_SET_IV (&des3, iv);
-      else
-	memset(des3.iv, 0, sizeof(des3.iv));
+      memset (des3.iv, 0, sizeof (des3.iv));
+      /* XXX Use CBC_SET_IV (&des, iv), but how with ivlen? */
+      memcpy (des3.iv, iv,
+	      ivlen < sizeof (des3.iv) ? ivlen : sizeof (des3.iv));
       if (decryptp)
 	CBC_DECRYPT (&des3, des3_decrypt, inlen, *out, in);
       else
 	CBC_ENCRYPT (&des3, des3_encrypt, inlen, *out, in);
+      if (ivout && ivoutlen)
+	{
+	  *ivoutlen = sizeof (des3.iv);
+	  *ivout = xmalloc (*ivoutlen);
+	  /* XXX see above */
+	  memcpy (*ivout, des3.iv, *ivoutlen);
+	}
       break;
 
     case SHISHI_AES128_CTS_HMAC_SHA1_96:
     case SHISHI_AES256_CTS_HMAC_SHA1_96:
-      if (iv)
-	CBC_CTS_SET_IV (&aes, iv);
-      else
-	memset(aes.iv, 0, sizeof(aes.iv));
+      memset (aes.iv, 0, sizeof (aes.iv));
+      /* XXX Use CBC_SET_IV (&des, iv), but how with ivlen? */
+      memcpy (aes.iv, iv, ivlen < sizeof (aes.iv) ? ivlen : sizeof (aes.iv));
       if (decryptp)
 	{
 	  aes_set_decrypt_key (&aes.ctx, shishi_key_length (key),
@@ -509,10 +534,17 @@ simplified_dencrypt (Shishi * handle,
 			       shishi_key_value (key));
 	  CBC_CTS_ENCRYPT (&aes, aes_encrypt, inlen, *out, in);
 	}
+      if (ivout && ivoutlen)
+	{
+	  *ivoutlen = sizeof (aes.iv);
+	  *ivout = xmalloc (*ivoutlen);
+	  /* XXX see above */
+	  memcpy (*ivout, aes.iv, *ivoutlen);
+	}
       break;
     }
-
 #endif
+
   return SHISHI_OK;
 }
 
@@ -521,6 +553,7 @@ simplified_encrypt (Shishi * handle,
 		    Shishi_key * key,
 		    int keyusage,
 		    const char *iv, size_t ivlen,
+		    char **ivout, size_t * ivoutlen,
 		    const char *in, size_t inlen, char **out, size_t * outlen)
 {
   int res;
@@ -555,8 +588,9 @@ simplified_encrypt (Shishi * handle,
       if (res != SHISHI_OK)
 	goto done;
 
-      res = simplified_dencrypt (handle, privacykey, iv, ivlen,
-				 pt, ptlen, &ct, &ctlen, 0);
+      res =
+	simplified_dencrypt (handle, privacykey, iv, ivlen, ivout, ivoutlen,
+			     pt, ptlen, &ct, &ctlen, 0);
       if (res != SHISHI_OK)
 	goto done;
 
@@ -591,7 +625,7 @@ simplified_encrypt (Shishi * handle,
     }
   else
     {
-      res = simplified_dencrypt (handle, key, iv, ivlen,
+      res = simplified_dencrypt (handle, key, iv, ivlen, ivout, ivoutlen,
 				 in, inlen, out, outlen, 0);
     }
 
@@ -603,6 +637,7 @@ simplified_decrypt (Shishi * handle,
 		    Shishi_key * key,
 		    int keyusage,
 		    const char *iv, size_t ivlen,
+		    char **ivout, size_t * ivoutlen,
 		    const char *in, size_t inlen, char **out, size_t * outlen)
 {
   int res;
@@ -611,15 +646,16 @@ simplified_decrypt (Shishi * handle,
     {
       Shishi_key *privacykey = NULL, *integritykey = NULL;
       int blen = shishi_cipher_blocksize (shishi_key_type (key));
-      size_t hlen = 20; /* XXX only works for SHA-1 */
+      size_t hlen = 20;		/* XXX only works for SHA-1 */
 
       res = simplified_derivekey (handle, key, keyusage,
 				  SHISHI_DERIVEKEYMODE_PRIVACY, &privacykey);
       if (res != SHISHI_OK)
 	goto done;
 
-      res = simplified_dencrypt (handle, privacykey, iv, ivlen,
-				 in, inlen - hlen, out, outlen, 1);
+      res =
+	simplified_dencrypt (handle, privacykey, iv, ivlen, ivout, ivoutlen,
+			     in, inlen - hlen, out, outlen, 1);
       if (res != SHISHI_OK)
 	goto done;
 
@@ -647,7 +683,7 @@ simplified_decrypt (Shishi * handle,
     }
   else
     {
-      res = simplified_dencrypt (handle, key, iv, ivlen,
+      res = simplified_dencrypt (handle, key, iv, ivlen, ivout, ivoutlen,
 				 in, inlen, out, outlen, 1);
     }
 
@@ -718,6 +754,7 @@ typedef int (*Shishi_encrypt_function) (Shishi * handle,
 					Shishi_key * key,
 					int keyusage,
 					const char *iv, size_t ivlen,
+					char **ivout, size_t * ivoutlen,
 					const char *in, size_t inlen,
 					char **out, size_t * outlen);
 
@@ -725,6 +762,7 @@ typedef int (*Shishi_decrypt_function) (Shishi * handle,
 					Shishi_key * key,
 					int keyusage,
 					const char *iv, size_t ivlen,
+					char **ivout, size_t * ivoutlen,
 					const char *in, size_t inlen,
 					char **out, size_t * outlen);
 
@@ -1193,7 +1231,7 @@ static checksuminfo *checksums[] = {
 
 /**
  * shishi_checksum_supported_p:
- * @type: encryption type, see Shishi_etype.
+ * @type: checksum type, see Shishi_cksumtype.
  *
  * Return value: Return 0 iff checksum is unsupported.
  **/
@@ -1211,7 +1249,7 @@ shishi_checksum_supported_p (int32_t type)
 
 /**
  * shishi_checksum_name:
- * @type: encryption type, see Shishi_etype.
+ * @type: checksum type, see Shishi_cksumtype.
  *
  * Return value: Return name of checksum type,
  * e.g. "hmac-sha1-96-aes256", as defined in the standards.
@@ -1234,9 +1272,9 @@ shishi_checksum_name (int32_t type)
 
 /**
  * shishi_checksum_cksumlen:
- * @type: encryption type, see Shishi_etype.
+ * @type: checksum type, see Shishi_cksumtype.
  *
- * Return value: Return length of checksum used for the encryption type,
+ * Return value: Return length of checksum used for the checksum type,
  * as defined in the standards.
  **/
 size_t
@@ -1255,7 +1293,8 @@ shishi_checksum_cksumlen (int32_t type)
  * shishi_checksum_parse:
  * @checksum: name of checksum type, e.g. "hmac-sha1-96-aes256".
  *
- * Return value: Return checksum type corresponding to a string.
+ * Return value: Return checksum type, see Shishi_cksumtype,
+ * corresponding to a string.
  **/
 int
 shishi_checksum_parse (const char *checksum)
@@ -1463,7 +1502,8 @@ shishi_checksum (Shishi * handle,
       return SHISHI_CRYPTO_ERROR;
     }
 
-  res = (*checksum) (handle, key, keyusage, cksumtype, in, inlen, out, outlen);
+  res =
+    (*checksum) (handle, key, keyusage, cksumtype, in, inlen, out, outlen);
 
   if (VERBOSECRYPTO (handle))
     {
@@ -1477,34 +1517,46 @@ shishi_checksum (Shishi * handle,
 }
 
 /**
- * shishi_encrypt_iv_etype:
+ * shishi_encrypt_ivupdate_etype:
  * @handle: shishi handle as allocated by shishi_init().
  * @key: key to encrypt with.
  * @keyusage: integer specifying what this key is encrypting.
- * @etype: integer specifying what decryption method to use.
- * @iv: input array with initialization vector.
+ * @etype: integer specifying what cipher to use.
+ * @iv: input array with initialization vector
  * @ivlen: size of input array with initialization vector.
+ * @ivout: output array with newly allocated updated initialization vector.
+ * @ivoutlen: size of output array with updated initialization vector.
  * @in: input array with data to encrypt.
  * @inlen: size of input array with data to encrypt.
- * @out: output array with encrypted data.
- * @outlen: on input, holds maximum size of output array, on output,
- *          holds actual size of output array.
+ * @out: output array with newly allocated encrypted data.
+ * @outlen: output variable with size of newly allocated output array.
  *
- * Encrypts data using key, possibly altered by supplied key usage.
- * If key usage is 0, no key derivation is used.
+ * Encrypts data as per encryption method using specified
+ * initialization vector and key.  The key actually used is derived
+ * using the key usage.  If key usage is 0, no key derivation is used.
+ * The OUT buffer must be deallocated by the caller.  If IVOUT or
+ * IVOUTLEN is NULL, the updated IV is not saved anywhere.
  *
- * If OUT is NULL, this functions only set OUTLEN.  This usage may be
- * used by the caller to allocate the proper buffer size.
+ * Note that DECRYPT(ENCRYPT(data)) does not necessarily yield data
+ * exactly, some Kerberos encryption types add pad to make the data
+ * fit into the block size of the encryption algorithm.  Furthermore,
+ * the pad is not guaranteed to look in any special way, although
+ * existing implementations often pad with the zero byte.  This means
+ * that you may have to "frame" data, so it is possible to infer the
+ * original length after decryption.  Compare ASN.1 DER which contains
+ * such information.
  *
  * Return value: Returns %SHISHI_OK iff successful.
  **/
 int
-shishi_encrypt_iv_etype (Shishi * handle,
-			 Shishi_key * key,
-			 int keyusage,
-			 int32_t etype,
-			 char *iv, size_t ivlen,
-			 char *in, size_t inlen, char **out, size_t * outlen)
+shishi_encrypt_ivupdate_etype (Shishi * handle,
+			       Shishi_key * key,
+			       int keyusage,
+			       int32_t etype,
+			       const char *iv, size_t ivlen,
+			       char **ivout, size_t * ivoutlen,
+			       const char *in, size_t inlen,
+			       char **out, size_t * outlen)
 {
   Shishi_encrypt_function encrypt;
   int res;
@@ -1520,6 +1572,13 @@ shishi_encrypt_iv_etype (Shishi * handle,
       escapeprint (in, inlen);
       hexprint (in, inlen);
       puts ("");
+      if (iv)
+	{
+	  printf ("\t ;; iv (%d):\n", ivlen);
+	  escapeprint (iv, ivlen);
+	  hexprint (iv, ivlen);
+	  puts ("");
+	}
     }
 
   encrypt = _shishi_cipher_encrypt (etype);
@@ -1530,7 +1589,8 @@ shishi_encrypt_iv_etype (Shishi * handle,
       return SHISHI_CRYPTO_ERROR;
     }
 
-  res = (*encrypt) (handle, key, keyusage, iv, ivlen, in, inlen, out, outlen);
+  res = (*encrypt) (handle, key, keyusage,
+		    iv, ivlen, ivout, ivoutlen, in, inlen, out, outlen);
 
   if (VERBOSECRYPTO (handle))
     {
@@ -1538,27 +1598,177 @@ shishi_encrypt_iv_etype (Shishi * handle,
       escapeprint (*out, *outlen);
       hexprint (*out, *outlen);
       puts ("");
+      if (ivout && ivoutlen)
+	{
+	  printf ("\t ;; iv out:\n");
+	  escapeprint (*ivout, *ivoutlen);
+	  hexprint (*ivout, *ivoutlen);
+	  puts ("");
+	}
     }
 
   return res;
 }
 
 /**
- * shishi_encrypt:
+ * shishi_encrypt_iv_etype:
  * @handle: shishi handle as allocated by shishi_init().
  * @key: key to encrypt with.
  * @keyusage: integer specifying what this key is encrypting.
+ * @etype: integer specifying what cipher to use.
+ * @iv: input array with initialization vector
+ * @ivlen: size of input array with initialization vector.
  * @in: input array with data to encrypt.
  * @inlen: size of input array with data to encrypt.
- * @out: output array with encrypted data.
- * @outlen: on input, holds maximum size of output array, on output,
- *          holds actual size of output array.
+ * @out: output array with newly allocated encrypted data.
+ * @outlen: output variable with size of newly allocated output array.
  *
- * Encrypts data using key, possibly altered by supplied key usage.
- * If key usage is 0, no key derivation is used.
+ * Encrypts data as per encryption method using specified
+ * initialization vector and key.  The key actually used is derived
+ * using the key usage.  If key usage is 0, no key derivation is used.
+ * The OUT buffer must be deallocated by the caller.  The next IV is
+ * lost, see shishi_encrypt_ivupdate_etype if you need it.
  *
- * If OUT is NULL, this functions only set OUTLEN.  This usage may be
- * used by the caller to allocate the proper buffer size.
+ * Note that DECRYPT(ENCRYPT(data)) does not necessarily yield data
+ * exactly, some Kerberos encryption types add pad to make the data
+ * fit into the block size of the encryption algorithm.  Furthermore,
+ * the pad is not guaranteed to look in any special way, although
+ * existing implementations often pad with the zero byte.  This means
+ * that you may have to "frame" data, so it is possible to infer the
+ * original length after decryption.  Compare ASN.1 DER which contains
+ * such information.
+ *
+ * Return value: Returns %SHISHI_OK iff successful.
+ **/
+int
+shishi_encrypt_iv_etype (Shishi * handle,
+			 Shishi_key * key,
+			 int keyusage,
+			 int32_t etype,
+			 const char *iv, size_t ivlen,
+			 const char *in, size_t inlen,
+			 char **out, size_t * outlen)
+{
+  return shishi_encrypt_ivupdate_etype (handle, key, keyusage, etype,
+					iv, ivlen, NULL, NULL,
+					in, inlen, out, outlen);
+}
+
+/**
+ * shishi_encrypt_etype:
+ * @handle: shishi handle as allocated by shishi_init().
+ * @key: key to encrypt with.
+ * @keyusage: integer specifying what this key is encrypting.
+ * @etype: integer specifying what cipher to use.
+ * @in: input array with data to encrypt.
+ * @inlen: size of input array with data to encrypt.
+ * @out: output array with newly allocated encrypted data.
+ * @outlen: output variable with size of newly allocated output array.
+ *
+ * Encrypts data as per encryption method using specified
+ * initialization vector and key.  The key actually used is derived
+ * using the key usage.  If key usage is 0, no key derivation is used.
+ * The OUT buffer must be deallocated by the caller.  The default IV
+ * is used, see shishi_encrypt_iv_etype if you need to alter it. The
+ * next IV is lost, see shishi_encrypt_ivupdate_etype if you need it.
+ *
+ * Note that DECRYPT(ENCRYPT(data)) does not necessarily yield data
+ * exactly, some Kerberos encryption types add pad to make the data
+ * fit into the block size of the encryption algorithm.  Furthermore,
+ * the pad is not guaranteed to look in any special way, although
+ * existing implementations often pad with the zero byte.  This means
+ * that you may have to "frame" data, so it is possible to infer the
+ * original length after decryption.  Compare ASN.1 DER which contains
+ * such information.
+ *
+ * Return value: Returns %SHISHI_OK iff successful.
+ **/
+int
+shishi_encrypt_etype (Shishi * handle,
+		      Shishi_key * key,
+		      int keyusage,
+		      int32_t etype,
+		      const char *in, size_t inlen,
+		      char **out, size_t * outlen)
+{
+  return shishi_encrypt_ivupdate_etype (handle, key, keyusage,
+					shishi_key_type (key),
+					NULL, 0, NULL, NULL,
+					in, inlen, out, outlen);
+}
+
+/**
+ * shishi_encrypt_ivupdate:
+ * @handle: shishi handle as allocated by shishi_init().
+ * @key: key to encrypt with.
+ * @keyusage: integer specifying what this key is encrypting.
+ * @iv: input array with initialization vector
+ * @ivlen: size of input array with initialization vector.
+ * @ivout: output array with newly allocated updated initialization vector.
+ * @ivoutlen: size of output array with updated initialization vector.
+ * @in: input array with data to encrypt.
+ * @inlen: size of input array with data to encrypt.
+ * @out: output array with newly allocated encrypted data.
+ * @outlen: output variable with size of newly allocated output array.
+ *
+ * Encrypts data using specified initialization vector and key.  The
+ * key actually used is derived using the key usage.  If key usage is
+ * 0, no key derivation is used.  The OUT buffer must be deallocated
+ * by the caller.  If IVOUT or IVOUTLEN is NULL, the updated IV is not
+ * saved anywhere.
+ *
+ * Note that DECRYPT(ENCRYPT(data)) does not necessarily yield data
+ * exactly, some Kerberos encryption types add pad to make the data
+ * fit into the block size of the encryption algorithm.  Furthermore,
+ * the pad is not guaranteed to look in any special way, although
+ * existing implementations often pad with the zero byte.  This means
+ * that you may have to "frame" data, so it is possible to infer the
+ * original length after decryption.  Compare ASN.1 DER which contains
+ * such information.
+ *
+ * Return value: Returns %SHISHI_OK iff successful.
+ **/
+int
+shishi_encrypt_ivupdate (Shishi * handle,
+			 Shishi_key * key,
+			 int keyusage,
+			 const char *iv, size_t ivlen,
+			 char **ivout, size_t * ivoutlen,
+			 const char *in, size_t inlen,
+			 char **out, size_t * outlen)
+{
+  return shishi_encrypt_ivupdate_etype (handle, key, keyusage,
+					shishi_key_type (key),
+					iv, ivlen, ivout, ivoutlen,
+					in, inlen, out, outlen);
+}
+
+/**
+ * shishi_encrypt_iv:
+ * @handle: shishi handle as allocated by shishi_init().
+ * @key: key to encrypt with.
+ * @keyusage: integer specifying what this key is encrypting.
+ * @iv: input array with initialization vector
+ * @ivlen: size of input array with initialization vector.
+ * @in: input array with data to encrypt.
+ * @inlen: size of input array with data to encrypt.
+ * @out: output array with newly allocated encrypted data.
+ * @outlen: output variable with size of newly allocated output array.
+ *
+ * Encrypts data using specified initialization vector and key.  The
+ * key actually used is derived using the key usage.  If key usage is
+ * 0, no key derivation is used.  The OUT buffer must be deallocated
+ * by the caller.  The next IV is lost, see shishi_encrypt_ivupdate if
+ * you need it.
+ *
+ * Note that DECRYPT(ENCRYPT(data)) does not necessarily yield data
+ * exactly, some Kerberos encryption types add pad to make the data
+ * fit into the block size of the encryption algorithm.  Furthermore,
+ * the pad is not guaranteed to look in any special way, although
+ * existing implementations often pad with the zero byte.  This means
+ * that you may have to "frame" data, so it is possible to infer the
+ * original length after decryption.  Compare ASN.1 DER which contains
+ * such information.
  *
  * Return value: Returns %SHISHI_OK iff successful.
  **/
@@ -1566,12 +1776,13 @@ int
 shishi_encrypt_iv (Shishi * handle,
 		   Shishi_key * key,
 		   int keyusage,
-		   char *iv, size_t ivlen,
-		   char *in, size_t inlen, char **out, size_t * outlen)
+		   const char *iv, size_t ivlen,
+		   const char *in, size_t inlen, char **out, size_t * outlen)
 {
-  return shishi_encrypt_iv_etype (handle, key, keyusage,
-				  shishi_key_type (key), NULL, 0, in, inlen,
-				  out, outlen);
+  return shishi_encrypt_ivupdate_etype (handle, key, keyusage,
+					shishi_key_type (key),
+					iv, ivlen, NULL, NULL,
+					in, inlen, out, outlen);
 }
 
 /**
@@ -1581,15 +1792,23 @@ shishi_encrypt_iv (Shishi * handle,
  * @keyusage: integer specifying what this key is encrypting.
  * @in: input array with data to encrypt.
  * @inlen: size of input array with data to encrypt.
- * @out: output array with encrypted data.
- * @outlen: on input, holds maximum size of output array, on output,
- *          holds actual size of output array.
+ * @out: output array with newly allocated encrypted data.
+ * @outlen: output variable with size of newly allocated output array.
  *
- * Encrypts data using key, possibly altered by supplied key usage.
- * If key usage is 0, no key derivation is used.
+ * Encrypts data using specified key.  The key actually used is
+ * derived using the key usage.  If key usage is 0, no key derivation
+ * is used.  The OUT buffer must be deallocated by the caller.  The
+ * default IV is used, see shishi_encrypt_iv if you need to alter it.
+ * The next IV is lost, see shishi_encrypt_ivupdate if you need it.
  *
- * If OUT is NULL, this functions only set OUTLEN.  This usage may be
- * used by the caller to allocate the proper buffer size.
+ * Note that DECRYPT(ENCRYPT(data)) does not necessarily yield data
+ * exactly, some Kerberos encryption types add pad to make the data
+ * fit into the block size of the encryption algorithm.  Furthermore,
+ * the pad is not guaranteed to look in any special way, although
+ * existing implementations often pad with the zero byte.  This means
+ * that you may have to "frame" data, so it is possible to infer the
+ * original length after decryption.  Compare ASN.1 DER which contains
+ * such information.
  *
  * Return value: Returns %SHISHI_OK iff successful.
  **/
@@ -1599,39 +1818,53 @@ shishi_encrypt (Shishi * handle,
 		int keyusage,
 		char *in, size_t inlen, char **out, size_t * outlen)
 {
-  return shishi_encrypt_iv (handle, key, keyusage, NULL, 0,
-			    in, inlen, out, outlen);
+  return shishi_encrypt_ivupdate_etype (handle, key, keyusage,
+					shishi_key_type (key),
+					NULL, 0, NULL, NULL,
+					in, inlen, out, outlen);
 }
 
 /**
- * shishi_decrypt_iv_etype:
+ * shishi_decrypt_ivupdate_etype:
  * @handle: shishi handle as allocated by shishi_init().
  * @key: key to decrypt with.
  * @keyusage: integer specifying what this key is decrypting.
- * @etype: integer specifying what decryption method to use.
- * @iv: input array with initialization vector.
+ * @etype: integer specifying what cipher to use.
+ * @iv: input array with initialization vector
  * @ivlen: size of input array with initialization vector.
+ * @ivout: output array with newly allocated updated initialization vector.
+ * @ivoutlen: size of output array with updated initialization vector.
  * @in: input array with data to decrypt.
  * @inlen: size of input array with data to decrypt.
- * @out: output array with decrypted data.
- * @outlen: on input, holds maximum size of output array, on output,
- *          holds actual size of output array.
+ * @out: output array with newly allocated decrypted data.
+ * @outlen: output variable with size of newly allocated output array.
  *
- * Decrypts data using key, possibly altered by supplied key usage.
- * If key usage is 0, no key derivation is used.
+ * Decrypts data as per encryption method using specified
+ * initialization vector and key.  The key actually used is derived
+ * using the key usage.  If key usage is 0, no key derivation is used.
+ * The OUT buffer must be deallocated by the caller.  If IVOUT or
+ * IVOUTLEN is NULL, the updated IV is not saved anywhere.
  *
- * If OUT is NULL, this functions only set OUTLEN.  This usage may be
- * used by the caller to allocate the proper buffer size.
+ * Note that DECRYPT(ENCRYPT(data)) does not necessarily yield data
+ * exactly, some Kerberos encryption types add pad to make the data
+ * fit into the block size of the encryption algorithm.  Furthermore,
+ * the pad is not guaranteed to look in any special way, although
+ * existing implementations often pad with the zero byte.  This means
+ * that you may have to "frame" data, so it is possible to infer the
+ * original length after decryption.  Compare ASN.1 DER which contains
+ * such information.
  *
  * Return value: Returns %SHISHI_OK iff successful.
  **/
 int
-shishi_decrypt_iv_etype (Shishi * handle,
-			 Shishi_key * key,
-			 int keyusage,
-			 int32_t etype,
-			 char *iv, size_t ivlen,
-			 char *in, size_t inlen, char **out, size_t * outlen)
+shishi_decrypt_ivupdate_etype (Shishi * handle,
+			       Shishi_key * key,
+			       int keyusage,
+			       int32_t etype,
+			       const char *iv, size_t ivlen,
+			       char **ivout, size_t * ivoutlen,
+			       const char *in, size_t inlen,
+			       char **out, size_t * outlen)
 {
   Shishi_decrypt_function decrypt;
   int res;
@@ -1657,7 +1890,8 @@ shishi_decrypt_iv_etype (Shishi * handle,
       return SHISHI_CRYPTO_ERROR;
     }
 
-  res = (*decrypt) (handle, key, keyusage, iv, ivlen, in, inlen, out, outlen);
+  res = (*decrypt) (handle, key, keyusage,
+		    iv, ivlen, ivout, ivoutlen, in, inlen, out, outlen);
 
   if (VERBOSECRYPTO (handle))
     {
@@ -1670,24 +1904,164 @@ shishi_decrypt_iv_etype (Shishi * handle,
   return res;
 }
 
+/**
+ * shishi_decrypt_iv_etype:
+ * @handle: shishi handle as allocated by shishi_init().
+ * @key: key to decrypt with.
+ * @keyusage: integer specifying what this key is decrypting.
+ * @etype: integer specifying what cipher to use.
+ * @iv: input array with initialization vector
+ * @ivlen: size of input array with initialization vector.
+ * @in: input array with data to decrypt.
+ * @inlen: size of input array with data to decrypt.
+ * @out: output array with newly allocated decrypted data.
+ * @outlen: output variable with size of newly allocated output array.
+ *
+ * Decrypts data as per encryption method using specified
+ * initialization vector and key.  The key actually used is derived
+ * using the key usage.  If key usage is 0, no key derivation is used.
+ * The OUT buffer must be deallocated by the caller.  The next IV is
+ * lost, see shishi_decrypt_ivupdate_etype if you need it.
+ *
+ * Note that DECRYPT(ENCRYPT(data)) does not necessarily yield data
+ * exactly, some Kerberos encryption types add pad to make the data
+ * fit into the block size of the encryption algorithm.  Furthermore,
+ * the pad is not guaranteed to look in any special way, although
+ * existing implementations often pad with the zero byte.  This means
+ * that you may have to "frame" data, so it is possible to infer the
+ * original length after decryption.  Compare ASN.1 DER which contains
+ * such information.
+ *
+ * Return value: Returns %SHISHI_OK iff successful.
+ **/
+int
+shishi_decrypt_iv_etype (Shishi * handle,
+			 Shishi_key * key,
+			 int keyusage,
+			 int32_t etype,
+			 const char *iv, size_t ivlen,
+			 const char *in, size_t inlen,
+			 char **out, size_t * outlen)
+{
+  return shishi_decrypt_ivupdate_etype (handle, key, keyusage, etype,
+					iv, ivlen, NULL, NULL,
+					in, inlen, out, outlen);
+}
+
+/**
+ * shishi_decrypt_etype:
+ * @handle: shishi handle as allocated by shishi_init().
+ * @key: key to decrypt with.
+ * @keyusage: integer specifying what this key is decrypting.
+ * @etype: integer specifying what cipher to use.
+ * @in: input array with data to decrypt.
+ * @inlen: size of input array with data to decrypt.
+ * @out: output array with newly allocated decrypted data.
+ * @outlen: output variable with size of newly allocated output array.
+ *
+ * Decrypts data as per encryption method using specified key.  The
+ * key actually used is derived using the key usage.  If key usage is
+ * 0, no key derivation is used.  The OUT buffer must be deallocated
+ * by the caller.  The default IV is used, see shishi_decrypt_iv_etype
+ * if you need to alter it.  The next IV is lost, see
+ * shishi_decrypt_ivupdate_etype if you need it.
+ *
+ * Note that DECRYPT(ENCRYPT(data)) does not necessarily yield data
+ * exactly, some Kerberos encryption types add pad to make the data
+ * fit into the block size of the encryption algorithm.  Furthermore,
+ * the pad is not guaranteed to look in any special way, although
+ * existing implementations often pad with the zero byte.  This means
+ * that you may have to "frame" data, so it is possible to infer the
+ * original length after decryption.  Compare ASN.1 DER which contains
+ * such information.
+ *
+ * Return value: Returns %SHISHI_OK iff successful.
+ **/
+int
+shishi_decrypt_etype (Shishi * handle,
+		      Shishi_key * key,
+		      int keyusage,
+		      int32_t etype,
+		      const char *in, size_t inlen,
+		      char **out, size_t * outlen)
+{
+  return shishi_decrypt_ivupdate_etype (handle, key, keyusage, etype,
+					NULL, 0, NULL, NULL,
+					in, inlen, out, outlen);
+}
+
+/**
+ * shishi_decrypt_ivupdate:
+ * @handle: shishi handle as allocated by shishi_init().
+ * @key: key to decrypt with.
+ * @keyusage: integer specifying what this key is decrypting.
+ * @iv: input array with initialization vector
+ * @ivlen: size of input array with initialization vector.
+ * @ivout: output array with newly allocated updated initialization vector.
+ * @ivoutlen: size of output array with updated initialization vector.
+ * @in: input array with data to decrypt.
+ * @inlen: size of input array with data to decrypt.
+ * @out: output array with newly allocated decrypted data.
+ * @outlen: output variable with size of newly allocated output array.
+ *
+ * Decrypts data using specified initialization vector and key.  The
+ * key actually used is derived using the key usage.  If key usage is
+ * 0, no key derivation is used.  The OUT buffer must be deallocated
+ * by the caller.  If IVOUT or IVOUTLEN is NULL, the updated IV is not
+ * saved anywhere.
+ *
+ * Note that DECRYPT(ENCRYPT(data)) does not necessarily yield data
+ * exactly, some Kerberos encryption types add pad to make the data
+ * fit into the block size of the encryption algorithm.  Furthermore,
+ * the pad is not guaranteed to look in any special way, although
+ * existing implementations often pad with the zero byte.  This means
+ * that you may have to "frame" data, so it is possible to infer the
+ * original length after decryption.  Compare ASN.1 DER which contains
+ * such information.
+ *
+ * Return value: Returns %SHISHI_OK iff successful.
+ **/
+int
+shishi_decrypt_ivupdate (Shishi * handle,
+			 Shishi_key * key,
+			 int keyusage,
+			 const char *iv, size_t ivlen,
+			 char **ivout, size_t * ivoutlen,
+			 const char *in, size_t inlen,
+			 char **out, size_t * outlen)
+{
+  return shishi_decrypt_ivupdate_etype (handle, key, keyusage,
+					shishi_key_type (key),
+					iv, ivlen, ivout, ivoutlen,
+					in, inlen, out, outlen);
+}
+
 /**
  * shishi_decrypt_iv:
  * @handle: shishi handle as allocated by shishi_init().
  * @key: key to decrypt with.
  * @keyusage: integer specifying what this key is decrypting.
- * @iv: input array with initialization vector.
+ * @iv: input array with initialization vector
  * @ivlen: size of input array with initialization vector.
  * @in: input array with data to decrypt.
  * @inlen: size of input array with data to decrypt.
- * @out: output array with decrypted data.
- * @outlen: on input, holds maximum size of output array, on output,
- *          holds actual size of output array.
+ * @out: output array with newly allocated decrypted data.
+ * @outlen: output variable with size of newly allocated output array.
  *
- * Decrypts data using key, possibly altered by supplied key usage.
- * If key usage is 0, no key derivation is used.
+ * Decrypts data using specified initialization vector and key.  The
+ * key actually used is derived using the key usage.  If key usage is
+ * 0, no key derivation is used.  The OUT buffer must be deallocated
+ * by the caller.  The next IV is lost, see
+ * shishi_decrypt_ivupdate_etype if you need it.
  *
- * If OUT is NULL, this functions only set OUTLEN.  This usage may be
- * used by the caller to allocate the proper buffer size.
+ * Note that DECRYPT(ENCRYPT(data)) does not necessarily yield data
+ * exactly, some Kerberos encryption types add pad to make the data
+ * fit into the block size of the encryption algorithm.  Furthermore,
+ * the pad is not guaranteed to look in any special way, although
+ * existing implementations often pad with the zero byte.  This means
+ * that you may have to "frame" data, so it is possible to infer the
+ * original length after decryption.  Compare ASN.1 DER which contains
+ * such information.
  *
  * Return value: Returns %SHISHI_OK iff successful.
  **/
@@ -1695,12 +2069,13 @@ int
 shishi_decrypt_iv (Shishi * handle,
 		   Shishi_key * key,
 		   int keyusage,
-		   char *iv, size_t ivlen,
-		   char *in, size_t inlen, char **out, size_t * outlen)
+		   const char *iv, size_t ivlen,
+		   const char *in, size_t inlen, char **out, size_t * outlen)
 {
-  return shishi_decrypt_iv_etype (handle, key, keyusage,
-				  shishi_key_type (key),
-				  iv, ivlen, in, inlen, out, outlen);
+  return shishi_decrypt_ivupdate_etype (handle, key, keyusage,
+					shishi_key_type (key),
+					iv, ivlen, NULL, NULL,
+					in, inlen, out, outlen);
 }
 
 /**
@@ -1710,15 +2085,23 @@ shishi_decrypt_iv (Shishi * handle,
  * @keyusage: integer specifying what this key is decrypting.
  * @in: input array with data to decrypt.
  * @inlen: size of input array with data to decrypt.
- * @out: output array with decrypted data.
- * @outlen: on input, holds maximum size of output array, on output,
- *          holds actual size of output array.
+ * @out: output array with newly allocated decrypted data.
+ * @outlen: output variable with size of newly allocated output array.
  *
- * Decrypts data using key, possibly altered by supplied key usage.
- * If key usage is 0, no key derivation is used.
+ * Decrypts data specified key.  The key actually used is derived
+ * using the key usage.  If key usage is 0, no key derivation is used.
+ * The OUT buffer must be deallocated by the caller.  The default IV
+ * is used, see shishi_decrypt_iv if you need to alter it.  The next
+ * IV is lost, see shishi_decrypt_ivupdate if you need it.
  *
- * If OUT is NULL, this functions only set OUTLEN.  This usage may be
- * used by the caller to allocate the proper buffer size.
+ * Note that DECRYPT(ENCRYPT(data)) does not necessarily yield data
+ * exactly, some Kerberos encryption types add pad to make the data
+ * fit into the block size of the encryption algorithm.  Furthermore,
+ * the pad is not guaranteed to look in any special way, although
+ * existing implementations often pad with the zero byte.  This means
+ * that you may have to "frame" data, so it is possible to infer the
+ * original length after decryption.  Compare ASN.1 DER which contains
+ * such information.
  *
  * Return value: Returns %SHISHI_OK iff successful.
  **/
@@ -1726,10 +2109,12 @@ int
 shishi_decrypt (Shishi * handle,
 		Shishi_key * key,
 		int keyusage,
-		char *in, size_t inlen, char **out, size_t * outlen)
+		const char *in, size_t inlen, char **out, size_t * outlen)
 {
-  return shishi_decrypt_iv (handle, key, keyusage, NULL, 0,
-			    in, inlen, out, outlen);
+  return shishi_decrypt_ivupdate_etype (handle, key, keyusage,
+					shishi_key_type (key),
+					NULL, 0, NULL, NULL,
+					in, inlen, out, outlen);
 }
 
 /**
@@ -1757,8 +2142,8 @@ shishi_randomize (Shishi * handle, char *data, size_t datalen)
   fd = open (device, O_RDONLY);
   if (fd < 0)
     {
-      shishi_error_printf(handle, "Could not open random device: %s\n",
-			  strerror (errno));
+      shishi_error_printf (handle, "Could not open random device: %s\n",
+			   strerror (errno));
       return SHISHI_FOPEN_ERROR;
     }
   else
