@@ -37,8 +37,8 @@ _shishi_print_armored_data (Shishi * handle,
 			    FILE * fh,
 			    Shishi_asn1 asn1, char *asn1type, char *headers)
 {
-  char der[BUFSIZ];
-  int derlen = BUFSIZ;
+  char *der;
+  size_t derlen;
   char b64der[BUFSIZ];
   int res;
   size_t i;
@@ -48,7 +48,7 @@ _shishi_print_armored_data (Shishi * handle,
 
   asn1_print_structure (fh, asn1, "", ASN1_PRINT_NAME_TYPE_VALUE);
 
-  res = shishi_a2d_field (handle, asn1, "", der, &derlen);
+  res = shishi_new_a2d (handle, asn1, &der, &derlen);
   if (res != ASN1_SUCCESS)
     {
       shishi_error_printf (handle, "Could not DER encode %s: %s\n",
@@ -56,7 +56,7 @@ _shishi_print_armored_data (Shishi * handle,
       return !SHISHI_OK;
     }
 
-  shishi_to_base64 (b64der, der, derlen, sizeof (b64der));
+  base64_to (b64der, der, derlen, sizeof (b64der));
 
   fprintf (fh, HEADERBEG "\n", asn1type);
 
@@ -84,18 +84,18 @@ int
 _shishi_save_data (Shishi * handle, FILE * fh, Shishi_asn1 asn1,
 		   char *asn1type)
 {
-  char der[BUFSIZ];
-  int derlen;
+  char *der;
+  size_t derlen;
   int res;
-  int i;
+  size_t i;
 
   derlen = sizeof (der);
-  res = shishi_a2d_field (handle, asn1, asn1type, der, &derlen);
+  res = shishi_a2d_new_field (handle, asn1, asn1type, &der, &derlen);
   if (res != SHISHI_OK)
     {
       shishi_error_printf (handle, "Could not DER encode %s: %s\n",
 			   asn1type, shishi_strerror (res));
-      return !SHISHI_OK;
+      return SHISHI_ASN1_ERROR;
     }
 
   i = fwrite (der, sizeof (der[0]), derlen, fh);
@@ -224,7 +224,7 @@ _shishi_ticket_input (Shishi * handle,
 	  return res;
 	}
 
-      derlen = shishi_from_base64 (&der[0], b64der);
+      derlen = base64_from (&der[0], b64der);
     }
   else
     {
@@ -279,7 +279,7 @@ _shishi_enckdcreppart_input (Shishi * handle,
 	  return res;
 	}
 
-      derlen = shishi_from_base64 (&der[0], b64der);
+      derlen = base64_from (&der[0], b64der);
     }
   else
     {
@@ -356,7 +356,7 @@ _shishi_kdcreq_input (Shishi * handle, FILE * fh, Shishi_asn1 * asreq,
 	  return res;
 	}
 
-      derlen = shishi_from_base64 (&der[0], b64der);
+      derlen = base64_from (&der[0], b64der);
     }
   else
     {
@@ -417,7 +417,7 @@ _shishi_kdcrep_input (Shishi * handle, FILE * fh, Shishi_asn1 * asrep,
 	  return res;
 	}
 
-      derlen = shishi_from_base64 (&der[0], b64der);
+      derlen = base64_from (&der[0], b64der);
     }
   else
     {
@@ -477,7 +477,7 @@ _shishi_apreq_input (Shishi * handle, FILE * fh, Shishi_asn1 * apreq,
 	  return res;
 	}
 
-      derlen = shishi_from_base64 (&der[0], b64der);
+      derlen = base64_from (&der[0], b64der);
     }
   else
     {
@@ -524,7 +524,7 @@ _shishi_aprep_input (Shishi * handle, FILE * fh, Shishi_asn1 * aprep,
 	  return res;
 	}
 
-      derlen = shishi_from_base64 (&der[0], b64der);
+      derlen = base64_from (&der[0], b64der);
     }
   else
     {
@@ -573,7 +573,7 @@ _shishi_encapreppart_input (Shishi * handle, FILE * fh,
 	  return res;
 	}
 
-      derlen = shishi_from_base64 (&der[0], b64der);
+      derlen = base64_from (&der[0], b64der);
     }
   else
     {
@@ -621,7 +621,7 @@ _shishi_authenticator_input (Shishi * handle,
 	  return res;
 	}
 
-      derlen = shishi_from_base64 (&der[0], b64der);
+      derlen = base64_from (&der[0], b64der);
     }
   else
     {
@@ -669,7 +669,7 @@ _shishi_krberror_input (Shishi * handle,
 	  return res;
 	}
 
-      derlen = shishi_from_base64 (&der[0], b64der);
+      derlen = base64_from (&der[0], b64der);
     }
   else
     {
@@ -716,7 +716,7 @@ _shishi_safe_input (Shishi * handle, FILE * fh, Shishi_asn1 * safe, int type)
 	  return res;
 	}
 
-      derlen = shishi_from_base64 (&der[0], b64der);
+      derlen = base64_from (&der[0], b64der);
     }
   else
     {
@@ -791,12 +791,12 @@ shishi_key_parse (Shishi * handle, FILE * fh, Shishi_key ** key)
 
       if (in_body)
 	{
-	  buflen = shishi_from_base64 (buffer, line);
+	  buflen = base64_from (buffer, line);
 	  shishi_key_value_set (lkey, buffer);
 	}
       else
 	{
-	  if (in_body == 0 && strcmp (line, "") == 0)
+	  if (strcmp (line, "") == 0 || strcmp (line, " ") == 0)
 	    in_body = 1;
 
 	  if (strncmp (line, "Keytype: ", strlen ("Keytype: ")) == 0)
@@ -857,7 +857,7 @@ shishi_key_print (Shishi * handle, FILE * fh, Shishi_key * key)
   char b64key[BUFSIZ];
   size_t i;
 
-  shishi_to_base64 (b64key, shishi_key_value (key),
+  base64_to (b64key, shishi_key_value (key),
 		    shishi_key_length (key), sizeof (b64key));
 
   fprintf (fh, HEADERBEG "\n", "KEY");
@@ -881,12 +881,14 @@ shishi_key_print (Shishi * handle, FILE * fh, Shishi_key * key)
   if ((i + 1) % 64 != 0)
     fprintf (fh, "\n");
 
+#if 0
   if (VERBOSENOICE (handle))
     {
       for (i = 0; i < shishi_key_length (key); i++)
 	fprintf (stdout, "%02x", shishi_key_value (key)[i] & 0xFF);
       fprintf (stdout, "\n");
     }
+#endif
 
   fprintf (fh, HEADEREND "\n", "KEY");
 
