@@ -241,7 +241,8 @@ shishi_enckdcreppart_populate_encticketpart (Shishi * handle,
   if (res != SHISHI_OK)
     return SHISHI_ASN1_ERROR;
 
-  res = shishi_asn1_read2 (handle, encticketpart, "renew-till", &buf, &buflen);
+  res =
+    shishi_asn1_read2 (handle, encticketpart, "renew-till", &buf, &buflen);
   if (res != SHISHI_OK && res != SHISHI_ASN1_NO_ELEMENT)
     return SHISHI_ASN1_ERROR;
 
