@@ -33,22 +33,6 @@
 #include <ctype.h>
 #endif
 
-#if HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#ifdef HAVE_NETDB_H
-#include <netdb.h>
-#endif
-
-#if defined HAVE_DECL_H_ERRNO && !HAVE_DECL_H_ERRNO
-//extern int h_errno;
-#endif
-
-#ifdef HAVE_PWD_H
-#include <pwd.h>
-#endif
-
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
@@ -69,6 +53,22 @@
 #include <sys/stat.h>
 #endif
 
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+#ifdef HAVE_NETDB_H
+#include <netdb.h>
+#endif
+
+#if defined HAVE_DECL_H_ERRNO && !HAVE_DECL_H_ERRNO
+//extern int h_errno;
+#endif
+
+#ifdef HAVE_PWD_H
+#include <pwd.h>
+#endif
+
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
@@ -77,14 +77,6 @@
 #include <errno.h>
 #endif
 
-#if HAVE_INTTYPES_H
-# include <inttypes.h>
-#else
-# if HAVE_STDINT_H
-#  include <stdint.h>
-# endif
-#endif
-
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
 # include <time.h>
@@ -127,11 +119,15 @@
 #include <resolv.h>
 #endif
 
+#include "gettext.h"
+
 #include "setenv.h"
 #include "vasprintf.h"
-#include "gettext.h"
 #include "xalloc.h"
+#include "xstrndup.h"
 #include "getdate.h"
+#include "base64.h"
+
 #include "shishi.h"
 
 #define _(String) dgettext (PACKAGE, String)
@@ -163,17 +159,24 @@
 #define HOST_NAME_MAX BUFSIZ
 #endif
 
+enum {
+  UDP,
+  TCP,
+  TLS
+};
+
 struct Shishi_kdcinfo
 {
   char *name;
   struct sockaddr sockaddress;
+  int protocol;
 };
 
 struct Shishi_realminfo
 {
   char *name;
   struct Shishi_kdcinfo *kdcaddresses;
-  int nkdcaddresses;
+  size_t nkdcaddresses;
 };
 
 struct Shishi
@@ -182,14 +185,14 @@ struct Shishi
   int verbose;
   char *default_realm;
   char *default_principal;
-  int kdctimeout;
-  int kdcretries;
+  size_t kdctimeout;
+  size_t kdcretries;
   int ticketlife;
   int renewlife;
   int32_t *clientkdcetypes;
-  int nclientkdcetypes;
+  size_t nclientkdcetypes;
   struct Shishi_realminfo *realminfos;
-  int nrealminfos;
+  size_t nrealminfos;
   char *kdc;
   char error[1024];
   char gztime_buf[40];
@@ -205,8 +208,8 @@ struct Shishi
 #define TICKET_FILE BASE_DIR "/tickets"
 #define USERCFG_FILE BASE_DIR "/shishi.conf"
 
-#define TICKETLIFE (60*60*8)   /* Work day */
-#define RENEWLIFE (60*60*24*7) /* Week */
+#define TICKETLIFE (60*60*8)	/* Work day */
+#define RENEWLIFE (60*60*24*7)	/* Week */
 
 int
 _shishi_print_armored_data (Shishi * handle,
@@ -244,6 +247,12 @@ shishi_asn1_integer2_field (Shishi * handle,
 			    Shishi_asn1 node, unsigned long *i,
 			    const char *field);
 
+extern time_t xtime (time_t * t);
+extern int xgettimeofday (struct timeval *tv, struct timezone *tz);
+
+struct Shishi_realminfo *
+shishi_realminfo (Shishi * handle, const char *realm);
+
 #if defined(WITH_DMALLOC) && WITH_DMALLOC
 #include <dmalloc.h>
 #endif
@@ -255,23 +264,23 @@ shishi_asn1_integer2_field (Shishi * handle,
 
 typedef struct dnshost_st
 {
-  struct dnshost_st   *next;
+  struct dnshost_st *next;
 
-  unsigned int        type;
-  unsigned int        class;
-  unsigned int        ttl;
+  unsigned int type;
+  unsigned int class;
+  unsigned int ttl;
 
-  void                *rr;
+  void *rr;
 } *dnshost_t;
 
 typedef struct dns_srv_st
 {
-  unsigned int        priority;
-  unsigned int        weight;
-  unsigned int        port;
-  unsigned int        rweight;
+  unsigned int priority;
+  unsigned int weight;
+  unsigned int port;
+  unsigned int rweight;
 
-  char                name[256];
+  char name[256];
 } *dns_srv_t;
 
 dnshost_t _shishi_resolv (const char *zone, unsigned int type);
