@@ -196,8 +196,7 @@ shishi_kdcreq_sendrecv (Shishi * handle, Shishi_asn1 kdcreq,
  **/
 int
 shishi_kdc_copy_crealm (Shishi * handle,
-			Shishi_asn1 kdcrep,
-			Shishi_asn1 encticketpart)
+			Shishi_asn1 kdcrep, Shishi_asn1 encticketpart)
 {
   char *buf;
   size_t buflen;
@@ -297,7 +296,7 @@ shishi_kdc_copy_cname (Shishi * handle,
   int i, n;
 
   res = shishi_asn1_read2 (handle, encticketpart,
-			  "cname.name-type", &buf, &buflen);
+			   "cname.name-type", &buf, &buflen);
   if (res != SHISHI_OK)
     return res;
 
@@ -378,13 +377,15 @@ shishi_as_check_cname (Shishi * handle, Shishi_asn1 asreq, Shishi_asn1 asrep)
   for (i = 1; i <= j; i++)
     {
       asprintf (&format, "req-body.cname.name-string.?%d", i);
-      res = shishi_asn1_read2 (handle, asreq, format, &reqcname, &reqcnamelen);
+      res =
+	shishi_asn1_read2 (handle, asreq, format, &reqcname, &reqcnamelen);
       free (format);
       if (res != SHISHI_OK)
 	return res;
 
       asprintf (&format, "cname.name-string.?%d", i);
-      res = shishi_asn1_read2 (handle, asrep, format, &repcname, &repcnamelen);
+      res =
+	shishi_asn1_read2 (handle, asrep, format, &repcname, &repcnamelen);
       free (format);
       if (res != SHISHI_OK)
 	return res;
@@ -558,6 +559,7 @@ shishi_kdc_check_nonce (Shishi * handle,
  * @handle: shishi handle as allocated by shishi_init().
  * @tgsreq: input variable that holds the sent KDC-REQ.
  * @tgsrep: input variable that holds the received KDC-REP.
+ * @authenticator: input variable with Authenticator from AP-REQ in KDC-REQ.
  * @oldenckdcreppart: input variable with EncKDCRepPart used in request.
  * @enckdcreppart: output variable that holds new EncKDCRepPart.
  *
@@ -573,9 +575,13 @@ int
 shishi_tgs_process (Shishi * handle,
 		    Shishi_asn1 tgsreq,
 		    Shishi_asn1 tgsrep,
-		    Shishi_asn1 oldenckdcreppart, Shishi_asn1 * enckdcreppart)
+		    Shishi_asn1 authenticator,
+		    Shishi_asn1 oldenckdcreppart,
+		    Shishi_asn1 * enckdcreppart)
 {
-  Shishi_key *key;
+  Shishi_key *tktkey;
+  Shishi_key *subkey;
+  int use_subkey;
   int etype;
   int res;
 
@@ -583,18 +589,40 @@ shishi_tgs_process (Shishi * handle,
   if (res != SHISHI_OK)
     return res;
 
-  res = shishi_enckdcreppart_get_key (handle, oldenckdcreppart, &key);
+  res = shishi_authenticator_get_subkey (handle, authenticator, &subkey);
+  use_subkey = (res != SHISHI_ASN1_NO_ELEMENT);
+  if (res != SHISHI_OK && res != SHISHI_ASN1_NO_ELEMENT)
+    return res;
+
+  res = shishi_enckdcreppart_get_key (handle, oldenckdcreppart, &tktkey);
   if (res != SHISHI_OK)
     return res;
 
-  if (etype != shishi_key_type (key))
+  if (etype != shishi_key_type (use_subkey ? subkey : tktkey))
     return SHISHI_TGSREP_BAD_KEYTYPE;
 
-  res = shishi_kdc_process (handle, tgsreq, tgsrep, key,
+  res = shishi_kdc_process (handle, tgsreq, tgsrep,
+			    use_subkey ? subkey : tktkey,
+			    use_subkey ?
+			    SHISHI_KEYUSAGE_ENCTGSREPPART_AUTHENTICATOR_KEY :
 			    SHISHI_KEYUSAGE_ENCTGSREPPART_SESSION_KEY,
 			    enckdcreppart);
+  if (res == SHISHI_CRYPTO_ERROR && use_subkey)
+    {
+      int tmpres;
 
-  return res;
+      tmpres = shishi_kdc_process (handle, tgsreq, tgsrep, tktkey,
+				   SHISHI_KEYUSAGE_ENCTGSREPPART_SESSION_KEY,
+				   enckdcreppart);
+      if (tmpres != SHISHI_OK)
+	return res;
+
+      shishi_warn (handle, "KDC bug: Reply encrypted using wrong key.");
+    }
+  else if (res != SHISHI_OK)
+    return res;
+
+  return SHISHI_OK;
 }
 
 /**
