@@ -128,7 +128,7 @@ shishi_parse_name (Shishi * handle, const char *name,
 	*realm = xstrdup (p);
     }
   else if (realm)
-    *realm = strdup(shishi_realm_default (handle));
+    *realm = strdup (shishi_realm_default (handle));
 
   return SHISHI_OK;
 }
@@ -185,13 +185,16 @@ shishi_principal_name_get (Shishi * handle,
 			   const char *namefield, char *out, size_t * outlen)
 {
   int res;
-  char format[BUFSIZ];
+  char *format;
   size_t totlen = 0;
   int len;
   int i, j, n;
 
-  sprintf (format, "%s.name-string", namefield);
+  /* FIXME: allocate output instead of writing inline */
+
+  asprintf (&format, "%s.name-string", namefield);
   res = shishi_asn1_number_of_elements (handle, namenode, format, &n);
+  free (format);
   if (res != SHISHI_OK)
     return res;
 
@@ -199,8 +202,9 @@ shishi_principal_name_get (Shishi * handle,
   for (i = 1; i <= n; i++)
     {
       len = *outlen - totlen;
-      sprintf (format, "%s.name-string.?%d", namefield, i);
+      asprintf (&format, "%s.name-string.?%d", namefield, i);
       res = shishi_asn1_read (handle, namenode, format, &out[totlen], &len);
+      free (format);
       if (res != SHISHI_OK)
 	return res;
 
@@ -245,6 +249,8 @@ shishi_principal_name_realm_get (Shishi * handle,
   size_t totlen = 0;
   int len;
 
+  /* FIXME: allocate output instead of writing inline */
+
   totlen = *outlen;
   res = shishi_principal_name_get (handle, namenode, namefield, out, &totlen);
   if (res != SHISHI_OK)
@@ -307,11 +313,9 @@ shishi_principal_name_set (Shishi * handle,
   char *buf, *asn1name;
   int i;
 
-  asprintf (&buf, "%d", name_type);
   asprintf (&asn1name, "%s.name-type", namefield);
-  res = shishi_asn1_write (handle, namenode, asn1name, buf, 0);
+  res = shishi_asn1_write_int32 (handle, namenode, asn1name, name_type);
   free (asn1name);
-  free (buf);
   if (res != SHISHI_OK)
     return res;
 
@@ -364,24 +368,19 @@ shishi_principal_set (Shishi * handle,
   int res;
   int i;
 
-  tmpname = strdup (name);
-  if (tmpname == NULL)
-    return SHISHI_MALLOC_ERROR;
-
-  namebuf = malloc (sizeof (*namebuf));
-  if (namebuf == NULL)
-    return SHISHI_MALLOC_ERROR;
+  tmpname = xstrdup (name);
+  namebuf = xmalloc (sizeof (*namebuf));
 
   for (i = 0;
        (namebuf[i] = strtok_r (i == 0 ? tmpname : NULL, "/", &tokptr)); i++)
     {
-      namebuf = realloc (namebuf, (i + 2) * sizeof (*namebuf));
-      if (namebuf == NULL)
-	return SHISHI_MALLOC_ERROR;
+      namebuf = xrealloc (namebuf, (i + 2) * sizeof (*namebuf));
     }
 
   res = shishi_principal_name_set (handle, namenode, namefield,
 				   SHISHI_NT_UNKNOWN, namebuf);
+  free (namebuf);
+  free (tmpname);
   if (res != SHISHI_OK)
     {
       shishi_error_printf (handle, _("Could not set principal name: %s\n"),
@@ -389,9 +388,6 @@ shishi_principal_set (Shishi * handle,
       return res;
     }
 
-  free (namebuf);
-  free (tmpname);
-
   return SHISHI_OK;
 }
 
@@ -411,5 +407,5 @@ shishi_server_for_local_service (Shishi * handle, const char *service)
   if (ret != 0)
     strcpy (&buf[strlen (service) + 1], "localhost");
 
-  return xstrdup(buf);
+  return xstrdup (buf);
 }
