@@ -160,20 +160,21 @@ shishi_safe_safe_set (Shishi_safe * safe, Shishi_asn1 asn1safe)
 /**
  * shishi_safe_safe_der:
  * @safe: safe as allocated by shishi_safe().
- * @out: output array with der encoding of SAFE.
- * @outlen: length of output array with der encoding of SAFE.
+ * @out: output array with newly allocated DER encoding of SAFE.
+ * @outlen: length of output array with DER encoding of SAFE.
  *
  * DER encode SAFE structure.  Typically shishi_safe_build() is used
- * instead to build the SAFE structure first.
+ * to build the SAFE structure first.  @out is allocated by this
+ * function, and it is the responsibility of caller to deallocate it.
  *
  * Return value: Returns SHISHI_OK iff successful.
  **/
 int
-shishi_safe_safe_der (Shishi_safe * safe, char *out, int *outlen)
+shishi_safe_safe_der (Shishi_safe * safe, char **out, size_t *outlen)
 {
   int rc;
 
-  rc = shishi_a2d (safe->handle, safe->safe, out, outlen);
+  rc = shishi_new_a2d (safe->handle, safe->safe, out, outlen);
   if (rc != SHISHI_OK)
     return rc;
 
@@ -371,18 +372,18 @@ shishi_safe_from_file (Shishi * handle, Shishi_asn1 * safe,
  * @handle: shishi handle as allocated by shishi_init().
  * @safe: safe as allocated by shishi_safe().
  * @cksumtype: output checksum type.
- * @cksum: output checksum data from SAFE.
- * @cksumlen: on input, maximum size of output checksum data buffer,
- *            on output, actual size of output checksum data buffer.
+ * @cksum: output array with newly allocated checksum data from SAFE.
+ * @cksumlen: output size of output checksum data buffer.
  *
- * Read checksum value from KRB-SAFE.
+ * Read checksum value from KRB-SAFE.  @cksum is allocated by this
+ * function, and it is the responsibility of caller to deallocate it.
  *
  * Return value: Returns SHISHI_OK iff successful.
  **/
 int
 shishi_safe_cksum (Shishi * handle,
 		   Shishi_asn1 safe,
-		   int32_t * cksumtype, char *cksum, size_t * cksumlen)
+		   int32_t * cksumtype, char **cksum, size_t * cksumlen)
 {
   int res;
 
@@ -390,7 +391,7 @@ shishi_safe_cksum (Shishi * handle,
   if (res != SHISHI_OK)
     return res;
 
-  res = shishi_asn1_read (handle, safe, "cksum.checksum", cksum, cksumlen);
+  res = shishi_asn1_read2 (handle, safe, "cksum.checksum", cksum, cksumlen);
   if (res != SHISHI_OK)
     return res;
 
@@ -435,21 +436,21 @@ shishi_safe_set_cksum (Shishi * handle,
  * shishi_safe_user_data:
  * @handle: shishi handle as allocated by shishi_init().
  * @safe: safe as allocated by shishi_safe().
- * @userdata: output user data from KRB-SAFE.
- * @userdatalen: on input, maximum size of output user data buffer,
- *               on output, actual size of output user data buffer.
+ * @userdata: output array with newly allocated user data from KRB-SAFE.
+ * @userdatalen: output size of output user data buffer.
  *
- * Read user data value from KRB-SAFE.
+ * Read user data value from KRB-SAFE.  @userdata is allocated by this
+ * function, and it is the responsibility of caller to deallocate it.
  *
  * Return value: Returns SHISHI_OK iff successful.
  **/
 int
 shishi_safe_user_data (Shishi * handle,
-		       Shishi_asn1 safe, char *userdata, size_t * userdatalen)
+		       Shishi_asn1 safe, char **userdata, size_t * userdatalen)
 {
   int res;
 
-  res = shishi_asn1_read (handle, safe, "safe-body.user-data",
+  res = shishi_asn1_read2 (handle, safe, "safe-body.user-data",
 			  userdata, userdatalen);
   if (res != SHISHI_OK)
     return res;
@@ -498,8 +499,8 @@ int
 shishi_safe_build (Shishi_safe * safe, Shishi_key * key)
 {
   int rc;
-  char buffer[BUFSIZ];
-  int buflen;
+  char *buffer;
+  size_t buflen;
   char *cksum;
   int cksumlen;
   int cksumtype = shishi_cipher_defaultcksumtype (shishi_key_type (key));
@@ -508,8 +509,7 @@ shishi_safe_build (Shishi_safe * safe, Shishi_key * key)
   if (rc != SHISHI_OK)
     return rc;
 
-  buflen = sizeof (buffer);
-  rc = shishi_safe_safe_der (safe, buffer, &buflen);
+  rc = shishi_safe_safe_der (safe, &buffer, &buflen);
   if (rc != SHISHI_OK)
     return rc;
 
@@ -518,9 +518,9 @@ shishi_safe_build (Shishi_safe * safe, Shishi_key * key)
   if (VERBOSEASN1 (safe->handle))
     shishi_key_print (safe->handle, stdout, key);
 
-  rc =
-    shishi_checksum (safe->handle, key, SHISHI_KEYUSAGE_KRB_SAFE, cksumtype,
-		     buffer, buflen, &cksum, &cksumlen);
+  rc = shishi_checksum (safe->handle, key, SHISHI_KEYUSAGE_KRB_SAFE,
+			cksumtype, buffer, buflen, &cksum, &cksumlen);
+  free (buffer);
   if (rc != SHISHI_OK)
     return rc;
 
@@ -549,35 +549,46 @@ shishi_safe_build (Shishi_safe * safe, Shishi_key * key)
 int
 shishi_safe_verify (Shishi_safe * safe, Shishi_key * key)
 {
-  char cksum[MAX_CKSUM_LEN];
+  char *cksum;
   size_t cksumlen;
   int cksumtype;
-  char cksum2[MAX_CKSUM_LEN];
+  char *cksum2;
   size_t cksum2len;
   int cksumtype2;
   int rc;
 
-  cksumlen = sizeof (cksum);
   rc = shishi_safe_cksum (safe->handle, safe->safe,
-			  &cksumtype, cksum, &cksumlen);
+			  &cksumtype, &cksum, &cksumlen);
   if (rc != SHISHI_OK)
-    return rc;
+    goto error;
 
   rc = shishi_safe_build (safe, key);
   if (rc != SHISHI_OK)
-    return rc;
+    goto error;
 
-  cksum2len = sizeof (cksum2);
   rc = shishi_safe_cksum (safe->handle, safe->safe,
-			  &cksumtype2, cksum2, &cksum2len);
+			  &cksumtype2, &cksum2, &cksum2len);
   if (rc != SHISHI_OK)
-    return rc;
+    goto error;
 
   if (cksumtype != cksumtype2)
-    return SHISHI_SAFE_BAD_KEYTYPE;
+    {
+      rc = SHISHI_SAFE_BAD_KEYTYPE;
+      goto error;
+    }
 
   if (cksum2len != cksumlen || memcmp (cksum, cksum2, cksumlen) != 0)
-    return SHISHI_SAFE_VERIFY_FAILED;
+    {
+      rc = SHISHI_SAFE_VERIFY_FAILED;
+      goto error;
+    }
 
   return SHISHI_OK;
+
+ error:
+  if (cksum)
+    free (cksum);
+  if (cksum2)
+    free (cksum2);
+  return rc;
 }
