@@ -27,10 +27,10 @@
 #include <time.h>		/* time_t */
 #include <shishi-int.h>		/* uint32_t */
 
-#define SHISHI_VERSION "0.0.3"
+#define SHISHI_VERSION "0.0.4"
 
 /* Error codes */
-enum
+typedef enum
 {
   SHISHI_OK = 0,
   SHISHI_ASN1_ERROR = 1,
@@ -76,35 +76,75 @@ enum
   SHISHI_PKCS5_INVALID_DERIVED_KEY_LENGTH = 41,
   SHISHI_PKCS5_DERIVED_KEY_TOO_LONG = 42,
   SHISHI_INVALID_PRINCIPAL_NAME = 43,
-  SHISHI_INVALID_ARGUMENT = 44
-};
+  SHISHI_INVALID_ARGUMENT = 44,
+  SHISHI_ASN1_NO_VALUE = 45,
+  SHISHI_CONNECT_ERROR = 46,
+  SHISHI_LAST_ERROR = 46
+}
+Shishi_rc;
 
-/* This is not specified in the ASN.1 syntax for some reason. */
 typedef enum
 {
-  SHISHI_NT_UNKNOWN = 0,	/* Name type not known */
-  SHISHI_NT_PRINCIPAL,		/* Just the name of the principal as in */
-  /* DCE, or for users */
-  SHISHI_NT_SRV_INST,		/* Service and other unique instance (krbtgt) */
-  SHISHI_NT_SRV_HST,		/* Service with host name as instance */
-  /* (telnet, rcommands) */
-  SHISHI_NT_SRV_XHST,		/* Service with host as remaining components */
-  SHISHI_NT_UID			/* Unique ID */
+  /* Name type not known */
+  SHISHI_NT_UNKNOWN = 0,
+  /* Just the name of the principal as in DCE, or for users */
+  SHISHI_NT_PRINCIPAL = 1,
+  /* Service and other unique instance (krbtgt) */
+  SHISHI_NT_SRV_INST = 2,
+  /* Service with host name as instance (telnet, rcommands) */
+  SHISHI_NT_SRV_HST = 3,
+  /* Service with host as remaining components */
+  SHISHI_NT_SRV_XHST = 4,
+  /* Unique ID */
+  SHISHI_NT_UID = 5,
+  /* Encoded X.509 Distingished name [RFC 2253] */
+  SHISHI_NT_X500_PRINCIPAL = 6,
+  /* Name in form of SMTP email name (e.g. user@foo.com) */
+  SHISHI_NT_SMTP_NAME = 7,
+  /*  Enterprise name - may be mapped to principal name */
+  SHISHI_NT_ENTERPRISE = 10
 }
 Shishi_name_type;
 
-/* This is not specified in the ASN.1 syntax for some reason. */
 typedef enum
 {
-  SHISHI_PA_TGS_REQ = 1,	/* padata is APREQ */
-  SHISHI_PA_ENC_TIMESTAMP,	/* timestamp pre authentication */
-  SHISHI_PA_PW_SALT,		/* salt for string-to-key */
-  SHISHI_PA_ETYPE_INFO = 10,	/* in krb-error, request for preauth */
-  SHISHI_PA_USE_SPECIFIED_KVNO = 20	/* use specified kvno */
+  SHISHI_PA_TGS_REQ = 1,
+  SHISHI_PA_ENC_TIMESTAMP = 2,
+  SHISHI_PA_PW_SALT = 3,
+  SHISHI_PA_RESERVED = 4,
+  SHISHI_PA_ENC_UNIX_TIME = 5,		/* (deprecated) */
+  SHISHI_PA_SANDIA_SECUREID = 6,
+  SHISHI_PA_SESAME = 7,
+  SHISHI_PA_OSF_DCE = 8,
+  SHISHI_PA_CYBERSAFE_SECUREID = 9,
+  SHISHI_PA_AFS3_SALT = 10,
+  SHISHI_PA_ETYPE_INFO = 11,
+  SHISHI_PA_SAM_CHALLENGE = 12,		/* (sam/otp) */
+  SHISHI_PA_SAM_RESPONSE = 13,		/* (sam/otp) */
+  SHISHI_PA_PK_AS_REQ = 14,		/* (pkinit) */
+  SHISHI_PA_PK_AS_REP = 15,		/* (pkinit) */
+  SHISHI_PA_ETYPE_INFO2 = 19,		/* (replaces pa_etype_info) */
+  SHISHI_PA_USE_SPECIFIED_KVNO = 20,
+  SHISHI_PA_SAM_REDIRECT = 21,		/* (sam/otp) */
+  SHISHI_PA_GET_FROM_TYPED_DATA = 22,	/* (embedded in typed data) */
+  SHISHI_TD_PADATA = 22,		/* (embeds padata) */
+  SHISHI_PA_SAM_ETYPE_INFO = 23,	/* (sam/otp) */
+  SHISHI_PA_ALT_PRINC = 24,		/* (crawdad@fnal.gov) */
+  SHISHI_PA_SAM_CHALLENGE2 = 30,	/* (kenh@pobox.com) */
+  SHISHI_PA_SAM_RESPONSE2 = 31,		/* (kenh@pobox.com) */
+  SHISHI_PA_EXTRA_TGT = 41,		/* Reserved extra TGT */
+  SHISHI_TD_PKINIT_CMS_CERTIFICATES = 101, /* CertificateSet from CMS */
+  SHISHI_TD_KRB_PRINCIPAL = 102,	/* PrincipalName */
+  SHISHI_TD_KRB_REALM = 103,		/* Realm */
+  SHISHI_TD_TRUSTED_CERTIFIERS = 104,	/* from PKINIT */
+  SHISHI_TD_CERTIFICATE_INDEX = 105,	/* from PKINIT */
+  SHISHI_TD_APP_DEFINED_ERROR = 106,	/* application specific */
+  SHISHI_TD_REQ_NONCE = 107,		/* INTEGER */
+  SHISHI_TD_REQ_SEQ = 108,		/* INTEGER */
+  SHISHI_PA_PAC_REQUEST = 128		/* (jbrezak@exchange.microsoft.com) */
 }
 Shishi_padata_type;
 
-/* This is not specified in the ASN.1 syntax for some reason. */
 typedef enum
 {
   SHISHI_TR_DOMAIN_X500_COMPRESS = 1
@@ -162,10 +202,30 @@ Shishi_KDCOptions;
 
 typedef enum
 {
-  SHISHI_MSGTYPE_AS_REQ = 0x10,
-  SHISHI_MSGTYPE_AS_REP = 0x11,
-  SHISHI_MSGTYPE_TGS_REQ = 0x12,
-  SHISHI_MSGTYPE_TGS_REP = 0x13
+  /* Request for initial authentication */
+  SHISHI_MSGTYPE_AS_REQ = 10,
+  /* Response to SHISHI_MSGTYPE_AS_REQ request */
+  SHISHI_MSGTYPE_AS_REP = 11,
+  /* Request for authentication based on TGT */
+  SHISHI_MSGTYPE_TGS_REQ = 12,
+  /* Response to SHISHI_MSGTYPE_TGS_REQ request */
+  SHISHI_MSGTYPE_TGS_REP = 13,
+  /* application request to server */
+  SHISHI_MSGTYPE_AP_REQ = 14,
+  /* Response to SHISHI_MSGTYPE_AP_REQ_MUTUAL */
+  SHISHI_MSGTYPE_AP_REP = 15,
+  /* Reserved for user-to-user krb_tgt_request */
+  SHISHI_MSGTYPE_RESERVED16 = 16,
+  /* Reserved for user-to-user krb_tgt_reply */
+  SHISHI_MSGTYPE_RESERVED17 = 17,
+  /* Safe (checksummed) application message */
+  SHISHI_MSGTYPE_SAFE = 20,
+  /* Private (encrypted) application message */
+  SHISHI_MSGTYPE_PRIV = 21,
+  /* Private (encrypted) message to forward credentials */
+  SHISHI_MSGTYPE_CRED = 22,
+  /* Error response */
+  SHISHI_MSGTYPE_ERROR = 30
 }
 Shishi_msgtype;
 
@@ -203,7 +263,7 @@ typedef enum
   SHISHI_RSA_MD4_DES_K = 6,
   SHISHI_RSA_MD5 = 7,
   SHISHI_RSA_MD5_DES = 8,
-  SHISHI_RSA_MD5_DES_GSS = 9,  /* XXX */
+  SHISHI_RSA_MD5_DES_GSS = 9,	/* XXX */
   SHISHI_HMAC_SHA1_96_AES128 = 10,
   SHISHI_HMAC_SHA1_96_AES256 = 11,
   SHISHI_HMAC_SHA1_DES3_KD = 12
@@ -284,6 +344,148 @@ typedef enum
 }
 Shishi_keyusage;
 
+typedef enum
+{
+  /* No error */
+  SHISHI_KDC_ERR_NONE = 0,
+  /* Client's entry in database has expired */
+  SHISHI_KDC_ERR_NAME_EXP = 1,
+  /* Server's entry in database has expired */
+  SHISHI_KDC_ERR_SERVICE_EXP = 2,
+  /* Requested protocol version number  - not supported */
+  SHISHI_KDC_ERR_BAD_PVNO = 3,
+  /* Client's key encrypted in old master key */
+  SHISHI_KDC_ERR_C_OLD_MAST_KVNO = 4,
+  /* Server's key encrypted in old master key */
+  SHISHI_KDC_ERR_S_OLD_MAST_KVNO = 5,
+  /* Client not found in Kerberos database */
+  SHISHI_KDC_ERR_C_PRINCIPAL_UNKNOWN = 6,
+  /* Server not found in Kerberos database */
+  SHISHI_KDC_ERR_S_PRINCIPAL_UNKNOWN = 7,
+  /* Multiple principal entries in database */
+  SHISHI_KDC_ERR_PRINCIPAL_NOT_UNIQUE = 8,
+  /* The client or server has a null key */
+  SHISHI_KDC_ERR_NULL_KEY = 9,
+  /* Ticket not eligible for postdating */
+  SHISHI_KDC_ERR_CANNOT_POSTDATE = 10,
+  /* Requested start time is later than end time */
+  SHISHI_KDC_ERR_NEVER_VALID = 11,
+  /* KDC policy rejects request */
+  SHISHI_KDC_ERR_POLICY = 12,
+  /* KDC cannot accommodate requested option */
+  SHISHI_KDC_ERR_BADOPTION = 13,
+  /* KDC has no support for encryption type */
+  SHISHI_KDC_ERR_ETYPE_NOSUPP = 14,
+  /* KDC has no support for checksum type */
+  SHISHI_KDC_ERR_SUMTYPE_NOSUPP = 15,
+  /* KDC has no support for padata type */
+  SHISHI_KDC_ERR_PADATA_TYPE_NOSUPP = 16,
+  /* KDC has no support for transited type */
+  SHISHI_KDC_ERR_TRTYPE_NOSUPP = 17,
+  /* Clients credentials have been revoked */
+  SHISHI_KDC_ERR_CLIENT_REVOKED = 18,
+  /* Credentials for server have been revoked */
+  SHISHI_KDC_ERR_SERVICE_REVOKED = 19,
+  /* TGT has been revoked */
+  SHISHI_KDC_ERR_TGT_REVOKED = 20,
+  /* Client not yet valid - try again later */
+  SHISHI_KDC_ERR_CLIENT_NOTYET = 21,
+  /* Server not yet valid - try again later */
+  SHISHI_KDC_ERR_SERVICE_NOTYET = 22,
+  /* Password has expired - change password to reset */
+  SHISHI_KDC_ERR_KEY_EXPIRED = 23,
+  /* Pre-authentication information was invalid */
+  SHISHI_KDC_ERR_PREAUTH_FAILED = 24,
+  /* Additional pre-authenticationrequired */
+  SHISHI_KDC_ERR_PREAUTH_REQUIRED = 25,
+  /* Requested server and ticket don't match */
+  SHISHI_KDC_ERR_SERVER_NOMATCH = 26,
+  /* Server principal valid for user = 2,user only */
+  SHISHI_KDC_ERR_MUST_USE_USER2USER = 27,
+  /* KDC Policy rejects transited path */
+  SHISHI_KDC_ERR_PATH_NOT_ACCPETED = 28,
+  /* A service is not available */
+  SHISHI_KDC_ERR_SVC_UNAVAILABLE = 29,
+  /* Integrity check on decrypted field failed */
+  SHISHI_KRB_AP_ERR_BAD_INTEGRITY = 31,
+  /* Ticket expired */
+  SHISHI_KRB_AP_ERR_TKT_EXPIRED = 32,
+  /* Ticket not yet valid */
+  SHISHI_KRB_AP_ERR_TKT_NYV = 33,
+  /* Request is a replay */
+  SHISHI_KRB_AP_ERR_REPEAT = 34,
+  /* The ticket isn't for us */
+  SHISHI_KRB_AP_ERR_NOT_US = 35,
+  /* Ticket and authenticator don't match */
+  SHISHI_KRB_AP_ERR_BADMATCH = 36,
+  /* Clock skew too great */
+  SHISHI_KRB_AP_ERR_SKEW = 37,
+  /* Incorrect net address */
+  SHISHI_KRB_AP_ERR_BADADDR = 38,
+  /* Protocol version mismatch */
+  SHISHI_KRB_AP_ERR_BADVERSION = 39,
+  /* Invalid msg type */
+  SHISHI_KRB_AP_ERR_MSG_TYPE = 40,
+  /* Message stream modified */
+  SHISHI_KRB_AP_ERR_MODIFIED = 41,
+  /* Message out of order */
+  SHISHI_KRB_AP_ERR_BADORDER = 42,
+  /* Specified version of key is not available */
+  SHISHI_KRB_AP_ERR_BADKEYVER = 44,
+  /* Service key not available */
+  SHISHI_KRB_AP_ERR_NOKEY = 45,
+  /* Mutual authentication failed */
+  SHISHI_KRB_AP_ERR_MUT_FAIL = 46,
+  /* Incorrect message direction */
+  SHISHI_KRB_AP_ERR_BADDIRECTION = 47,
+  /* Alternative authentication method required */
+  SHISHI_KRB_AP_ERR_METHOD = 48,
+  /* Incorrect sequence number in message */
+  SHISHI_KRB_AP_ERR_BADSEQ = 49,
+  /* Inappropriate type of checksum in message */
+  SHISHI_KRB_AP_ERR_INAPP_CKSUM = 50,
+  /* Policy rejects transited path */
+  SHISHI_KRB_AP_PATH_NOT_ACCEPTED = 51,
+  /* Response too big for UDP, retry with TCP */
+  SHISHI_KRB_ERR_RESPONSE_TOO_BIG = 52,
+  /* Generic error (description in e-text) */
+  SHISHI_KRB_ERR_GENERIC = 60,
+  /* Field is too long for this implementation */
+  SHISHI_KRB_ERR_FIELD_TOOLONG = 61,
+  /* Reserved for PKINIT */
+  SHISHI_KDC_ERROR_CLIENT_NOT_TRUSTED = 62,
+  /* Reserved for PKINIT */
+  SHISHI_KDC_ERROR_KDC_NOT_TRUSTED = 63,
+  /* Reserved for PKINIT */
+  SHISHI_KDC_ERROR_INVALID_SIG = 64,
+  /* Reserved for PKINIT */
+  SHISHI_KDC_ERR_KEY_TOO_WEAK = 65,
+  /* Reserved for PKINIT */
+  SHISHI_KDC_ERR_CERTIFICATE_MISMATCH = 66,
+  /* No TGT available to validate USER-TO-USER */
+  SHISHI_KRB_AP_ERR_NO_TGT = 67,
+  /* USER-TO-USER TGT issued different KDC */
+  SHISHI_KDC_ERR_WRONG_REALM = 68,
+  /* Ticket must be for USER-TO-USER */
+  SHISHI_KRB_AP_ERR_USER_TO_USER_REQUIRED = 69,
+  /* Reserved for PKINIT */
+  SHISHI_KDC_ERR_CANT_VERIFY_CERTIFICATE = 70,
+  /* Reserved for PKINIT */
+  SHISHI_KDC_ERR_INVALID_CERTIFICATE = 71,
+  /* Reserved for PKINIT */
+  SHISHI_KDC_ERR_REVOKED_CERTIFICATE = 72,
+  /* Reserved for PKINIT */
+  SHISHI_KDC_ERR_REVOCATION_STATUS_UNKNOWN = 73,
+  /* Reserved for PKINIT */
+  SHISHI_KDC_ERR_REVOCATION_STATUS_UNAVAILABLE = 74,
+  /* Reserved for PKINIT */
+  SHISHI_KDC_ERR_CLIENT_NAME_MISMATCH = 75,
+  /* Reserved for PKINIT */
+  SHISHI_KDC_ERR_KDC_NAME_MISMATCH = 76,
+  SHISHI_LAST_ERROR_CODE = 76
+}
+Shishi_error_code;
+
 typedef enum
 {
   SHISHI_TKTSHINTFLAGS_ACCEPT_EXPIRED = 1
@@ -406,8 +608,9 @@ extern int shishi_ticket_srealmserver_set (Shishi * handle,
 					   char *server);
 extern int shishi_ticket_set_server (Shishi * handle, Shishi_asn1 ticket,
 				     const char *server);
-extern int shishi_ticket_realm_get (Shishi * handle, Shishi_asn1 ticket,
-				    char *realm, int *realmlen);
+extern int shishi_ticket_realm_get (Shishi * handle,
+				    Shishi_asn1 ticket,
+				    char **realm, size_t *realmlen);
 extern int shishi_ticket_realm_set (Shishi * handle, Shishi_asn1 ticket,
 				    const char *realm);
 extern int shishi_ticket_get_enc_part_etype (Shishi * handle,
@@ -443,7 +646,7 @@ extern Shishi_tkt *shishi_tkt2 (Shishi * handle,
 				Shishi_asn1 enckdcreppart,
 				Shishi_asn1 kdcrep);
 extern int shishi_tkt_pretty_print (Shishi_tkt * tkt, FILE * fh);
-extern int shishi_tkt_realm (Shishi_tkt * tkt, char *realm, int *realmlen);
+extern int shishi_tkt_realm (Shishi_tkt * tkt, char **realm, size_t *realmlen);
 extern int shishi_tkt_cnamerealm (Shishi_tkt * tkt,
 				  char *cnamerealm, int *cnamerealmlen);
 extern int shishi_tkt_cnamerealm_p (Shishi_tkt * tkt, const char *client);
@@ -621,6 +824,27 @@ extern int shishi_authenticator_add_cksum (Shishi * handle,
 					   int keyusage,
 					   char *data, int datalen);
 extern int
+shishi_authenticator_remove_subkey (Shishi * handle,
+				    Shishi_asn1 authenticator);
+extern Shishi_asn1
+shishi_authenticator_subkey (Shishi * handle);
+extern int
+shishi_authenticator_get_subkey (Shishi * handle,
+			    Shishi_asn1 authenticator,
+			    Shishi_key ** subkey);
+extern int
+shishi_authenticator_set_subkey (Shishi * handle,
+				Shishi_asn1 authenticator,
+				int32_t subkeytype,
+				 char *subkey, size_t subkeylen);
+extern int
+shishi_authenticator_add_random_subkey (Shishi * handle,
+				 Shishi_asn1 authenticator);
+extern int
+shishi_authenticator_add_subkey (Shishi * handle,
+				 Shishi_asn1 authenticator,
+				 Shishi_key *subkey);
+extern int
 shishi_authenticator_clear_authorizationdata (Shishi * handle,
 					      Shishi_asn1 authenticator);
 extern int
@@ -653,18 +877,18 @@ extern int shishi_authenticator_save (Shishi * handle, FILE * fh,
 extern Shishi_asn1 shishi_as_req (Shishi_as * as);
 extern int shishi_as_req_build (Shishi_as * as);
 extern void shishi_as_req_set (Shishi_as * as, Shishi_asn1 asreq);
-extern int shishi_as_req_der (Shishi_as * as, char *out, int *outlen);
+extern int shishi_as_req_der (Shishi_as * as, char **out, size_t *outlen);
 extern int shishi_as_req_der_set (Shishi_as * as, char *der, size_t derlen);
 extern Shishi_asn1 shishi_as_rep (Shishi_as * as);
 extern void shishi_as_rep_set (Shishi_as * as, Shishi_asn1 asrep);
 extern int shishi_as_rep_build (Shishi_as * as, Shishi_key * key);
-extern int shishi_as_rep_der (Shishi_as * as, char *out, int *outlen);
+extern int shishi_as_rep_der (Shishi_as * as, char **out, size_t *outlen);
 extern int shishi_as_rep_der_set (Shishi_as * as, char *der, size_t derlen);
 extern Shishi_asn1 shishi_as_encasreppart (Shishi_as * as);
 extern void shishi_as_encasreppart_set (Shishi_as * as,
 					Shishi_asn1 encasreppart);
 extern Shishi_asn1 shishi_as_krberror (Shishi_as * as);
-extern int shishi_as_krberror_der (Shishi_as * as, char *out, int *outlen);
+extern int shishi_as_krberror_der (Shishi_as * as, char **out, size_t *outlen);
 extern void shishi_as_krberror_set (Shishi_as * as, Shishi_asn1 krberror);
 extern Shishi_tkt *shishi_as_tkt (Shishi_as * as);
 extern void shishi_as_tkt_set (Shishi_as * as, Shishi_tkt * tkt);
@@ -694,10 +918,19 @@ extern Shishi_tkt *shishi_tgs_tgtkt (Shishi_tgs * tgs);
 extern void shishi_tgs_tgtkt_set (Shishi_tgs * tgs, Shishi_tkt * tgtkt);
 extern Shishi_ap *shishi_tgs_ap (Shishi_tgs * tgs);
 extern Shishi_asn1 shishi_tgs_req (Shishi_tgs * tgs);
+extern int shishi_tgs_req_der (Shishi_tgs * tgs, char **out, size_t *outlen);
+extern int shishi_tgs_req_der_set (Shishi_tgs * tgs, char *der, size_t derlen);
+extern void shishi_tgs_req_set (Shishi_tgs * tgs, Shishi_asn1 tgsreq);
 extern int shishi_tgs_req_build (Shishi_tgs * tgs);
+extern int shishi_tgs_req_process (Shishi_tgs * tgs);
 extern Shishi_asn1 shishi_tgs_rep (Shishi_tgs * tgs);
+extern int shishi_tgs_rep_der (Shishi_tgs * tgs, char **out, size_t *outlen);
+extern int shishi_tgs_rep_build (Shishi_tgs * tgs, Shishi_key * key);
 extern int shishi_tgs_rep_process (Shishi_tgs * tgs);
 extern Shishi_asn1 shishi_tgs_krberror (Shishi_tgs * tgs);
+extern int shishi_tgs_krberror_der (Shishi_tgs * tgs, char **out,
+				    size_t *outlen);
+extern void shishi_tgs_krberror_set (Shishi_tgs * tgs, Shishi_asn1 krberror);
 extern Shishi_tkt *shishi_tgs_tkt (Shishi_tgs * tgs);
 extern void shishi_tgs_tkt_set (Shishi_tgs * tgs, Shishi_tkt * tkt);
 extern int shishi_tgs_sendrecv (Shishi_tgs * tgs);
@@ -766,6 +999,13 @@ extern int shishi_kdcreq_options_set (Shishi * handle, Shishi_asn1 kdcreq,
 extern int shishi_kdcreq_options_add (Shishi * handle, Shishi_asn1 kdcreq,
 				      int option);
 extern int shishi_kdcreq_clear_padata (Shishi * handle, Shishi_asn1 kdcreq);
+extern int shishi_kdcreq_get_padata (Shishi * handle,
+				     Shishi_asn1 kdcreq,
+				     Shishi_padata_type padatatype,
+				     char **out, size_t *outlen);
+extern int shishi_kdcreq_get_padata_tgs (Shishi * handle,
+					 Shishi_asn1 kdcreq,
+					 Shishi_asn1 *apreq);
 extern int shishi_kdcreq_add_padata (Shishi * handle,
 				     Shishi_asn1 kdcreq,
 				     int padatatype, char *data, int datalen);
@@ -797,8 +1037,10 @@ extern int shishi_as_derive_salt (Shishi * handle,
 				  Shishi_asn1 asreq,
 				  Shishi_asn1 asrep, char *salt,
 				  size_t * saltlen);
-extern int shishi_tgs_process (Shishi * handle, Shishi_asn1 tgsreq,
+extern int shishi_tgs_process (Shishi * handle,
+			       Shishi_asn1 tgsreq,
 			       Shishi_asn1 tgsrep,
+			       Shishi_asn1 authenticator,
 			       Shishi_asn1 oldenckdcreppart,
 			       Shishi_asn1 * enckdcreppart);
 extern int shishi_as_process (Shishi * handle, Shishi_asn1 asreq,
@@ -882,29 +1124,106 @@ extern int shishi_krberror_read (Shishi * handle, FILE * fh,
 				 Shishi_asn1 * krberror);
 extern int shishi_krberror_from_file (Shishi * handle, Shishi_asn1 * krberror,
 				      int filetype, char *filename);
-extern const char *shishi_krberror_errorcode_message (Shishi * handle,
-						      int errorcode);
-extern const char *shishi_krberror_message (Shishi * handle,
-					    Shishi_asn1 krberror);
+extern int shishi_krberror_build (Shishi * handle, Shishi_asn1 krberror);
+extern int shishi_krberror_der (Shishi * handle,
+				Shishi_asn1 krberror,
+				char **out, size_t *outlen);
+extern int shishi_krberror_crealm (Shishi * handle,
+				   Shishi_asn1 krberror,
+				   char **realm, size_t *realmlen);
+extern int shishi_krberror_remove_crealm (Shishi * handle,
+					  Shishi_asn1 krberror);
+extern int shishi_krberror_set_crealm (Shishi * handle,
+				       Shishi_asn1 krberror,
+				       const char *crealm);
+extern int shishi_krberror_cname (Shishi * handle,
+				  Shishi_asn1 krberror,
+				  char *out, size_t *outlen);
+extern int shishi_krberror_set_cname (Shishi * handle,
+				      Shishi_asn1 krberror,
+				      Shishi_name_type name_type,
+				      const char *cname[]);
+extern int shishi_krberror_remove_cname (Shishi * handle,
+					 Shishi_asn1 krberror);
+extern int shishi_krberror_client_set (Shishi * handle,
+				       Shishi_asn1 krberror,
+				       const char *client);
+extern int shishi_krberror_realm (Shishi * handle,
+				  Shishi_asn1 krberror,
+				  char **realm, size_t *realmlen);
+extern int shishi_krberror_set_realm (Shishi * handle,
+				      Shishi_asn1 krberror,
+				      const char *realm);
+extern int shishi_krberror_sname (Shishi * handle,
+				  Shishi_asn1 krberror,
+				  char *out, size_t *outlen);
+extern int shishi_krberror_remove_sname (Shishi * handle,
+					 Shishi_asn1 krberror);
+extern int shishi_krberror_set_sname (Shishi * handle,
+				      Shishi_asn1 krberror,
+				      Shishi_name_type name_type,
+				      const char *sname[]);
+extern int shishi_krberror_server_set (Shishi * handle,
+				       Shishi_asn1 krberror,
+				       const char *server);
+extern int shishi_krberror_ctime_get (Shishi * handle,
+				      Shishi_asn1 krberror, char *ctime);
+extern int shishi_krberror_ctime_set (Shishi * handle,
+				      Shishi_asn1 krberror, char *ctime);
+extern int shishi_krberror_remove_ctime (Shishi * handle,
+					 Shishi_asn1 krberror);
+extern int shishi_krberror_cusec_get (Shishi * handle,
+				      Shishi_asn1 krberror,
+				      int *cusec);
+extern int shishi_krberror_cusec_set (Shishi * handle,
+				      Shishi_asn1 krberror,
+				      int cusec);
+extern int shishi_krberror_remove_cusec (Shishi * handle,
+					 Shishi_asn1 krberror);
+extern int shishi_krberror_stime_get (Shishi * handle,
+				      Shishi_asn1 krberror, char *stime);
+extern int shishi_krberror_stime_set (Shishi * handle,
+				      Shishi_asn1 krberror, char *stime);
+extern int shishi_krberror_susec_get (Shishi * handle,
+				      Shishi_asn1 krberror, int *susec);
+extern int shishi_krberror_susec_set (Shishi * handle,
+				      Shishi_asn1 krberror,
+				      int susec);
+extern int shishi_krberror_errorcode_set (Shishi * handle,
+					  Shishi_asn1 krberror,
+					  int errorcode);
 extern int shishi_krberror_etext (Shishi * handle, Shishi_asn1 krberror,
-				  char *etext, size_t * etextlen);
+				  char **etext, size_t * etextlen);
+extern int shishi_krberror_set_etext (Shishi * handle,
+				      Shishi_asn1 krberror,
+				      const char *etext);
+extern int shishi_krberror_remove_etext (Shishi * handle,
+					 Shishi_asn1 krberror);
+extern int shishi_krberror_edata (Shishi * handle, Shishi_asn1 krberror,
+				  char **edata, size_t * edatalen);
+extern int shishi_krberror_set_edata (Shishi * handle,
+				      Shishi_asn1 krberror,
+				      const char *edata);
+extern int shishi_krberror_remove_edata (Shishi * handle,
+					 Shishi_asn1 krberror);
 extern int shishi_krberror_errorcode (Shishi * handle,
-				      Shishi_asn1 krberror, int *errorcode);
+				      Shishi_asn1 krberror,
+				      int *errorcode);
 extern int shishi_krberror_errorcode_fast (Shishi * handle,
 					   Shishi_asn1 krberror);
 extern int shishi_krberror_pretty_print (Shishi * handle,
 					 FILE * fh, Shishi_asn1 krberror);
-
-/* base64.c */
-extern void shishi_to_base64 (unsigned char *out, const unsigned char *in,
-			      int len, int olen);
-extern int shishi_from_base64 (unsigned char *out, const unsigned char *in);
+extern const char *shishi_krberror_errorcode_message (Shishi * handle,
+						      int errorcode);
+extern const char *shishi_krberror_message (Shishi * handle,
+					    Shishi_asn1 krberror);
 
 /* vasprintf.c */
 extern int shishi_asprintf (char **result, char *format, ...);
 
 /* gztime.c */
 extern const char *shishi_generalize_time (Shishi * handle, time_t t);
+extern const char *shishi_generalize_now (Shishi * handle);
 extern time_t shishi_generalize_ctime (Shishi * handle, const char *t);
 
 /* crypto.c */
@@ -930,42 +1249,84 @@ extern int shishi_random_to_key (Shishi * handle,
 				 int32_t keytype,
 				 char *random, size_t randomlen,
 				 Shishi_key * outkey);
+extern int shishi_encrypt_ivupdate_etype (Shishi * handle,
+					  Shishi_key * key,
+					  int keyusage,
+					  int32_t etype,
+					  const char *iv, size_t ivlen,
+					  char **ivout, size_t * ivoutlen,
+					  const char *in, size_t inlen,
+					  char **out, size_t * outlen);
+extern int shishi_encrypt_iv_etype (Shishi * handle,
+				    Shishi_key * key,
+				    int keyusage,
+				    int32_t etype,
+				    const char *iv, size_t ivlen,
+				    const char *in, size_t inlen,
+				    char **out, size_t * outlen);
+extern int shishi_encrypt_etype (Shishi * handle,
+				 Shishi_key * key,
+				 int keyusage,
+				 int32_t etype,
+				 const char *in, size_t inlen,
+				 char **out, size_t * outlen);
+extern int shishi_encrypt_ivupdate (Shishi * handle,
+				    Shishi_key * key,
+				    int keyusage,
+				    const char *iv, size_t ivlen,
+				    char **ivout, size_t * ivoutlen,
+				    const char *in, size_t inlen,
+				    char **out, size_t * outlen);
+extern int shishi_encrypt_iv (Shishi * handle,
+			      Shishi_key * key,
+			      int keyusage,
+			      const char *iv, size_t ivlen,
+			      const char *in, size_t inlen,
+			      char **out, size_t * outlen);
 extern int shishi_encrypt (Shishi * handle,
 			   Shishi_key * key,
 			   int keyusage,
 			   char *in, size_t inlen,
 			   char **out, size_t * outlen);
-extern int shishi_encrypt_iv (Shishi * handle,
-			      Shishi_key * key,
-			      int keyusage,
-			      char *iv, size_t ivlen,
-			      char *in, size_t inlen,
-			      char **out, size_t * outlen);
-extern int shishi_encrypt_iv_etype (Shishi * handle,
-				    Shishi_key * key,
-				    int keyusage,
-				    int32_t etype,
-				    char *iv, size_t ivlen,
-				    char *in, size_t inlen,
-				    char **out, size_t * outlen);
-extern int shishi_decrypt (Shishi * handle,
-			   Shishi_key * key,
-			   int keyusage,
-			   char *in, size_t inlen,
-			   char **out, size_t * outlen);
-extern int shishi_decrypt_iv (Shishi * handle,
-			      Shishi_key * key,
-			      int keyusage,
-			      char *iv, size_t ivlen,
-			      char *in, size_t inlen,
-			      char **out, size_t * outlen);
+extern int shishi_decrypt_ivupdate_etype (Shishi * handle,
+					  Shishi_key * key,
+					  int keyusage,
+					  int32_t etype,
+					  const char *iv, size_t ivlen,
+					  char **ivout, size_t * ivoutlen,
+					  const char *in, size_t inlen,
+					  char **out, size_t * outlen);
 extern int shishi_decrypt_iv_etype (Shishi * handle,
 				    Shishi_key * key,
 				    int keyusage,
 				    int32_t etype,
-				    char *iv, size_t ivlen,
-				    char *in, size_t inlen,
+				    const char *iv, size_t ivlen,
+				    const char *in, size_t inlen,
 				    char **out, size_t * outlen);
+extern int shishi_decrypt_etype (Shishi * handle,
+				 Shishi_key * key,
+				 int keyusage,
+				 int32_t etype,
+				 const char *in, size_t inlen,
+				 char **out, size_t * outlen);
+extern int shishi_decrypt_ivupdate (Shishi * handle,
+				    Shishi_key * key,
+				    int keyusage,
+				    const char *iv, size_t ivlen,
+				    char **ivout, size_t * ivoutlen,
+				    const char *in, size_t inlen,
+				    char **out, size_t * outlen);
+extern int shishi_decrypt_iv (Shishi * handle,
+			      Shishi_key * key,
+			      int keyusage,
+			      const char *iv, size_t ivlen,
+			      const char *in, size_t inlen,
+			      char **out, size_t * outlen);
+extern int shishi_decrypt (Shishi * handle,
+			   Shishi_key * key,
+			   int keyusage,
+			   const char *in, size_t inlen,
+			   char **out, size_t * outlen);
 extern int shishi_checksum (Shishi * handle,
 			    Shishi_key * key,
 			    int keyusage,
@@ -985,9 +1346,7 @@ extern int shishi_n_fold (Shishi * handle, char *in, size_t inlen,
 			  char *out, size_t outlen);
 extern int shishi_pbkdf2_sha1 (const char *P, size_t Plen,
 			       const char *S, size_t Slen,
-			       unsigned int c,
-			       unsigned int dkLen,
-			       char *DK);
+			       unsigned int c, unsigned int dkLen, char *DK);
 
 /* version.c */
 extern const char *shishi_check_version (const char *req_version);
@@ -1009,9 +1368,9 @@ extern int shishi_a2d_field (Shishi * handle,
 extern int shishi_a2d (Shishi * handle, Shishi_asn1 node,
 		       char *der, int *len);
 extern int shishi_a2d_new_field (Shishi * handle, Shishi_asn1 node,
-				 const char *field, char **der, int *len);
+				 const char *field, char **der, size_t *len);
 extern int shishi_new_a2d (Shishi * handle, Shishi_asn1 node,
-			   char **der, int *len);
+			   char **der, size_t *len);
 extern int shishi_asn1_done (Shishi * handle, Shishi_asn1 node);
 extern int shishi_asn1_read (Shishi * handle, Shishi_asn1 node,
 			     const char *field, char *data, size_t * datalen);
@@ -1109,6 +1468,7 @@ extern const char *shishi_ap_option2string (int option);
 extern int shishi_ap_string2option (const char *str);
 
 extern int shishi_ap (Shishi * handle, Shishi_ap ** ap);
+extern int shishi_ap_nosubkey (Shishi * handle, Shishi_ap ** ap);
 extern int shishi_ap_set_tktoptions (Shishi_ap * ap,
 				     Shishi_tkt * tkt, int options);
 extern int shishi_ap_tktoptions (Shishi * handle,
@@ -1153,17 +1513,20 @@ extern void shishi_ap_authenticator_set (Shishi_ap * ap,
 
 extern Shishi_asn1 shishi_ap_req (Shishi_ap * ap);
 extern void shishi_ap_req_set (Shishi_ap * ap, Shishi_asn1 apreq);
-extern int shishi_ap_req_der (Shishi_ap * ap, char *out, int *outlen);
+extern int shishi_ap_req_der (Shishi_ap * ap, char **out, size_t *outlen);
 extern int shishi_ap_req_der_set (Shishi_ap * ap, char *der, size_t derlen);
-extern int shishi_ap_req_der_new (Shishi_ap * ap, char **out, int *outlen);
 extern int shishi_ap_req_build (Shishi_ap * ap);
 extern int shishi_ap_req_asn1 (Shishi_ap * ap, Shishi_asn1 * apreq);
+extern Shishi_key *shishi_ap_key (Shishi_ap * ap);
 extern int shishi_ap_req_process (Shishi_ap * ap, Shishi_key * key);
+extern int shishi_ap_req_process_keyusage (Shishi_ap * ap,
+					   Shishi_key * key,
+					   int32_t keyusage);
 extern int shishi_ap_req_build (Shishi_ap * ap);
 
 extern Shishi_asn1 shishi_ap_rep (Shishi_ap * ap);
 extern void shishi_ap_rep_set (Shishi_ap * ap, Shishi_asn1 aprep);
-extern int shishi_ap_rep_der (Shishi_ap * ap, char *out, size_t * outlen);
+extern int shishi_ap_rep_der (Shishi_ap * ap, char **out, size_t * outlen);
 extern int shishi_ap_rep_der_set (Shishi_ap * ap, char *der, size_t derlen);
 extern int shishi_ap_rep_verify (Shishi_ap * ap);
 extern int shishi_ap_rep_verify_der (Shishi_ap * ap, char *der,
@@ -1219,20 +1582,20 @@ extern int shishi_key_from_string (Shishi * handle,
 				   Shishi_key ** outkey);
 
 /* keys.c */
-extern Shishi_key *
-shishi_keys_for_serverrealm_in_file (Shishi * handle,
-				     const char *filename,
-				     const char *server,
-				     const char *realm);
-extern Shishi_key *
-shishi_keys_for_server_in_file (Shishi * handle,
-				const char *filename,
-				const char *server);
-extern Shishi_key *
-shishi_keys_for_localservicerealm_in_file (Shishi * handle,
-					   const char *filename,
-					   const char *service,
-					   const char *realm);
+extern Shishi_key *shishi_keys_for_serverrealm_in_file (Shishi * handle,
+							const char *filename,
+							const char *server,
+							const char *realm);
+extern Shishi_key *shishi_keys_for_server_in_file (Shishi * handle,
+						   const char *filename,
+						   const char *server);
+extern Shishi_key *shishi_keys_for_localservicerealm_in_file (Shishi * handle,
+							      const char
+							      *filename,
+							      const char
+							      *service,
+							      const char
+							      *realm);
 
 /* hostkeys.c */
 extern const char *shishi_hostkeys_default_file (Shishi * handle);
@@ -1287,7 +1650,7 @@ extern int shishi_encapreppart_get_key (Shishi * handle,
 					size_t * keyvalue_len);
 extern int shishi_encapreppart_seqnumber_get (Shishi * handle,
 					      Shishi_asn1 encapreppart,
-					      uint32_t *seqnumber);
+					      uint32_t * seqnumber);
 
 /* apreq.c */
 extern Shishi_asn1 shishi_apreq (Shishi * handle);
@@ -1301,10 +1664,6 @@ extern int shishi_apreq_to_file (Shishi * handle, Shishi_asn1 apreq,
 extern int shishi_apreq_read (Shishi * handle, FILE * fh,
 			      Shishi_asn1 * apreq);
 extern int shishi_apreq_save (Shishi * handle, FILE * fh, Shishi_asn1 apreq);
-extern int shishi_apreq_options (Shishi * handle, Shishi_asn1 apreq,
-				 int *flags);
-extern int shishi_apreq_options_set (Shishi * handle, Shishi_asn1 apreq,
-				     int options);
 extern int shishi_apreq_set_ticket (Shishi * handle, Shishi_asn1 apreq,
 				    Shishi_asn1 ticket);
 extern int shishi_apreq_set_authenticator (Shishi * handle, Shishi_asn1 apreq,
@@ -1323,6 +1682,8 @@ extern int shishi_apreq_options_set (Shishi * handle, Shishi_asn1 apreq,
 				     int options);
 extern int shishi_apreq_options_add (Shishi * handle, Shishi_asn1 apreq,
 				     int option);
+extern int shishi_apreq_options_remove (Shishi * handle, Shishi_asn1 apreq,
+					int option);
 extern int shishi_apreq_get_ticket (Shishi * handle, Shishi_asn1 apreq,
 				    Shishi_asn1 * ticket);
 extern int shishi_apreq_get_authenticator_etype (Shishi * handle,
@@ -1439,7 +1800,8 @@ extern Shishi_key *shishi_safe_key (Shishi_safe * safe);
 extern void shishi_safe_key_set (Shishi_safe * safe, Shishi_key * key);
 extern Shishi_asn1 shishi_safe_safe (Shishi_safe * safe);
 extern void shishi_safe_safe_set (Shishi_safe * safe, Shishi_asn1 asn1safe);
-extern int shishi_safe_safe_der (Shishi_safe * safe, char *out, int *outlen);
+extern int shishi_safe_safe_der (Shishi_safe * safe, char **out,
+				 size_t *outlen);
 extern int shishi_safe_safe_der_set (Shishi_safe * safe,
 				     char *der, size_t derlen);
 extern int shishi_safe_print (Shishi * handle, FILE * fh, Shishi_asn1 safe);
@@ -1453,14 +1815,15 @@ extern int shishi_safe_from_file (Shishi * handle, Shishi_asn1 * safe,
 extern int shishi_safe_cksum (Shishi * handle,
 			      Shishi_asn1 safe,
 			      int32_t * cksumtype,
-			      char *cksum, size_t * cksumlen);
+			      char **cksum, size_t * cksumlen);
 extern int shishi_safe_set_cksum (Shishi * handle,
 				  Shishi_asn1 safe,
 				  int32_t cksumtype,
 				  char *cksum, size_t cksumlen);
 extern int shishi_safe_user_data (Shishi * handle,
 				  Shishi_asn1 safe,
-				  char *userdata, size_t * userdatalen);
+				  char **userdata,
+				  size_t * userdatalen);
 extern int shishi_safe_set_user_data (Shishi * handle,
 				      Shishi_asn1 safe,
 				      char *userdata, size_t userdatalen);
