@@ -111,6 +111,7 @@
 #endif
 
 #include "getdate.h"
+#include "xalloc.h"
 #include "error.h"
 
 #include <argp.h>
@@ -123,53 +124,53 @@
 
 /* Long parameters only */
 enum
-  {
-    OPTION_REQUEST = 300,
-    OPTION_SENDRECV,
-    OPTION_RESPONSE,
-    OPTION_WRITE_AP_REQUEST_FILE,
-    OPTION_WRITE_AUTHENTICATOR_FILE,
-    OPTION_WRITE_REQUEST_FILE,
-    OPTION_WRITE_RESPONSE_FILE,
-    OPTION_READ_REQUEST_FILE,
-    OPTION_READ_RESPONSE_FILE,
-    OPTION_SERVER,
-    OPTION_CLIENT,
-    OPTION_CLIENT_NAME,
-    OPTION_REALM,
-    OPTION_SERVER_NAME,
-    OPTION_TICKET_GRANTER,
-    OPTION_FORCE_AS,
-    OPTION_FORCE_TGS,
-    OPTION_CRYPTO_ENCRYPT,
-    OPTION_CRYPTO_DECRYPT,
-    OPTION_CRYPTO_ALGORITHM,
-    OPTION_CRYPTO_KEY_VERSION,
-    OPTION_CRYPTO_KEY_USAGE,
-    OPTION_CRYPTO_KEY_VALUE,
-    OPTION_CRYPTO_READ_KEY_FILE,
-    OPTION_CRYPTO_WRITE_KEY_FILE,
-    OPTION_CRYPTO_READ_DATA_FILE,
-    OPTION_CRYPTO_WRITE_DATA_FILE,
-    OPTION_CRYPTO_RANDOM,
-    OPTION_CRYPTO_PARAMETER,
-    OPTION_CRYPTO_PASSWORD,
-    OPTION_CRYPTO_SALT,
-    OPTION_CRYPTO_DEBUG,
-    OPTION_CRYPTO_GENERATE_KEY,
-    OPTION_VERBOSE_LIBRARY,
-    OPTION_LIST,
-    OPTION_DESTROY,
-    OPTION_CRYPTO,
-    OPTION_RENEW,
-    OPTION_RENEWABLE,
-    OPTION_STARTTIME,
-    OPTION_ENDTIME,
-    OPTION_RENEW_TILL,
-    OPTION_CFG_SYSTEM,
-    OPTION_CFG_USER,
-    OPTION_WRITE_TICKET_FILE
-  };
+{
+  OPTION_REQUEST = 300,
+  OPTION_SENDRECV,
+  OPTION_RESPONSE,
+  OPTION_WRITE_AP_REQUEST_FILE,
+  OPTION_WRITE_AUTHENTICATOR_FILE,
+  OPTION_WRITE_REQUEST_FILE,
+  OPTION_WRITE_RESPONSE_FILE,
+  OPTION_READ_REQUEST_FILE,
+  OPTION_READ_RESPONSE_FILE,
+  OPTION_SERVER,
+  OPTION_CLIENT,
+  OPTION_CLIENT_NAME,
+  OPTION_REALM,
+  OPTION_SERVER_NAME,
+  OPTION_TICKET_GRANTER,
+  OPTION_FORCE_AS,
+  OPTION_FORCE_TGS,
+  OPTION_CRYPTO_ENCRYPT,
+  OPTION_CRYPTO_DECRYPT,
+  OPTION_CRYPTO_KEY_VERSION,
+  OPTION_CRYPTO_KEY_USAGE,
+  OPTION_CRYPTO_KEY_VALUE,
+  OPTION_CRYPTO_READ_KEY_FILE,
+  OPTION_CRYPTO_WRITE_KEY_FILE,
+  OPTION_CRYPTO_READ_DATA_FILE,
+  OPTION_CRYPTO_WRITE_DATA_FILE,
+  OPTION_CRYPTO_RANDOM,
+  OPTION_CRYPTO_PARAMETER,
+  OPTION_CRYPTO_PASSWORD,
+  OPTION_CRYPTO_SALT,
+  OPTION_CRYPTO_STR2KEY,
+  OPTION_CRYPTO_DEBUG,
+  OPTION_CRYPTO_GENERATE_KEY,
+  OPTION_CRYPTO,
+  OPTION_VERBOSE_LIBRARY,
+  OPTION_LIST,
+  OPTION_DESTROY,
+  OPTION_RENEW,
+  OPTION_RENEWABLE,
+  OPTION_STARTTIME,
+  OPTION_ENDTIME,
+  OPTION_RENEW_TILL,
+  OPTION_CFG_SYSTEM,
+  OPTION_CFG_USER,
+  OPTION_WRITE_TICKET_FILE
+};
 
 #define TYPE_TEXT_NAME "text"
 #define TYPE_DER_NAME "der"
@@ -185,12 +186,14 @@ struct arguments
   int command;
   char *ticketfile;
   char *ticketwritefile;
-  const char *realm;
   char *systemcfgfile;
   char *usercfgfile;
   const char *client;
+  const char *crealm;
   const char *cname;
   const char *sname;
+  const char *srealm;
+  const char *server;
   char *tgtname;
   int forceas_p;
   int forcetgs_p;
@@ -233,16 +236,24 @@ crypto (Shishi * handle, struct arguments arg)
   if (arg.cname == NULL)
     arg.cname = shishi_principal_default (handle);
 
-  if (arg.realm == NULL)
-    arg.realm = shishi_realm_default (handle);
+  if (arg.crealm == NULL)
+    arg.crealm = shishi_realm_default (handle);
 
   if (arg.salt == NULL)
     {
-      arg.salt = malloc (strlen (arg.realm) + strlen (arg.cname) + 1);
-      if (!arg.salt)
-	return SHISHI_MALLOC_ERROR;
-      strcpy (arg.salt, arg.realm);
-      strcat (arg.salt, arg.cname);
+      char *cname, *tok, *tokptr;
+
+      cname = xstrdup (arg.cname);
+      arg.salt = xstrdup (arg.crealm);
+      tok = strtok_r (cname, "/", &tokptr);
+      while (tok)
+	{
+	  arg.salt =
+	    xrealloc (arg.salt, strlen (arg.salt) + strlen (tok) + 1);
+	  strcat (arg.salt, tok);
+	  tok = strtok_r (NULL, "/", &tokptr);
+	}
+      free (cname);
     }
 
   rc = shishi_key (handle, &key);
@@ -256,7 +267,7 @@ crypto (Shishi * handle, struct arguments arg)
   shishi_key_type_set (key, arg.algorithm);
   shishi_key_version_set (key, arg.kvno);
   shishi_key_principal_set (key, arg.cname);
-  shishi_key_realm_set (key, arg.realm);
+  shishi_key_realm_set (key, arg.crealm);
 
   if (arg.password)
     {
@@ -317,11 +328,6 @@ crypto (Shishi * handle, struct arguments arg)
       return SHISHI_OK;
     }
 
-  if (shishi_key_type (key) == SHISHI_NULL && !arg.silent)
-    fprintf (stderr,
-	     "warning: using %s is silly, consider using --algorithm.\n",
-	     shishi_cipher_name (arg.algorithm));
-
   if (arg.verbose ||
       ((arg.password || arg.random || arg.keyvalue) &&
        !(arg.encrypt_p || arg.decrypt_p)))
@@ -330,100 +336,99 @@ crypto (Shishi * handle, struct arguments arg)
     }
 
 #if 0
-  currently broken
-    if (arg.encrypt_p || arg.decrypt_p)
-      {
-	if (arg.inputfile)
-	  {
-	    infh = fopen (arg.inputfile, "r");
-	    if (infh == NULL)
-	      {
-		shishi_error_printf (handle, _("`%s': %s\n"),
-				     arg.inputfile, strerror (errno));
-		return SHISHI_FOPEN_ERROR;
-	      }
-	  }
-	else
-	  infh = stdin;
+  currently broken if (arg.encrypt_p || arg.decrypt_p)
+    {
+      if (arg.inputfile)
+	{
+	  infh = fopen (arg.inputfile, "r");
+	  if (infh == NULL)
+	    {
+	      shishi_error_printf (handle, _("`%s': %s\n"),
+				   arg.inputfile, strerror (errno));
+	      return SHISHI_FOPEN_ERROR;
+	    }
+	}
+      else
+	infh = stdin;
 
-	if (arg.outputfile)
-	  {
-	    outfh = fopen (arg.outputfile, "w");
-	    if (outfh == NULL)
-	      {
-		shishi_error_printf (handle, _("`%s': %s\n"),
-				     arg.inputfile, strerror (errno));
-		return SHISHI_FOPEN_ERROR;
-	      }
-	  }
-	else
-	  outfh = stdout;
+      if (arg.outputfile)
+	{
+	  outfh = fopen (arg.outputfile, "w");
+	  if (outfh == NULL)
+	    {
+	      shishi_error_printf (handle, _("`%s': %s\n"),
+				   arg.inputfile, strerror (errno));
+	      return SHISHI_FOPEN_ERROR;
+	    }
+	}
+      else
+	outfh = stdout;
 
-	outlen = fread (out, sizeof (out[0]),
-			sizeof (out) / sizeof (out[0]), infh);
-	if (outlen == 0)
-	  {
-	    fprintf (stderr, _("Error reading `%s'\n"), arg.inputfile);
-	    return !SHISHI_OK;
-	  }
-	if (arg.verbose)
-	  printf (_("Read %d bytes...\n"), outlen);
+      outlen = fread (out, sizeof (out[0]),
+		      sizeof (out) / sizeof (out[0]), infh);
+      if (outlen == 0)
+	{
+	  fprintf (stderr, _("Error reading `%s'\n"), arg.inputfile);
+	  return !SHISHI_OK;
+	}
+      if (arg.verbose)
+	printf (_("Read %d bytes...\n"), outlen);
 
-	if (arg.encrypt_p)
-	  rc = shishi_encrypt (handle, key, arg.keyusage,
-			       out, outlen, &in, &inlen);
-	else
-	  rc = shishi_decrypt (handle, key, arg.keyusage,
-			       in, inlen, &out, &outlen);
-	if (rc != SHISHI_OK)
-	  {
-	    shishi_error_printf (handle, _("Error ciphering\n"));
-	    return rc;
-	  }
+      if (arg.encrypt_p)
+	rc = shishi_encrypt (handle, key, arg.keyusage,
+			     out, outlen, &in, &inlen);
+      else
+	rc = shishi_decrypt (handle, key, arg.keyusage,
+			     in, inlen, &out, &outlen);
+      if (rc != SHISHI_OK)
+	{
+	  shishi_error_printf (handle, _("Error ciphering\n"));
+	  return rc;
+	}
 
-	if (arg.outputtype == SHISHI_FILETYPE_HEX)
-	  {
-	    for (i = 0; i < inlen; i++)
-	      {
-		if ((i % 16) == 0)
-		  fprintf (outfh, "\n");
-		fprintf (outfh, "%02x ", in[i]);
-	      }
-	    fprintf (outfh, "\n");
-	  }
-	else if (arg.outputtype == SHISHI_FILETYPE_BINARY)
-	  {
-	    i = fwrite (in, sizeof (in[0]), inlen, outfh);
-	    if (i != inlen)
-	      {
-		fprintf (stderr, _("Short write (%d < %d)...\n"), i, inlen);
-		return 1;
-	      }
-	    printf (_("Wrote %d bytes...\n"), inlen);
-	  }
+      if (arg.outputtype == SHISHI_FILETYPE_HEX)
+	{
+	  for (i = 0; i < inlen; i++)
+	    {
+	      if ((i % 16) == 0)
+		fprintf (outfh, "\n");
+	      fprintf (outfh, "%02x ", in[i]);
+	    }
+	  fprintf (outfh, "\n");
+	}
+      else if (arg.outputtype == SHISHI_FILETYPE_BINARY)
+	{
+	  i = fwrite (in, sizeof (in[0]), inlen, outfh);
+	  if (i != inlen)
+	    {
+	      fprintf (stderr, _("Short write (%d < %d)...\n"), i, inlen);
+	      return 1;
+	    }
+	  printf (_("Wrote %d bytes...\n"), inlen);
+	}
 
-	if (arg.outputfile)
-	  {
-	    rc = fclose (outfh);
-	    if (rc != 0)
-	      {
-		shishi_error_printf (handle, _("`%s': %s\n"),
-				     arg.outputfile, strerror (errno));
-		return SHISHI_FCLOSE_ERROR;
-	      }
-	  }
+      if (arg.outputfile)
+	{
+	  rc = fclose (outfh);
+	  if (rc != 0)
+	    {
+	      shishi_error_printf (handle, _("`%s': %s\n"),
+				   arg.outputfile, strerror (errno));
+	      return SHISHI_FCLOSE_ERROR;
+	    }
+	}
 
-	if (arg.inputfile)
-	  {
-	    rc = fclose (infh);
-	    if (rc != 0)
-	      {
-		shishi_error_printf (handle, _("`%s': %s\n"),
-				     arg.inputfile, strerror (errno));
-		return SHISHI_FCLOSE_ERROR;
-	      }
-	  }
-      }
+      if (arg.inputfile)
+	{
+	  rc = fclose (infh);
+	  if (rc != 0)
+	    {
+	      shishi_error_printf (handle, _("`%s': %s\n"),
+				   arg.inputfile, strerror (errno));
+	      return SHISHI_FCLOSE_ERROR;
+	    }
+	}
+    }
 #endif
 
   if (arg.writekeyfile)
@@ -514,29 +519,15 @@ parse_opt (int key, char *arg, struct argp_state *state)
       arguments->ticketfile = strdup (arg);
       break;
 
-    case OPTION_CRYPTO_ALGORITHM:
-      if (arguments->command != OPTION_CRYPTO)
-	argp_error (state, _("Option `%s' only valid with CRYPTO."),
-		    state->argv[state->next - 1]);
-      arguments->algorithm = shishi_cipher_parse (arg);
-      if (arguments->algorithm == -1)
-	argp_error (state, _("Unknown encryption type in `%s'"),
-		    state->argv[state->next - 1]);
-      break;
-
     case OPTION_CRYPTO_ENCRYPT:
-      if (arguments->command != OPTION_CRYPTO)
-	argp_error (state, _("Option `%s' only valid with CRYPTO."),
-		    state->argv[state->next - 1]);
+      arguments->command = OPTION_CRYPTO;
       if (arguments->decrypt_p)
 	argp_error (state, _("Cannot both encrypt and decrypt."));
       arguments->encrypt_p = 1;
       break;
 
     case OPTION_CRYPTO_DECRYPT:
-      if (arguments->command != OPTION_CRYPTO)
-	argp_error (state, _("Option `%s' only valid with CRYPTO."),
-		    state->argv[state->next - 1]);
+      arguments->command = OPTION_CRYPTO;
       if (arguments->encrypt_p)
 	argp_error (state, _("Cannot both encrypt and decrypt."));
       arguments->decrypt_p = 1;
@@ -602,6 +593,16 @@ parse_opt (int key, char *arg, struct argp_state *state)
       arguments->salt = strdup (arg);
       break;
 
+    case OPTION_CRYPTO_STR2KEY:
+      arguments->command = OPTION_CRYPTO;
+      if (arg)
+	{
+	  if (arguments->password)
+	    argp_error (state, _("Password specified twice."));
+	  arguments->password = strdup (arg);
+	}
+      break;
+
     case OPTION_CRYPTO_WRITE_DATA_FILE:
       if (arguments->command != OPTION_CRYPTO)
 	argp_error (state, _("Option `%s' only valid with CRYPTO."),
@@ -629,7 +630,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
 
     case OPTION_REALM:
-      arguments->realm = strdup (arg);
+      arguments->crealm = strdup (arg);
       break;
 
     case 'R':
@@ -668,10 +669,6 @@ parse_opt (int key, char *arg, struct argp_state *state)
       arguments->tgtname = strdup (arg);
       break;
 
-    case OPTION_CRYPTO:
-      arguments->command = OPTION_CRYPTO;
-      break;
-
     case 'l':
     case OPTION_LIST:
       arguments->command = OPTION_LIST;
@@ -683,10 +680,12 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
 
     case ARGP_KEY_ARG:
-      if (arguments->client)
+      if (arguments->server && arguments->client)
 	argp_error (state, _("Too many arguments: `%s'"), arg);
+      if (arguments->client)
+	arguments->server = strdup (arg);
       else
-	arguments->client = arg;
+	arguments->client = strdup (arg);
       break;
 
     default:
@@ -699,163 +698,165 @@ parse_opt (int key, char *arg, struct argp_state *state)
 static struct argp_option options[] = {
 
   {0, 0, 0, 0, "If no command is given, Shishi try to make sure you have a "
-   "ticket granting ticket for the default realm, and then display it."},
+   "ticket granting ticket for the default realm, and then display it.", 0},
 
   {"client-name", OPTION_CLIENT_NAME, "NAME", 0,
-   "Client name. Default is login username. Only for AS.", 10},
+   "Client name. Default is login username.", 10},
 
   {"destroy", 'd', 0, 0,
-   "Destroy tickets in local cache, subject to --server-name limiting."},
+   "Destroy tickets in local cache, subject to --client-name and "
+   "--server-name limiting.", 0},
 
   {"encryption-type", 'E', "ETYPE,[ETYPE...]", 0,
-   "Encryption types to use.  ETYPE is either registered name or integer."},
+   "Encryption types to use.  ETYPE is either registered name or integer.",
+   0},
 
   {"force-as", OPTION_FORCE_AS, 0, 0,
-   "Force AS mode. Default is to use TGS iff a TGT is found."},
+   "Force AS mode. Default is to use TGS iff a TGT is found.", 0},
 
   {"force-tgs", OPTION_FORCE_TGS, 0, 0,
-   "Force TGS mode. Default is to use TGS iff a TGT is found."},
+   "Force TGS mode. Default is to use TGS iff a TGT is found.", 0},
 
   {"endtime", 'e', "STRING", 0,
    "Specify when ticket validity should expire.  The time syntax may be "
    "relative (to the start time), such as \"20 hours\", or absolute, "
    "such as \"2001-02-03 04:05:06 CET\". The default is 8 hours after "
-   "the start time."},
+   "the start time.", 0},
 
   {"list", 'l', 0, 0,
-   "List tickets in local cache, subject to --server-name limiting."},
+   "List tickets in local cache, subject to --server-name limiting.", 0},
 
   {"renew", 'R', 0, 0,
    "Renew ticket.  Use --server-name to specify ticket, default is the "
-   "most recent renewable ticket granting ticket for the default realm."},
+   "most recent renewable ticket granting ticket for the default realm.", 0},
 
   {"renewable", OPTION_RENEWABLE, 0, 0,
-   "Get a renewable ticket."},
+   "Get a renewable ticket.", 0},
 
   {"renew-till", OPTION_RENEW_TILL, "STRING", 0,
    "Specify renewable life of ticket.  Implies --renewable.  Accepts same "
    "time syntax as --endtime.  If --renewable is specified, the default is 1 "
-   "week after the start time."},
+   "week after the start time.", 0},
 
   {"realm", OPTION_REALM, "REALM", 0,
    "Realm of server. Default is DNS domain of local host. For AS, this also "
-   "indicates realm of client."},
+   "indicates realm of client.", 0},
 
   {"server", OPTION_SERVER, "[FAMILY:]ADDRESS:SERVICE/TYPE", 0,
    "Send all requests to HOST instead of using normal logic to locate "
-   "KDC addresses (discouraged)."},
+   "KDC addresses (discouraged).", 0},
 
   {"server-name", OPTION_SERVER_NAME, "NAME", 0,
    "Server name. Default is \"krbtgt/REALM\" where REALM is server "
-   "realm (see --realm)."},
+   "realm (see --realm).", 0},
 
   {"starttime", 's', "STRING", 0,
    "Specify when ticket should start to be valid.  Accepts same time syntax "
-   "as --endtime. The default is to become valid immediately."},
+   "as --endtime. The default is to become valid immediately.", 0},
 
   {"ticket-granter", OPTION_TICKET_GRANTER, "NAME", 0,
    "Service name in ticket to use for authenticating request. Only for TGS. "
    "Defaults to \"krbtgt/REALM@REALM\" where REALM is server "
-   "realm (see --realm)."},
-
+   "realm (see --realm).", 0},
+#if 0
   {"key-value", OPTION_CRYPTO_KEY_VALUE, "KEY", 0,
-   "Cipher key to decrypt response (discouraged)."},
-
-  {"password", OPTION_CRYPTO_PASSWORD, "PASSWORD", 0,
-   "Password to decrypt response (discouraged).  Only for AS."},
+   "Cipher key to decrypt response (discouraged).", 0},
+#endif
 
   /************** CRYPTO */
 
   {0, 0, 0, 0,
    "Options for low-level cryptography (CRYPTO-OPTIONS):", 100},
 
-  {"algorithm", OPTION_CRYPTO_ALGORITHM, "ALGORITHM", 0,
-   "Cipher algorithm, expressed either as the etype integer or "
-   "the registered name."},
-
   {"client-name", OPTION_CLIENT_NAME, "NAME", 0,
-   "Username. Default is login name."},
-
+   "Username. Default is login name.", 0},
+#if 0
   {"decrypt", OPTION_CRYPTO_DECRYPT, 0, 0,
-   "Decrypt data."},
+   "Decrypt data.", 0},
 
   {"encrypt", OPTION_CRYPTO_ENCRYPT, 0, 0,
-   "Encrypt data."},
+   "Encrypt data.", 0},
 
   {"key-usage", OPTION_CRYPTO_KEY_USAGE, "KEYUSAGE", 0,
    "Encrypt or decrypt using specified key usage.  Default is 0, which "
-   "means no key derivation are performed."},
+   "means no key derivation are performed.", 0},
 
   {"key-value", OPTION_CRYPTO_KEY_VALUE, "KEY", 0,
-   "Base64 encoded key value."},
-
+   "Base64 encoded key value.", 0},
+#endif
   {"key-version", OPTION_CRYPTO_KEY_VERSION, "INTEGER", 0,
-   "Version number of key."},
-
-  {"password", OPTION_CRYPTO_PASSWORD, "PASSWORD", 0,
-   "Password used to generate key.  --client-name and --realm also modify "
-   "the computed key value."},
+   "Version number of key. Default is 0.", 0},
 
   {"random", OPTION_CRYPTO_RANDOM, 0, 0,
-   "Generate key from random data."},
-
+   "Generate key from random data.", 0},
+#if 0
   {"read-key-file", OPTION_CRYPTO_READ_KEY_FILE, "FILE", 0,
-   "Read cipher key from FILE"},
+   "Read cipher key from FILE", 0},
 
   {"read-data-file", OPTION_CRYPTO_READ_DATA_FILE, "[TYPE,]FILE", 0,
-   "Read data from FILE in TYPE, BASE64, HEX or BINARY (default)."},
-
+   "Read data from FILE in TYPE, BASE64, HEX or BINARY (default).", 0},
+#endif
   {"realm", OPTION_REALM, "REALM", 0,
-   "Realm of principal. Defaults to DNS domain of local host. "},
+   "Realm of principal. Defaults to DNS domain of local host. ", 0},
 
   {"salt", OPTION_CRYPTO_SALT, "SALT", 0,
-   "Salt to use when --password is specified. Defaults to using the"
-   "username (--client-name) and realm (--realm)."},
+   "Salt to use for --string-to-key. Defaults to concatenation of "
+   "realm and (unwrapped) client name.", 0},
+
+  {"string-to-key", OPTION_CRYPTO_STR2KEY, "[PASSWORD]", OPTION_ARG_OPTIONAL,
+   "Convert password into Kerberos key.  Note that --client-name, --realm, "
+   "and --salt influence the generated key.", 0},
 
   {"parameter", OPTION_CRYPTO_PARAMETER, "STRING", 0,
-   "String-to-key parameter to use when --password is specified. This data "
-   "is specific for each encryption algorithm and rarely needed."},
-
+   "String-to-key parameter. This data is specific for each encryption "
+   "algorithm and rarely needed.", 0},
+#if 0
   {"write-key-file", OPTION_CRYPTO_WRITE_KEY_FILE, "FILE", 0,
-   "Append cipher key to FILE"},
+   "Append cipher key to FILE", 0},
 
   {"write-data-file", OPTION_CRYPTO_WRITE_DATA_FILE, "[TYPE,]FILE", 0,
-   "Write data to FILE in TYPE, BASE64, HEX or BINARY (default)."},
-
+   "Write data to FILE in TYPE, BASE64, HEX or BINARY (default).", 0},
+#endif
   /************** OTHER */
 
   {0, 0, 0, 0, "Other options:", 200},
 
   {"verbose", 'v', 0, 0,
-   "Produce verbose output.",},
+   "Produce verbose output.", 0},
 
   {"verbose-library", OPTION_VERBOSE_LIBRARY, 0, 0,
-   "Produce verbose output in the library.",},
+   "Produce verbose output in the library.", 0},
 
   {"quiet", 'q', 0, 0,
-   "Don't produce any output."},
+   "Don't produce any output.", 0},
 
-  {"silent", 0, 0, OPTION_ALIAS},
+  {"silent", 0, 0, OPTION_ALIAS,
+   NULL, 0},
 
   {"system-configuration-file", OPTION_CFG_SYSTEM, "FILE", 0,
    "Read system wide configuration from file.  Default is " SYSTEMCFGFILE
-   "."},
+   ".", 0},
 
   {"configuration-file", OPTION_CFG_USER, "FILE", 0,
-   "Read user configuration from file.  Default is ~/.shishi/config."},
+   "Read user configuration from file.  Default is ~/.shishi/config.", 0},
 
   {"library-options", 'o', "STRING", 0,
-   "Parse STRING as a configuration file statement."},
+   "Parse STRING as a configuration file statement.", 0},
 
   {"ticket-file", 'c', "FILE", 0,
-   "Read tickets from FILE. Default is $HOME/.shishi/tickets."},
+   "Read tickets from FILE. Default is $HOME/.shishi/tickets.", 0},
 
   {"ticket-write-file", OPTION_WRITE_TICKET_FILE, "FILE", 0,
-   "Write tickets to FILE.  Default is to write them back to ticket file."},
+   "Write tickets to FILE.  Default is to write them back to ticket file.",
+   0},
 
-  {"NAME", 0, 0, OPTION_DOC | OPTION_NO_USAGE,
-   "Set client name and realm from NAME.  The --client-name and --realm can "
-   "be used to override part of NAME."},
+  {"CLIENT", 0, 0, OPTION_DOC | OPTION_NO_USAGE,
+   "Set client name and realm from NAME.  The --client-name and --realm "
+   "parameters can be used to override part of NAME.", 0},
+
+  {"SERVER", 0, 0, OPTION_DOC | OPTION_NO_USAGE,
+   "Set server name and realm from NAME.  The --server-name and "
+   "--server-realm parameters can be used to override part of SERVER.", 0},
 
   /************** EXAMPLES */
 
@@ -863,26 +864,30 @@ static struct argp_option options[] = {
 
   {"shishi", 0, 0, OPTION_DOC | OPTION_NO_USAGE,
    "Get a ticket granting ticket from the default KDC server for the "
-   "default user and realm."},
+   "default user and realm.", 0},
 
   {"shishi jas/admin@ACCOUNTING", 0, 0, OPTION_DOC | OPTION_NO_USAGE,
-   "Get a ticket for jas/admin in the ACCOUNTING realm."},
+   "Get a ticket for jas/admin in the ACCOUNTING realm.", 0},
 
   {"shishi --list --server-name=krbtgt/JOSEFSSON.ORG@JOSEFSSON.ORG",
    0, 0, OPTION_DOC | OPTION_NO_USAGE,
-   "List tickets for the Ticket Granting Service in the JOSEFSSON.ORG realm."},
+   "List tickets for the Ticket Granting Service in the JOSEFSSON.ORG realm.",
+   0},
 
-  {0}
+  {NULL, 0, 0, 0, NULL, 0}
 };
 
 static struct argp argp = {
   options,
   parse_opt,
-  "[NAME] [OPTION...]\n"
-  "--list [--server-name=NAME]\n"
-  "--destroy [--server-name=NAME]\n"
-  "--crypto [CRYPTO-OPTION...]\n",
-  "Shishi -- A Kerberos 5 implementation"
+  "[CLIENT [SERVER]] [OPTION...]\n"
+  "--list [CLIENT [SERVER]]\n"
+  "--destroy [CLIENT [SERVER]]\n"
+  "--string-to-key [CLIENT] [OPTION...]\n",
+  "Shishi -- A Kerberos 5 implementation",
+  NULL,
+  NULL,
+  NULL
 };
 
 int
@@ -891,12 +896,14 @@ main (int argc, char *argv[])
   struct arguments arg;
   Shishi *handle;
   int rc;
+  int32_t *etype;
 
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
   memset (&arg, 0, sizeof (arg));
+  arg.algorithm = -1;
   argp_parse (&argp, argc, argv, ARGP_IN_ORDER, 0, &arg);
 
   rc = shishi_init_with_paths (&handle, arg.ticketfile,
@@ -906,22 +913,38 @@ main (int argc, char *argv[])
 
   rc = shishi_cfg_clientkdcetype_set (handle, arg.etypes);
   if (rc != SHISHI_OK)
-    error (1, 0, "Could not set encryption types: %s\n", shishi_strerror (rc));
+    error (1, 0, "Could not set encryption types: %s\n",
+	   shishi_strerror (rc));
+
+  if (arg.algorithm == -1 && shishi_cfg_clientkdcetype (handle, &etype) > 0)
+    arg.algorithm = *etype;
 
   if (arg.client)
     {
       rc = shishi_parse_name (handle, arg.client,
 			      (char **) (arg.cname ? NULL : &arg.cname),
-			      (char **) (arg.realm ? NULL : &arg.realm));
+			      (char **) (arg.crealm ? NULL : &arg.crealm));
 
       if (rc != SHISHI_OK)
 	error (1, 0, "Could not parse principal \"%s\": %s\n", arg.client,
 	       shishi_strerror (rc));
     }
 
+  if (arg.server)
+    {
+      rc = shishi_parse_name (handle, arg.server,
+			      (char **) (arg.sname ? NULL : &arg.sname),
+			      (char **) (arg.srealm ? NULL : &arg.srealm));
+
+      if (rc != SHISHI_OK)
+	error (1, 0, "Could not parse principal \"%s\": %s\n", arg.server,
+	       shishi_strerror (rc));
+    }
+
   rc = shishi_cfg (handle, arg.lib_options);
   if (rc != SHISHI_OK)
-    error (1, 0, "Could not read library options: %s\n", shishi_strerror (rc));
+    error (1, 0, "Could not read library options: %s\n",
+	   shishi_strerror (rc));
 
   if (arg.verbose_library)
     {
@@ -932,7 +955,7 @@ main (int argc, char *argv[])
     }
 
   if (!arg.starttime)
-    arg.starttime = time(NULL);
+    arg.starttime = time (NULL);
 
   if (arg.endtime_str)
     {
@@ -951,8 +974,8 @@ main (int argc, char *argv[])
   if (arg.cname)
     shishi_principal_default_set (handle, arg.cname);
 
-  if (arg.realm)
-    shishi_realm_default_set (handle, arg.realm);
+  if (arg.crealm)
+    shishi_realm_default_set (handle, arg.crealm);
 
   if (!arg.tgtname)
     {
@@ -1045,7 +1068,8 @@ main (int argc, char *argv[])
 	    rc = shishi_tkt_pretty_print (tkt, stdout);
 	    if (rc != SHISHI_OK)
 	      fprintf (stderr, "Pretty printing ticket failed:\n%s\n%s\n",
-		       shishi_strerror (rc), shishi_strerror_details (handle));
+		       shishi_strerror (rc),
+		       shishi_strerror_details (handle));
 	  }
 
 	/* Get ticket using TGT ... */
@@ -1053,15 +1077,14 @@ main (int argc, char *argv[])
 	shishi_tgs_tgtkt_set (tgs, tkt);
 	if (rc == SHISHI_OK)
 	  rc = shishi_tgs_set_server (tgs, hint.server);
-	rc = shishi_kdcreq_options_add (handle, shishi_tgs_req(tgs),
-					SHISHI_KDCOPTIONS_RENEWABLE|
+	rc = shishi_kdcreq_options_add (handle, shishi_tgs_req (tgs),
+					SHISHI_KDCOPTIONS_RENEWABLE |
 					SHISHI_KDCOPTIONS_RENEW);
 	if (rc == SHISHI_OK)
-	  rc = shishi_asn1_write (handle, shishi_tgs_req(tgs),
+	  rc = shishi_asn1_write (handle, shishi_tgs_req (tgs),
 				  "req-body.rtime",
 				  shishi_generalize_time
-				  (handle, hint.renew_till),
-				  0);
+				  (handle, hint.renew_till), 0);
 	if (rc == SHISHI_OK)
 	  rc = shishi_tgs_req_build (tgs);
 	if (rc == SHISHI_OK)
@@ -1075,7 +1098,7 @@ main (int argc, char *argv[])
 	    if (rc == SHISHI_GOT_KRBERROR)
 	      shishi_krberror_pretty_print (handle, stdout,
 					    shishi_tgs_krberror (tgs));
-	    return NULL;
+	    break;
 	  }
 
 	tkt = shishi_tgs_tkt (tgs);
@@ -1083,7 +1106,7 @@ main (int argc, char *argv[])
 	  {
 	    printf ("No ticket in TGS-REP?!: %s\n",
 		    shishi_strerror_details (handle));
-	    return NULL;
+	    break;
 	  }
 
 	shishi_tkt_pretty_print (tkt, stdout);
@@ -1110,8 +1133,7 @@ main (int argc, char *argv[])
 	tkt = shishi_tkts_get (shishi_tkts_default (handle), &hint);
 	if (!tkt)
 	  {
-	    printf ("Could not get ticket for `%s'.\n",
-		    arg.tgtname ? arg.tgtname : arg.cname);
+	    printf ("Could not get ticket for `%s'.\n", hint.server);
 	    rc = !SHISHI_OK;
 	  }
 	else
@@ -1119,7 +1141,8 @@ main (int argc, char *argv[])
 	    rc = shishi_tkt_pretty_print (tkt, stdout);
 	    if (rc != SHISHI_OK)
 	      fprintf (stderr, "Pretty printing ticket failed:\n%s\n%s\n",
-		       shishi_strerror (rc), shishi_strerror_details (handle));
+		       shishi_strerror (rc),
+		       shishi_strerror_details (handle));
 	  }
       }
       break;
