@@ -147,13 +147,13 @@ struct arguments
 {
   int silent, verbose;
   char *cfgfile;
+  char *keyfile;
   char *setuid;
   struct listenspec *listenspec;
   int nlistenspec;
+  Shishi_key *tgskey;
 };
 
-struct arguments arg;
-
 const char *argp_program_version = PACKAGE_STRING;
 const char *argp_program_bug_address = PACKAGE_BUGREPORT;
 
@@ -180,6 +180,10 @@ parse_opt (int key, char *arg, struct argp_state *state)
       arguments->cfgfile = strdup (arg);
       break;
 
+    case 'k':
+      arguments->keyfile = strdup (arg);
+      break;
+
     case 'u':
       arguments->setuid = strdup (arg);
       break;
@@ -319,46 +323,79 @@ parse_opt (int key, char *arg, struct argp_state *state)
 static struct argp_option options[] = {
 
   {"verbose", 'v', 0, 0,
-   "Produce verbose output."},
+   "Produce verbose output.", 0},
 
   {"quiet", 'q', 0, 0,
-   "Don't produce any output."},
+   "Don't produce any output.", 0},
 
-  {"silent", 's', 0, OPTION_ALIAS},
+  {"silent", 's', 0, OPTION_ALIAS,
+   NULL, 0},
 
   {"configuration-file", 'c', "FILE", 0,
-   "Read configuration from file.  Default is " SYSTEMCFGFILE "."},
+   "Read configuration from file.  Default is " SYSTEMCFGFILE ".", 0},
 
   {"listen", 'l', "[FAMILY:]ADDRESS:SERVICE/TYPE,...", 0,
    "What to listen on. Family is \"IPv4\" or \"IPv6\", if absent the "
    "family is decided by gethostbyname(ADDRESS). An address of \"*\" "
    "indicates all addresses on the local host. "
-   "The default is \"" LISTEN_DEFAULT "\"."},
+   "The default is \"" LISTEN_DEFAULT "\".", 0},
+
+  {"key-file", 'k', "FILE", 0,
+   "Read keys from file.  Default is " KDCKEYFILE ".", 0},
 
   {"setuid", 'u', "NAME", 0,
-   "After binding socket, set user identity."},
+   "After binding socket, set user identity.", 0},
 
-  {0}
+  {NULL, 0, NULL, 0,
+   NULL, 0}
 };
 
 static struct argp argp = {
   options,
   parse_opt,
   NULL,
-  "Shishid -- Key Distribution Center network daemon"
+  "Shishid -- Key Distribution Center network daemon",
+  NULL,
+  NULL,
+  NULL
 };
 
+static char *fatal_krberror;
+static size_t fatal_krberror_len;
+
 static int
-asreq1 (Shishi * handle, Shishi_as * as)
+setup_fatal_krberror (Shishi * handle)
+{
+  Shishi_asn1 krberr;
+  int rc;
+
+  krberr = shishi_krberror (handle);
+  if (!krberr)
+    return SHISHI_MALLOC_ERROR;
+
+  rc = shishi_krberror_set_etext (handle, krberr,
+				  "Internal KDC error, contact administrator");
+  if (rc != SHISHI_OK)
+    return rc;
+
+  rc = shishi_krberror_der (handle, krberr, &fatal_krberror,
+			    &fatal_krberror_len);
+  if (rc != SHISHI_OK)
+    return rc;
+
+  return SHISHI_OK;
+}
+
+static int
+asreq1 (Shishi * handle, struct arguments *arg, Shishi_as * as)
 {
   Shishi_tkt *tkt;
-  Shishi_key *sessionkey, *sessiontktkey, *userkey;
+  Shishi_key *sessionkey, *userkey;
   int etype, i;
   char buf[BUFSIZ];
   int buflen;
   int err;
-  char *username, *servername, *serverrealm;
-  char *password;
+  char *username, *servername, *realm;
 
   tkt = shishi_as_tkt (as);
   if (!tkt)
@@ -376,10 +413,8 @@ asreq1 (Shishi * handle, Shishi_as * as)
     return err;
 
   /* XXX use a "preferred server kdc etype" from shishi instead? */
+
   err = shishi_key_random (handle, etype, &sessionkey);
-  if (err)
-    return err;
-  err = shishi_key_random (handle, etype, &sessiontktkey);
   if (err)
     return err;
 
@@ -408,34 +443,24 @@ asreq1 (Shishi * handle, Shishi_as * as)
   if (err != SHISHI_OK)
     return err;
   buf[buflen] = '\0';
-  serverrealm = strdup (buf);
-  printf ("serverrealm %s\n", serverrealm);
+  realm = strdup (buf);
+  printf ("client & server realm %s\n", realm);
 
-  password = "foo";
-
-  err = shishi_tkt_clientrealm_set (tkt, serverrealm, username);
+  err = shishi_tkt_clientrealm_set (tkt, realm, username);
   if (err)
     return err;
 
-  err = shishi_tkt_serverrealm_set (tkt, serverrealm, servername);
+  err = shishi_tkt_serverrealm_set (tkt, realm, servername);
   if (err)
     return err;
 
-  buflen = sizeof (buf);
-  err = shishi_as_derive_salt (handle, shishi_as_req (as), shishi_as_rep (as),
-			       buf, &buflen);
-  if (err != SHISHI_OK)
-    return err;
+  userkey = shishi_keys_for_serverrealm_in_file (handle,
+						 arg->keyfile,
+						 username, realm);
+  if (!userkey)
+    return !SHISHI_OK;
 
-  err = shishi_key_from_string (handle,
-				etype,
-				password,
-				strlen (password),
-				buf, buflen, NULL, &userkey);
-  if (err != SHISHI_OK)
-    return err;
-
-  err = shishi_tkt_build (tkt, sessiontktkey);
+  err = shishi_tkt_build (tkt, arg->tgskey);
   if (err)
     return err;
 
@@ -443,7 +468,7 @@ asreq1 (Shishi * handle, Shishi_as * as)
   if (err)
     return err;
 
-  if (arg.verbose)
+  if (arg->verbose)
     {
       shishi_kdcreq_print (handle, stderr, shishi_as_req (as));
       shishi_encticketpart_print (handle, stderr,
@@ -458,7 +483,8 @@ asreq1 (Shishi * handle, Shishi_as * as)
 }
 
 static void
-asreq (Shishi * handle, Shishi_asn1 kdcreq, char **out, int *outlen)
+asreq (Shishi * handle, struct arguments *arg,
+       Shishi_asn1 kdcreq, char **out, int *outlen)
 {
   Shishi_as *as;
   int rc;
@@ -476,27 +502,209 @@ asreq (Shishi * handle, Shishi_asn1 kdcreq, char **out, int *outlen)
 
   shishi_as_req_set (as, kdcreq);
 
-  *out = malloc (BUFSIZ);
-  if (*out == NULL)
-    {
-      syslog (LOG_ERR, "Incoming request failed: Cannot allocate memory\n");
-      /* XXX hard coded KRB-ERROR? */
-      *out = NULL;
-      *outlen = 0;
-      return;
-    }
-  *outlen = BUFSIZ;
-
-  rc = asreq1 (handle, as);
+  rc = asreq1 (handle, arg, as);
   if (rc != SHISHI_OK)
     {
       syslog (LOG_NOTICE, "Could not answer request: %s: %s\n",
 	      shishi_strerror (rc),
 	      shishi_krberror_message (handle, shishi_as_krberror (as)));
-      rc = shishi_as_krberror_der (as, *out, outlen);
+      rc = shishi_as_krberror_der (as, out, outlen);
     }
   else
-    rc = shishi_as_rep_der (as, *out, outlen);
+    rc = shishi_as_rep_der (as, out, outlen);
+  if (rc != SHISHI_OK)
+    {
+      syslog (LOG_ERR,
+	      "Incoming request failed: Cannot DER encode reply: %s\n",
+	      shishi_strerror (rc));
+      /* XXX hard coded KRB-ERROR? */
+      *out = strdup ("aaaaaa");
+      *outlen = strlen (*out);
+      return;
+    }
+
+  return;
+}
+static int
+tgsreq1 (Shishi * handle, struct arguments *arg, Shishi_tgs * tgs)
+{
+  int rc;
+  Shishi_tkt *tkt;
+  Shishi_key *sessionkey, *sessiontktkey, *serverkey, *subkey, *keytouse;
+  char buf[BUFSIZ];
+  int buflen;
+  int err;
+  char *username, *servername, *realm;
+  int32_t etype;
+  int i;
+
+  tkt = shishi_tgs_tkt (tgs);
+  if (!tkt)
+    return SHISHI_MALLOC_ERROR;
+
+  i = 1;
+  do
+    {
+      err = shishi_kdcreq_etype (handle, shishi_tgs_req (tgs), &etype, i);
+      if (err == SHISHI_OK && shishi_cipher_supported_p (etype))
+	break;
+    }
+  while (err == SHISHI_OK);
+  if (err != SHISHI_OK)
+    return err;
+
+  /* XXX use a "preferred server kdc etype" from shishi instead? */
+
+  err = shishi_key_random (handle, etype, &sessionkey);
+  if (err)
+    return err;
+
+  err = shishi_tkt_key_set (tkt, sessionkey);
+  if (err)
+    return err;
+
+  /* extract pa-data and populate tgs->ap */
+  rc = shishi_tgs_req_process (tgs);
+  if (rc != SHISHI_OK)
+    return rc;
+
+  /* XXX check if ticket is for our tgt key */
+
+  /* decrypt ticket with our key, and decrypt authenticator using key in tkt */
+  rc = shishi_ap_req_process_keyusage
+    (shishi_tgs_ap (tgs), arg->tgskey,
+     SHISHI_KEYUSAGE_TGSREQ_APREQ_AUTHENTICATOR);
+  if (rc != SHISHI_OK)
+    return rc;
+
+  /* XXX check that checksum in authenticator match tgsreq.req-body */
+
+  buflen = sizeof (buf) - 1;
+  err = shishi_encticketpart_cname_get
+    (handle, shishi_tkt_encticketpart (shishi_ap_tkt (shishi_tgs_ap (tgs))),
+     buf, &buflen);
+  if (err != SHISHI_OK)
+    return err;
+  buf[buflen] = '\0';
+  username = strdup (buf);
+  printf ("username %s\n", username);
+
+  buflen = sizeof (buf) - 1;
+  err = shishi_kdcreq_sname_get (handle, shishi_tgs_req (tgs), buf, &buflen);
+  if (err != SHISHI_OK)
+    return err;
+  buf[buflen] = '\0';
+  servername = strdup (buf);
+  printf ("servername %s\n", servername);
+
+  buflen = sizeof (buf) - 1;
+  err = shishi_kdcreq_realm_get (handle, shishi_tgs_req (tgs), buf, &buflen);
+  if (err != SHISHI_OK)
+    return err;
+  buf[buflen] = '\0';
+  realm = strdup (buf);
+  printf ("server realm %s\n", realm);
+
+  err = shishi_tkt_clientrealm_set (tkt, realm, username);
+  if (err)
+    return err;
+
+  err = shishi_tkt_serverrealm_set (tkt, realm, servername);
+  if (err)
+    return err;
+
+  serverkey = shishi_keys_for_serverrealm_in_file (handle,
+						   arg->keyfile,
+						   servername, realm);
+  if (!serverkey)
+    return !SHISHI_OK;
+
+  err = shishi_tkt_build (tkt, serverkey);
+  if (err)
+    return err;
+
+  err = shishi_encticketpart_get_key
+    (handle,
+     shishi_tkt_encticketpart (shishi_ap_tkt (shishi_tgs_ap (tgs))),
+     &sessiontktkey);
+
+  err = shishi_authenticator_get_subkey
+    (handle, shishi_ap_authenticator (shishi_tgs_ap (tgs)), &subkey);
+  if (err != SHISHI_OK && err != SHISHI_ASN1_NO_ELEMENT)
+    return err;
+
+  if (err == SHISHI_OK)
+    keytouse = subkey;
+  else
+    keytouse = sessiontktkey;
+
+  err = shishi_tgs_rep_build (tgs, keytouse);
+  if (err)
+    return err;
+
+  if (arg->verbose)
+    {
+      puts ("KDC-REQ in:");
+      shishi_kdcreq_print (handle, stderr, shishi_tgs_req (tgs));
+      puts ("AP-REQ in KDC-REQ:");
+      shishi_apreq_print (handle, stderr,
+			  shishi_ap_req (shishi_tgs_ap (tgs)));
+      puts ("Authenticator in AP-REQ in KDC-REQ:");
+      shishi_authenticator_print (handle, stderr, shishi_ap_authenticator
+				  (shishi_tgs_ap (tgs)));
+      puts ("Ticket in AP-REQ:");
+      shishi_ticket_print (handle, stdout,
+			   shishi_tkt_ticket
+			   (shishi_ap_tkt (shishi_tgs_ap (tgs))));
+      puts ("EncTicketPart in AP-REQ:");
+      shishi_encticketpart_print (handle, stdout,
+				  shishi_tkt_encticketpart
+				  (shishi_ap_tkt (shishi_tgs_ap (tgs))));
+      puts ("Ticket in TGS-REP:");
+      shishi_ticket_print (handle, stdout, shishi_tkt_ticket (tkt));
+      puts ("EncTicketPart in TGS-REP:");
+      shishi_encticketpart_print (handle, stderr,
+				  shishi_tkt_encticketpart (tkt));
+      puts ("EncKDCRepPart in TGS-REP:");
+      shishi_enckdcreppart_print (handle, stderr,
+				  shishi_tkt_enckdcreppart (tkt));
+      puts ("KDC-REP:");
+      shishi_kdcrep_print (handle, stderr, shishi_tgs_rep (tgs));
+    }
+
+  return SHISHI_OK;
+}
+
+static void
+tgsreq (Shishi * handle, struct arguments *arg,
+	Shishi_asn1 kdcreq, char **out, int *outlen)
+{
+  Shishi_tgs *tgs;
+  int rc;
+
+  rc = shishi_tgs (handle, &tgs);
+  if (rc != SHISHI_OK)
+    {
+      syslog (LOG_ERR, "Incoming request failed: Cannot create TGS: %s\n",
+	      shishi_strerror (rc));
+      /* XXX hard coded KRB-ERROR? */
+      *out = strdup ("foo");
+      *outlen = strlen (*out);
+      return;
+    }
+
+  shishi_tgs_req_set (tgs, kdcreq);
+
+  rc = tgsreq1 (handle, arg, tgs);
+  if (rc != SHISHI_OK)
+    {
+      syslog (LOG_NOTICE, "Could not answer request: %s: %s\n",
+	      shishi_strerror (rc),
+	      shishi_krberror_message (handle, shishi_tgs_krberror (tgs)));
+      rc = shishi_tgs_krberror_der (tgs, out, outlen);
+    }
+  else
+    rc = shishi_tgs_rep_der (tgs, out, outlen);
   if (rc != SHISHI_OK)
     {
       syslog (LOG_ERR,
@@ -514,17 +722,24 @@ asreq (Shishi * handle, Shishi_asn1 kdcreq, char **out, int *outlen)
 static Shishi_msgtype
 get_msgtype (Shishi * handle, char *in, size_t inlen)
 {
-  if (inlen > 1)
-    return *in - 0x5a;
+  if (inlen > 1 && *in >= 0x60 && (unsigned char) *in <= 0x7F)
+    return *in - 0x60;
   else
     return 0;
 }
 
-static void
-process (Shishi * handle, char *in, int inlen, char **out, int *outlen)
+static int
+process_1 (Shishi * handle, struct arguments *arg,
+	   char *in, size_t inlen, char **out, size_t * outlen)
 {
   Shishi_asn1 kdcreq;
   Shishi_msgtype msgtype;
+  Shishi_asn1 krberr;
+  int rc;
+
+  krberr = shishi_krberror (handle);
+  if (!krberr)
+    return SHISHI_MALLOC_ERROR;
 
   fprintf (stderr, "Processing %d bytes...\n", inlen);
 
@@ -538,12 +753,15 @@ process (Shishi * handle, char *in, int inlen, char **out, int *outlen)
       kdcreq = shishi_der2asn1_asreq (handle, in, inlen);
       if (kdcreq)
 	{
-	  shishi_kdcreq_print (handle, stdout, kdcreq);
-	  asreq (handle, kdcreq, out, outlen);
+	  asreq (handle, arg, kdcreq, out, outlen);
+	  return SHISHI_OK;
 	}
       else
 	{
-	  fprintf (stderr, "bad as-req...\n");
+	  rc = shishi_krberror_set_etext (handle, krberr,
+					  "Cannot parse AS-REQ");
+	  if (rc != SHISHI_OK)
+	    return rc;
 	}
       break;
 
@@ -551,56 +769,78 @@ process (Shishi * handle, char *in, int inlen, char **out, int *outlen)
       kdcreq = shishi_der2asn1_tgsreq (handle, in, inlen);
       if (kdcreq)
 	{
-	  fprintf (stderr, "tgs-req...\n");
-	  shishi_kdcreq_print (handle, stdout, kdcreq);
+	  tgsreq (handle, arg, kdcreq, out, outlen);
+	  return SHISHI_OK;
 	}
       else
 	{
-	  fprintf (stderr, "bad tgs-req...\n");
+	  rc = shishi_krberror_set_etext (handle, krberr,
+					  "Cannot parse TGS-REQ");
+	  if (rc != SHISHI_OK)
+	    return rc;
 	}
       break;
 
     default:
-      /* XXX hard coded KRB-ERROR? */
-      *out = NULL;
-      *outlen = 0;
+      rc = shishi_krberror_set_etext (handle, krberr,
+				      "Unsupported message type");
+      if (rc != SHISHI_OK)
+	return rc;
+      break;
+    }
+
+  rc = shishi_krberror_der (handle, krberr, out, outlen);
+  if (rc != SHISHI_OK)
+    return rc;
+
+  return SHISHI_OK;
+}
+
+static void
+process (Shishi * handle, struct arguments *arg,
+	 char *in, int inlen, char **out, size_t * outlen)
+{
+  int rc;
+
+  *out = NULL;
+  *outlen = 0;
+
+  rc = process_1 (handle, arg, in, inlen, out, outlen);
+
+  if (rc != SHISHI_OK || *out == NULL || *outlen == 0)
+    {
+      *out = fatal_krberror;
+      *outlen = fatal_krberror_len;
     }
 }
 
 int quit = 0;
 
-void ctrlc (int signum);
-
-void
+static void
 ctrlc (int signum)
 {
   quit = 1;
 }
 
 static int
-doit (Shishi * handle, struct arguments arg)
+kdc_listen (struct arguments *arg)
 {
   struct listenspec *ls;
-  fd_set readfds;
-  struct sockaddr addr;
-  socklen_t length = sizeof (addr);
   int maxfd = 0;
-  int rc;
   int i;
-  int sent_bytes, read_bytes;
   int yes;
 
-  for (i = 0; i < arg.nlistenspec; i++)
+  for (i = 0; i < arg->nlistenspec; i++)
     {
-      ls = &arg.listenspec[i];
+      ls = &arg->listenspec[i];
 
-      if (!arg.silent)
+      if (!arg->silent)
 	printf ("Listening on %s...", ls->str);
 
       ls->sockfd = socket (ls->family, ls->type, 0);
       if (ls->sockfd < 0)
 	{
-	  if (!arg.silent)
+	  if (!arg->silent)
 	    printf ("failed\n");
 	  perror ("socket");
 	  ls->sockfd = 0;
@@ -611,7 +851,7 @@ doit (Shishi * handle, struct arguments arg)
       if (setsockopt (ls->sockfd, SOL_SOCKET, SO_REUSEADDR,
 		      (char *) &yes, sizeof (yes)) < 0)
 	{
-	  if (!arg.silent)
+	  if (!arg->silent)
 	    printf ("failed\n");
 	  perror ("setsockopt");
 	  close (ls->sockfd);
@@ -621,7 +861,7 @@ doit (Shishi * handle, struct arguments arg)
 
       if (bind (ls->sockfd, &ls->addr, sizeof (ls->addr)) != 0)
 	{
-	  if (!arg.silent)
+	  if (!arg->silent)
 	    printf ("failed\n");
 	  perror ("bind");
 	  close (ls->sockfd);
@@ -631,7 +871,7 @@ doit (Shishi * handle, struct arguments arg)
 
       if (ls->type == SOCK_STREAM && listen (ls->sockfd, 512) != 0)
 	{
-	  if (!arg.silent)
+	  if (!arg->silent)
 	    printf ("failed\n");
 	  perror ("listen");
 	  close (ls->sockfd);
@@ -640,7 +880,7 @@ doit (Shishi * handle, struct arguments arg)
 	}
 
       maxfd++;
-      if (!arg.silent)
+      if (!arg->silent)
 	printf ("done\n");
     }
 
@@ -650,34 +890,23 @@ doit (Shishi * handle, struct arguments arg)
       return 1;
     }
 
-  if (!arg.silent)
+  if (!arg->silent)
     printf ("Listening on %d ports...\n", maxfd);
 
-  if (arg.setuid)
-    {
-      struct passwd *passwd;
+  return 0;
+}
 
-      passwd = getpwnam (arg.setuid);
-      if (passwd == NULL)
-	{
-	  perror ("setuid: getpwnam");
-	  return 1;
-	}
-
-      rc = setuid (passwd->pw_uid);
-      if (rc == -1)
-	{
-	  perror ("setuid");
-	  return 1;
-	}
-
-      if (!arg.silent)
-	printf ("User identity set to `%s' (%d)...\n",
-		passwd->pw_name, passwd->pw_uid);
-    }
-
-  signal (SIGINT, ctrlc);
-  signal (SIGTERM, ctrlc);
+static int
+kdc_loop (Shishi * handle, struct arguments *arg)
+{
+  struct listenspec *ls;
+  fd_set readfds;
+  struct sockaddr addr;
+  socklen_t length = sizeof (addr);
+  int maxfd = 0;
+  int rc;
+  int i;
+  ssize_t sent_bytes, read_bytes;
 
   while (!quit)
     {
@@ -686,11 +915,11 @@ doit (Shishi * handle, struct arguments arg)
 	{
 	  FD_ZERO (&readfds);
 	  maxfd = 0;
-	  for (i = 0; i < arg.nlistenspec; i++)
+	  for (i = 0; i < arg->nlistenspec; i++)
 	    {
-	      if (arg.listenspec[i].sockfd >= maxfd)
-		maxfd = arg.listenspec[i].sockfd + 1;
-	      FD_SET (arg.listenspec[i].sockfd, &readfds);
+	      if (arg->listenspec[i].sockfd >= maxfd)
+		maxfd = arg->listenspec[i].sockfd + 1;
+	      FD_SET (arg->listenspec[i].sockfd, &readfds);
 	    }
 	}
       while ((rc = select (maxfd, &readfds, NULL, NULL, NULL)) == 0);
@@ -702,51 +931,51 @@ doit (Shishi * handle, struct arguments arg)
 	  continue;
 	}
 
-      for (i = 0; i < arg.nlistenspec; i++)
-	if (FD_ISSET (arg.listenspec[i].sockfd, &readfds))
+      for (i = 0; i < arg->nlistenspec; i++)
+	if (FD_ISSET (arg->listenspec[i].sockfd, &readfds))
 	  {
-	    if (arg.listenspec[i].type == SOCK_STREAM &&
-		arg.listenspec[i].family != -1)
+	    if (arg->listenspec[i].type == SOCK_STREAM &&
+		arg->listenspec[i].family != -1)
 	      {
 		fprintf (stderr, "New connection on %s...",
-			 arg.listenspec[i].str);
+			 arg->listenspec[i].str);
 
 		/* XXX search for closed fd's before allocating new entry */
-		arg.listenspec = realloc (arg.listenspec,
-					  sizeof (*arg.listenspec) *
-					  (arg.nlistenspec + 1));
-		if (arg.listenspec != NULL)
+		arg->listenspec = realloc (arg->listenspec,
+					   sizeof (*arg->listenspec) *
+					   (arg->nlistenspec + 1));
+		if (arg->listenspec != NULL)
 		  {
 		    struct sockaddr_in *sin;
 		    char *str;
 
-		    arg.nlistenspec++;
-		    ls = &arg.listenspec[arg.nlistenspec - 1];
+		    arg->nlistenspec++;
+		    ls = &arg->listenspec[arg->nlistenspec - 1];
 		    ls->bufpos = 0;
-		    ls->type = arg.listenspec[i].type;
+		    ls->type = arg->listenspec[i].type;
 		    ls->family = -1;
 		    length = sizeof (ls->addr);
-		    ls->sockfd = accept (arg.listenspec[i].sockfd,
+		    ls->sockfd = accept (arg->listenspec[i].sockfd,
 					 &ls->addr, &length);
 		    sin = (struct sockaddr_in *) &ls->addr;
 		    str = inet_ntoa (sin->sin_addr);
-		    ls->str = malloc (strlen (arg.listenspec[i].str) +
+		    ls->str = malloc (strlen (arg->listenspec[i].str) +
 				      strlen (" peer ") + strlen (str) + 1);
-		    sprintf (ls->str, "%s peer %s", arg.listenspec[i].str,
+		    sprintf (ls->str, "%s peer %s", arg->listenspec[i].str,
 			     str);
 		    puts (ls->str);
 		  }
 	      }
 	    else
 	      {
-		ls = &arg.listenspec[i];
+		ls = &arg->listenspec[i];
 
 		read_bytes = recvfrom (ls->sockfd, ls->buf + ls->bufpos,
 				       BUFSIZ - ls->bufpos, 0, &addr,
 				       &length);
 
-		if (arg.listenspec[i].type == SOCK_STREAM &&
-		    arg.listenspec[i].family == -1 && read_bytes == 0)
+		if (arg->listenspec[i].type == SOCK_STREAM &&
+		    arg->listenspec[i].family == -1 && read_bytes == 0)
 		  {
 		    printf ("Peer %s disconnected\n", ls->str);
 		    close (ls->sockfd);
@@ -760,14 +989,18 @@ doit (Shishi * handle, struct arguments arg)
 
 		printf ("Has %d bytes from %s\n", ls->bufpos, ls->str);
 
-		if (arg.listenspec[i].type == SOCK_DGRAM ||
-		    (ls->bufpos > 4
-		     && ntohl (*(int *) ls->buf) == ls->bufpos))
+		if (arg->listenspec[i].type == SOCK_DGRAM ||
+		    (ls->bufpos > 4 &&
+		     ntohl (*(int *) ls->buf) + 4 == ls->bufpos))
 		  {
 		    char *p;
-		    int plen;
+		    size_t plen;
 
-		    process (handle, ls->buf, ls->bufpos, &p, &plen);
+		    if (arg->listenspec[i].type == SOCK_STREAM)
+		      process (handle, arg, ls->buf + 4, ls->bufpos - 4,
+			       &p, &plen);
+		    else
+		      process (handle, arg, ls->buf, ls->bufpos, &p, &plen);
 
 		    if (p && plen > 0)
 		      {
@@ -776,17 +1009,18 @@ doit (Shishi * handle, struct arguments arg)
 					       0, &addr, length);
 			while (sent_bytes == -1 && errno == EAGAIN);
 
-			if (sent_bytes == -1)
+			if (sent_bytes < 0)
 			  perror ("write");
-			else if (sent_bytes > plen)
+			else if ((size_t) sent_bytes > plen)
 			  fprintf (stderr, "wrote %db but buffer only %db",
 				   sent_bytes, plen);
-			else if (sent_bytes < plen)
+			else if ((size_t) sent_bytes < plen)
 			  fprintf (stderr,
 				   "short write (%db) writing %d bytes\n",
 				   sent_bytes, plen);
 
-			free (p);
+			if (p != fatal_krberror)
+			  free (p);
 		      }
 
 		    ls->bufpos = 0;
@@ -795,49 +1029,166 @@ doit (Shishi * handle, struct arguments arg)
 	  }
     }
 
-  for (i = 0; i < arg.nlistenspec; i++)
-    if (arg.listenspec[i].sockfd)
-      {
-	if (!arg.silent)
-	  printf ("Closing %s...", arg.listenspec[i].str);
-	rc = close (arg.listenspec[i].sockfd);
-	if (rc != 0)
-	  {
-	    if (!arg.silent)
-	      printf ("failed\n");
-	    perror ("close");
-	  }
-	else if (!arg.silent)
-	  printf ("done\n");
-      }
+  return 0;
+}
+
+static int
+kdc_setuid (struct arguments *arg)
+{
+  struct passwd *passwd;
+  int rc;
+
+  if (!arg->setuid)
+    return 0;
+
+  passwd = getpwnam (arg->setuid);
+  if (passwd == NULL)
+    {
+      perror ("setuid: getpwnam");
+      return 1;
+    }
+
+  rc = setuid (passwd->pw_uid);
+  if (rc == -1)
+    {
+      perror ("setuid");
+      return 1;
+    }
+
+  if (!arg->silent)
+    printf ("User identity set to `%s' (%d)...\n",
+	    passwd->pw_name, passwd->pw_uid);
 
   return 0;
 }
 
-int
-main (int argc, char *argv[])
+static void
+kdc_unlisten (struct arguments *arg)
+{
+  int i;
+  int rc;
+
+  for (i = 0; i < arg->nlistenspec; i++)
+    if (arg->listenspec[i].sockfd)
+      {
+	if (!arg->silent)
+	  printf ("Closing %s...", arg->listenspec[i].str);
+	rc = close (arg->listenspec[i].sockfd);
+	if (rc != 0)
+	  {
+	    if (!arg->silent)
+	      printf ("failed\n");
+	    perror ("close");
+	  }
+	else if (!arg->silent)
+	  printf ("done\n");
+      }
+}
+
+static int
+launch (Shishi * handle, struct arguments *arg)
+{
+  int rc;
+
+  rc = kdc_listen (arg);
+  if (rc != 0)
+    return rc;
+
+  rc = kdc_setuid (arg);
+  if (rc != 0)
+    return rc;
+
+  signal (SIGINT, ctrlc);
+  signal (SIGTERM, ctrlc);
+
+  rc = kdc_loop (handle, arg);
+  if (rc != 0)
+    return rc;
+
+  kdc_unlisten (arg);
+
+  return 0;
+}
+
+static int
+setup (Shishi * handle, struct arguments *arg)
+{
+  char *tgtname;
+  int rc;
+
+  rc = setup_fatal_krberror (handle);
+  if (rc != SHISHI_OK)
+    {
+      syslog (LOG_ERR, "Cannot allocate fatal error message\n");
+      return 1;
+    }
+
+  asprintf (&tgtname, "krbtgt/%s", shishi_realm_default (handle));
+  arg->tgskey = shishi_keys_for_serverrealm_in_file
+    (handle, arg->keyfile, tgtname, shishi_realm_default (handle));
+  free (tgtname);
+  if (!arg->tgskey)
+    {
+      syslog (LOG_ERR, "Key for krbtgt/%s not found in %s\n",
+	      shishi_realm_default (handle), arg->keyfile);
+      return 1;
+    }
+
+  rc = launch (handle, arg);
+
+  shishi_key_done (&arg->tgskey);
+
+  return rc;
+}
+
+static int
+init (struct arguments *arg)
 {
   Shishi *handle;
   int rc;
 
-#ifdef LOG_PERROR
-  openlog (PACKAGE, LOG_CONS|LOG_PERROR, LOG_DAEMON);
-#else
-  openlog (PACKAGE, LOG_CONS, LOG_DAEMON);
-#endif
-  memset ((void *) &arg, 0, sizeof (arg));
-  argp_parse (&argp, argc, argv, ARGP_IN_ORDER, 0, &arg);
-
-  rc = shishi_init_server_with_paths (&handle, arg.cfgfile);
+  rc = shishi_init_server_with_paths (&handle, arg->cfgfile);
   if (rc != SHISHI_OK)
     {
       syslog (LOG_ERR, "Aborting due to library initialization failure\n");
       return 1;
     }
 
-  rc = doit (handle, arg);
+  rc = setup (handle, arg);
 
   shishi_done (handle);
+
+  return rc;
+}
+
+int
+main (int argc, char *argv[])
+{
+  struct arguments arg;
+  int rc;
+
+#ifdef LOG_PERROR
+  openlog (PACKAGE, LOG_CONS | LOG_PERROR, LOG_DAEMON);
+#else
+  openlog (PACKAGE, LOG_CONS, LOG_DAEMON);
+#endif
+
+  memset ((void *) &arg, 0, sizeof (arg));
+  argp_parse (&argp, argc, argv, ARGP_IN_ORDER, 0, &arg);
+
+  if (!arg.keyfile)
+    arg.keyfile = strdup (KDCKEYFILE);
+
+  if (!arg.cfgfile)
+    arg.cfgfile = strdup (SYSTEMCFGFILE);
+
+  rc = init (&arg);
+
+  free (arg.keyfile);
+  free (arg.cfgfile);
+  if (arg.setuid)
+    free (arg.setuid);
+
   closelog ();
 
   return rc;
